@@ -8,36 +8,89 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: EventHandlerSDL2.cxx 3302 2016-04-02 23:47:46Z stephena $
 //============================================================================
 
+#include "Logger.hxx"
 #include "OSystem.hxx"
 #include "EventHandlerSDL2.hxx"
 
+#include "ThreadDebugging.hxx"
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 EventHandlerSDL2::EventHandlerSDL2(OSystem& osystem)
-  : EventHandler(osystem)
+  : EventHandler{osystem}
 {
+  ASSERT_MAIN_THREAD;
+
+#ifdef GUI_SUPPORT
+  {
+    ostringstream buf;
+    myQwertz = int{'y'} == static_cast<int>(SDL_GetKeyFromScancode(SDL_Scancode(KBDK_Z)));
+    buf << "Keyboard: " << (myQwertz ? "QWERTZ" : "QWERTY");
+    Logger::debug(buf.str());
+  }
+#endif
+
+#ifdef JOYSTICK_SUPPORT
+  if(SDL_InitSubSystem(SDL_INIT_JOYSTICK) < 0)
+  {
+    ostringstream buf;
+    buf << "ERROR: Couldn't initialize SDL joystick support: " << SDL_GetError() << endl;
+    Logger::error(buf.str());
+  }
+  Logger::debug("EventHandlerSDL2::EventHandlerSDL2 SDL_INIT_JOYSTICK");
+#endif
+
+  SDL_SetHint(SDL_HINT_MOUSE_FOCUS_CLICKTHROUGH, "1");
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+EventHandlerSDL2::~EventHandlerSDL2()
+{
+  ASSERT_MAIN_THREAD;
+
+  if(SDL_WasInit(SDL_INIT_JOYSTICK))
+    SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EventHandlerSDL2::enableTextEvents(bool enable)
 {
+  ASSERT_MAIN_THREAD;
+
   if(enable)
     SDL_StartTextInput();
   else
     SDL_StopTextInput();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void EventHandlerSDL2::copyText(const string& text) const
+{
+  SDL_SetClipboardText(text.c_str());
+};
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string EventHandlerSDL2::pasteText(string& text) const
+{
+  if(SDL_HasClipboardText())
+    text = SDL_GetClipboardText();
+  else
+    text = "";
+
+  return text;
+};
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EventHandlerSDL2::pollEvent()
 {
+  ASSERT_MAIN_THREAD;
+
   while(SDL_PollEvent(&myEvent))
   {
     switch(myEvent.type)
@@ -46,10 +99,10 @@ void EventHandlerSDL2::pollEvent()
       case SDL_KEYUP:
       case SDL_KEYDOWN:
       {
-        if(!myEvent.key.repeat)
-          handleKeyEvent(StellaKey(myEvent.key.keysym.scancode),
-                         StellaMod(myEvent.key.keysym.mod),
-                         myEvent.key.type == SDL_KEYDOWN);
+        handleKeyEvent(static_cast<StellaKey>(myEvent.key.keysym.scancode),
+                       static_cast<StellaMod>(myEvent.key.keysym.mod),
+                       myEvent.key.type == SDL_KEYDOWN,
+                       myEvent.key.repeat);
         break;
       }
 
@@ -62,24 +115,26 @@ void EventHandlerSDL2::pollEvent()
       case SDL_MOUSEMOTION:
       {
         handleMouseMotionEvent(myEvent.motion.x, myEvent.motion.y,
-                               myEvent.motion.xrel, myEvent.motion.yrel, 0);
+                               myEvent.motion.xrel, myEvent.motion.yrel);
         break;
       }
 
       case SDL_MOUSEBUTTONDOWN:
       case SDL_MOUSEBUTTONUP:
       {
-        bool pressed = myEvent.button.type == SDL_MOUSEBUTTONDOWN;
+        // ToDo: check support of more buttons and double-click
         switch(myEvent.button.button)
         {
           case SDL_BUTTON_LEFT:
-            handleMouseButtonEvent(pressed ? EVENT_LBUTTONDOWN : EVENT_LBUTTONUP,
+            handleMouseButtonEvent(MouseButton::LEFT, myEvent.button.type == SDL_MOUSEBUTTONDOWN,
                                    myEvent.button.x, myEvent.button.y);
             break;
           case SDL_BUTTON_RIGHT:
-            handleMouseButtonEvent(pressed ? EVENT_RBUTTONDOWN : EVENT_RBUTTONUP,
+            handleMouseButtonEvent(MouseButton::RIGHT, myEvent.button.type == SDL_MOUSEBUTTONDOWN,
                                    myEvent.button.x, myEvent.button.y);
             break;
+          default:
+            break;
         }
         break;
       }
@@ -89,9 +144,9 @@ void EventHandlerSDL2::pollEvent()
         int x, y;
         SDL_GetMouseState(&x, &y);  // we need mouse position too
         if(myEvent.wheel.y < 0)
-          handleMouseButtonEvent(EVENT_WHEELDOWN, x, y);
+          handleMouseButtonEvent(MouseButton::WHEELDOWN, true, x, y);
         else if(myEvent.wheel.y > 0)
-          handleMouseButtonEvent(EVENT_WHEELUP, x, y);
+          handleMouseButtonEvent(MouseButton::WHEELUP, true, x, y);
         break;
       }
 
@@ -99,8 +154,8 @@ void EventHandlerSDL2::pollEvent()
       case SDL_JOYBUTTONUP:
       case SDL_JOYBUTTONDOWN:
       {
-        handleJoyEvent(myEvent.jbutton.which, myEvent.jbutton.button,
-                       myEvent.jbutton.state == SDL_PRESSED ? 1 : 0);
+        handleJoyBtnEvent(myEvent.jbutton.which, myEvent.jbutton.button,
+                          myEvent.jbutton.state == SDL_PRESSED);
         break;
       }
 
@@ -113,12 +168,17 @@ void EventHandlerSDL2::pollEvent()
 
       case SDL_JOYHATMOTION:
       {
-        int v = myEvent.jhat.value, value = 0;
-        if(v & SDL_HAT_UP)        value |= EVENT_HATUP_M;
-        if(v & SDL_HAT_DOWN)      value |= EVENT_HATDOWN_M;
-        if(v & SDL_HAT_LEFT)      value |= EVENT_HATLEFT_M;
-        if(v & SDL_HAT_RIGHT)     value |= EVENT_HATRIGHT_M;
-        if(v == SDL_HAT_CENTERED) value  = EVENT_HATCENTER_M;
+        int value = 0;
+        const int v = myEvent.jhat.value;
+        if(v == SDL_HAT_CENTERED)
+          value  = EVENT_HATCENTER_M;
+        else
+        {
+          if(v & SDL_HAT_UP)    value |= EVENT_HATUP_M;
+          if(v & SDL_HAT_DOWN)  value |= EVENT_HATDOWN_M;
+          if(v & SDL_HAT_LEFT)  value |= EVENT_HATLEFT_M;
+          if(v & SDL_HAT_RIGHT) value |= EVENT_HATRIGHT_M;
+        }
 
         handleJoyHatEvent(myEvent.jhat.which, myEvent.jhat.hat, value);
         break;  // SDL_JOYHATMOTION
@@ -126,19 +186,19 @@ void EventHandlerSDL2::pollEvent()
 
       case SDL_JOYDEVICEADDED:
       {
-        addJoystick(new JoystickSDL2(myEvent.jdevice.which));
+        addPhysicalJoystick(make_shared<JoystickSDL2>(myEvent.jdevice.which));
         break;  // SDL_JOYDEVICEADDED
       }
       case SDL_JOYDEVICEREMOVED:
       {
-        removeJoystick(myEvent.jdevice.which);
+        removePhysicalJoystick(myEvent.jdevice.which);
         break;  // SDL_JOYDEVICEREMOVED
       }
   #endif
 
       case SDL_QUIT:
       {
-        handleEvent(Event::Quit, 1);
+        handleEvent(Event::Quit);
         break;  // SDL_QUIT
       }
 
@@ -146,53 +206,59 @@ void EventHandlerSDL2::pollEvent()
         switch(myEvent.window.event)
         {
           case SDL_WINDOWEVENT_SHOWN:
-            handleSystemEvent(EVENT_WINDOW_SHOWN);
+            handleSystemEvent(SystemEvent::WINDOW_SHOWN);
             break;
           case SDL_WINDOWEVENT_HIDDEN:
-            handleSystemEvent(EVENT_WINDOW_HIDDEN);
+            handleSystemEvent(SystemEvent::WINDOW_HIDDEN);
             break;
           case SDL_WINDOWEVENT_EXPOSED:
-            handleSystemEvent(EVENT_WINDOW_EXPOSED);
+            handleSystemEvent(SystemEvent::WINDOW_EXPOSED);
             break;
           case SDL_WINDOWEVENT_MOVED:
-            handleSystemEvent(EVENT_WINDOW_MOVED,
+            handleSystemEvent(SystemEvent::WINDOW_MOVED,
                               myEvent.window.data1, myEvent.window.data1);
             break;
           case SDL_WINDOWEVENT_RESIZED:
-            handleSystemEvent(EVENT_WINDOW_RESIZED,
+            handleSystemEvent(SystemEvent::WINDOW_RESIZED,
                               myEvent.window.data1, myEvent.window.data1);
             break;
           case SDL_WINDOWEVENT_MINIMIZED:
-            handleSystemEvent(EVENT_WINDOW_MINIMIZED);
+            handleSystemEvent(SystemEvent::WINDOW_MINIMIZED);
             break;
           case SDL_WINDOWEVENT_MAXIMIZED:
-            handleSystemEvent(EVENT_WINDOW_MAXIMIZED);
+            handleSystemEvent(SystemEvent::WINDOW_MAXIMIZED);
             break;
           case SDL_WINDOWEVENT_RESTORED:
-            handleSystemEvent(EVENT_WINDOW_RESTORED);
+            handleSystemEvent(SystemEvent::WINDOW_RESTORED);
             break;
           case SDL_WINDOWEVENT_ENTER:
-            handleSystemEvent(EVENT_WINDOW_ENTER);
+            handleSystemEvent(SystemEvent::WINDOW_ENTER);
             break;
           case SDL_WINDOWEVENT_LEAVE:
-            handleSystemEvent(EVENT_WINDOW_LEAVE);
+            handleSystemEvent(SystemEvent::WINDOW_LEAVE);
             break;
           case SDL_WINDOWEVENT_FOCUS_GAINED:
-            handleSystemEvent(EVENT_WINDOW_FOCUS_GAINED);
+            handleSystemEvent(SystemEvent::WINDOW_FOCUS_GAINED);
             break;
           case SDL_WINDOWEVENT_FOCUS_LOST:
-            handleSystemEvent(EVENT_WINDOW_FOCUS_LOST);
+            handleSystemEvent(SystemEvent::WINDOW_FOCUS_LOST);
+            break;
+          default:
             break;
         }
         break;  // SDL_WINDOWEVENT
+
+      default:
+        break;
     }
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 EventHandlerSDL2::JoystickSDL2::JoystickSDL2(int idx)
-  : myStick(nullptr)
 {
+  ASSERT_MAIN_THREAD;
+
   myStick = SDL_JoystickOpen(idx);
   if(myStick)
   {
@@ -214,6 +280,8 @@ EventHandlerSDL2::JoystickSDL2::JoystickSDL2(int idx)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 EventHandlerSDL2::JoystickSDL2::~JoystickSDL2()
 {
-  if(myStick)
+  ASSERT_MAIN_THREAD;
+
+  if(SDL_WasInit(SDL_INIT_JOYSTICK) && myStick)
     SDL_JoystickClose(myStick);
 }
