@@ -1,27 +1,25 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: CpuDebug.cxx 3292 2016-02-27 22:18:30Z stephena $
 //============================================================================
 
 #include <sstream>
 
 #include "M6502.hxx"
+#include "System.hxx"
 #include "Debugger.hxx"
-#include "CartDebug.hxx"
 #include "TIADebug.hxx"
 
 #include "CpuDebug.hxx"
@@ -29,7 +27,7 @@
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CpuDebug::CpuDebug(Debugger& dbg, Console& console)
   : DebuggerSystem(dbg, console),
-    my6502(mySystem.m6502())
+    my6502{mySystem.m6502()}
 {
 }
 
@@ -47,6 +45,7 @@ const DebuggerState& CpuDebug::getState()
   myState.srcA = my6502.lastSrcAddressA();
   myState.srcX = my6502.lastSrcAddressX();
   myState.srcY = my6502.lastSrcAddressY();
+  myState.dest = my6502.lastWriteAddress();
 
   Debugger::set_bits(myState.PS, myState.PSbits);
 
@@ -67,20 +66,99 @@ void CpuDebug::saveOldState()
   myOldState.srcA = my6502.lastSrcAddressA();
   myOldState.srcX = my6502.lastSrcAddressX();
   myOldState.srcY = my6502.lastSrcAddressY();
+  myOldState.dest = my6502.lastWriteAddress();
 
   Debugger::set_bits(myOldState.PS, myOldState.PSbits);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::pc() const
+{
+  return mySystem.m6502().PC;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::sp() const
+{
+  return mySystem.m6502().SP;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::a() const
+{
+  return mySystem.m6502().A;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::x() const
+{
+  return mySystem.m6502().X;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::y() const
+{
+  return mySystem.m6502().Y;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::n() const
+{
+  return mySystem.m6502().N;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::v() const
+{
+  return mySystem.m6502().V;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::b() const
+{
+  return mySystem.m6502().B;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::d() const
+{
+  return mySystem.m6502().D;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::i() const
+{
+  return mySystem.m6502().I;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::z() const
+{
+  return !mySystem.m6502().notZ;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::c() const
+{
+  return mySystem.m6502().C;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int CpuDebug::icycles() const
+{
+  return mySystem.m6502().icycles;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CpuDebug::setPC(int pc)
 {
-  my6502.PC = uInt16(pc);
+  my6502.PC = static_cast<uInt16>(pc);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CpuDebug::setSP(int sp)
 {
-  my6502.SP = uInt8(sp);
+  my6502.SP = static_cast<uInt8>(sp);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -92,19 +170,19 @@ void CpuDebug::setPS(int ps)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CpuDebug::setA(int a)
 {
-  my6502.A = uInt8(a);
+  my6502.A = static_cast<uInt8>(a);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CpuDebug::setX(int x)
 {
-  my6502.X = uInt8(x);
+  my6502.X = static_cast<uInt8>(x);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CpuDebug::setY(int y)
 {
-  my6502.Y = uInt8(y);
+  my6502.Y = static_cast<uInt8>(y);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -149,6 +227,12 @@ void CpuDebug::setC(bool on)
   my6502.C = on;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void CpuDebug::setCycles(int cycles)
+{
+  my6502.icycles = cycles;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CpuDebug::toggleN()
 {
