@@ -1,20 +1,18 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: Cart4A50Widget.cxx 3308 2016-05-24 16:55:45Z stephena $
 //============================================================================
 
 #include "Cart4A50.hxx"
@@ -26,7 +24,7 @@ Cartridge4A50Widget::Cartridge4A50Widget(
       GuiObject* boss, const GUI::Font& lfont, const GUI::Font& nfont,
       int x, int y, int w, int h, Cartridge4A50& cart)
   : CartDebugWidget(boss, lfont, nfont, x, y, w, h),
-    myCart(cart)
+    myCart{cart}
 {
   string info =
     "4A50 cartridge - 128K ROM and 32K RAM, split in various bank configurations\n"
@@ -36,7 +34,7 @@ Cartridge4A50Widget::Cartridge4A50Widget(
     "High bank region (256B)  : $FE00 - $FEFF\n"
     "Fixed (last 256B of ROM) : $FF00 - $FFFF\n";
 
-  int xpos = 10,
+  int xpos = 2,
       ypos = addBaseInformation(cart.mySize, "John Payson / Supercat", info) +
                myLineHeight;
 
@@ -62,65 +60,65 @@ Cartridge4A50Widget::Cartridge4A50Widget(
   string highlabel   = "Set high 256B region ($FE00 - $FEFF): ";
   const int lwidth  = _font.getStringWidth(middlelabel),
             fwidth  = _font.getStringWidth("Inactive"),
-            flwidth = _font.getStringWidth("ROM: ");
+            flwidth = _font.getStringWidth("ROM ");
 
   // Lower bank/region configuration
-  xpos = 10;
+  xpos = 2;
   new StaticTextWidget(_boss, _font, xpos, ypos, lwidth,
-    myFontHeight, lowerlabel, kTextAlignLeft);
+    myFontHeight, lowerlabel, TextAlign::Left);
   ypos += myLineHeight + 8;
 
   xpos += 40;
   myROMLower =
     new PopUpWidget(boss, _font, xpos, ypos-2, fwidth, myLineHeight,
-                    items32, "ROM: ", flwidth, kROMLowerChanged);
+                    items32, "ROM ", flwidth, kROMLowerChanged);
   myROMLower->setTarget(this);
   addFocusWidget(myROMLower);
 
   xpos += myROMLower->getWidth() + 20;
   myRAMLower =
     new PopUpWidget(boss, _font, xpos, ypos-2, fwidth, myLineHeight,
-                    items16, "RAM: ", flwidth, kRAMLowerChanged);
+                    items16, "RAM ", flwidth, kRAMLowerChanged);
   myRAMLower->setTarget(this);
   addFocusWidget(myRAMLower);
 
   // Middle bank/region configuration
-  xpos = 10;  ypos += myLineHeight + 14;
+  xpos = 2;  ypos += myLineHeight + 14;
   new StaticTextWidget(_boss, _font, xpos, ypos, lwidth,
-    myFontHeight, middlelabel, kTextAlignLeft);
+    myFontHeight, middlelabel, TextAlign::Left);
   ypos += myLineHeight + 8;
 
   xpos += 40;
   myROMMiddle =
     new PopUpWidget(boss, _font, xpos, ypos-2, fwidth, myLineHeight,
-                    items32, "ROM: ", flwidth, kROMMiddleChanged);
+                    items32, "ROM ", flwidth, kROMMiddleChanged);
   myROMMiddle->setTarget(this);
   addFocusWidget(myROMMiddle);
 
   xpos += myROMMiddle->getWidth() + 20;
   myRAMMiddle =
     new PopUpWidget(boss, _font, xpos, ypos-2, fwidth, myLineHeight,
-                    items16, "RAM: ", flwidth, kRAMMiddleChanged);
+                    items16, "RAM ", flwidth, kRAMMiddleChanged);
   myRAMMiddle->setTarget(this);
   addFocusWidget(myRAMMiddle);
 
   // High bank/region configuration
-  xpos = 10;  ypos += myLineHeight + 14;
+  xpos = 2;  ypos += myLineHeight + 14;
   new StaticTextWidget(_boss, _font, xpos, ypos, lwidth,
-    myFontHeight, highlabel, kTextAlignLeft);
+    myFontHeight, highlabel, TextAlign::Left);
   ypos += myLineHeight + 8;
 
   xpos += 40;
   myROMHigh =
     new PopUpWidget(boss, _font, xpos, ypos-2, fwidth, myLineHeight,
-                    items256, "ROM: ", flwidth, kROMHighChanged);
+                    items256, "ROM ", flwidth, kROMHighChanged);
   myROMHigh->setTarget(this);
   addFocusWidget(myROMHigh);
 
   xpos += myROMHigh->getWidth() + 20;
   myRAMHigh =
     new PopUpWidget(boss, _font, xpos, ypos-2, fwidth, myLineHeight,
-                    items128, "RAM: ", flwidth, kRAMHighChanged);
+                    items128, "RAM ", flwidth, kRAMHighChanged);
   myRAMHigh->setTarget(this);
   addFocusWidget(myRAMHigh);
 }
@@ -171,7 +169,7 @@ void Cartridge4A50Widget::loadConfig()
 void Cartridge4A50Widget::handleCommand(CommandSender* sender,
                                       int cmd, int data, int id)
 {
-  myCart.unlockBank();
+  myCart.unlockHotspots();
 
   switch(cmd)
   {
@@ -258,9 +256,12 @@ void Cartridge4A50Widget::handleCommand(CommandSender* sender,
         myCart.bankROMHigh(0);
       }
       break;
+
+    default:
+      break;
   }
 
-  myCart.lockBank();
+  myCart.lockHotspots();
   invalidate();
 }
 
