@@ -1,23 +1,24 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: CartARWidget.cxx 3308 2016-05-24 16:55:45Z stephena $
 //============================================================================
 
 #include "CartAR.hxx"
+#include "OSystem.hxx"
+#include "Debugger.hxx"
+#include "CartDebug.hxx"
 #include "PopUpWidget.hxx"
 #include "CartARWidget.hxx"
 
@@ -26,16 +27,16 @@ CartridgeARWidget::CartridgeARWidget(
       GuiObject* boss, const GUI::Font& lfont, const GUI::Font& nfont,
       int x, int y, int w, int h, CartridgeAR& cart)
   : CartDebugWidget(boss, lfont, nfont, x, y, w, h),
-    myCart(cart)
+    myCart{cart}
 {
-  uInt16 size = myCart.mySize;
+  const size_t size = myCart.mySize;
 
   string info =
     "Supercharger cartridge, four 2K slices (3 RAM, 1 ROM)\n"
     "\nTHIS SCHEME IS NOT FULLY IMPLEMENTED OR TESTED\n";
 
-  int xpos = 10,
-      ypos = addBaseInformation(size, "Starpath", info) + myLineHeight;
+  constexpr int xpos = 2;
+  const int ypos = addBaseInformation(size, "Starpath", info) + myLineHeight;
 
   VariantList items;
   VarList::push_back(items, "  0");
@@ -71,9 +72,9 @@ CartridgeARWidget::CartridgeARWidget(
   VarList::push_back(items, " 30");
   VarList::push_back(items, " 31");
   myBank =
-    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth(" XX "),
-                    myLineHeight, items, "Set bank: ",
-                    _font.getStringWidth("Set bank: "), kBankChanged);
+    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth(" XX"),
+                    myLineHeight, items, "Set bank     ",
+                    0, kBankChanged);
   myBank->setTarget(this);
   addFocusWidget(myBank);
 }
@@ -81,7 +82,11 @@ CartridgeARWidget::CartridgeARWidget(
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeARWidget::loadConfig()
 {
-  myBank->setSelectedIndex(myCart.myCurrentBank);
+  CartDebug& cart = instance().debugger().cartDebug();
+  const CartState& state = static_cast<const CartState&>(cart.getState());
+  const CartState& oldstate = static_cast<const CartState&>(cart.getOldState());
+
+  myBank->setSelectedIndex(myCart.getBank(), state.bank != oldstate.bank);
 
   CartDebugWidget::loadConfig();
 }
@@ -92,9 +97,9 @@ void CartridgeARWidget::handleCommand(CommandSender* sender,
 {
   if(cmd == kBankChanged)
   {
-    myCart.unlockBank();
+    myCart.unlockHotspots();
     myCart.bank(myBank->getSelected());
-    myCart.lockBank();
+    myCart.lockHotspots();
     invalidate();
   }
 }
