@@ -1,23 +1,24 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: CartCMWidget.cxx 3308 2016-05-24 16:55:45Z stephena $
 //============================================================================
 
 #include "CartCM.hxx"
+#include "OSystem.hxx"
+#include "Debugger.hxx"
+#include "CartDebug.hxx"
 #include "RiotDebug.hxx"
 #include "DataGridWidget.hxx"
 #include "EditTextWidget.hxx"
@@ -30,9 +31,9 @@ CartridgeCMWidget::CartridgeCMWidget(
       GuiObject* boss, const GUI::Font& lfont, const GUI::Font& nfont,
       int x, int y, int w, int h, CartridgeCM& cart)
   : CartDebugWidget(boss, lfont, nfont, x, y, w, h),
-    myCart(cart)
+    myCart{cart}
 {
-  uInt16 size = 4 * 4096;
+  constexpr uInt16 size = 4 * 4096;
 
   string info =
     "CM cartridge, four 4K banks + 2K RAM\n"
@@ -42,7 +43,7 @@ CartridgeCMWidget::CartridgeCMWidget(
     "used to control the cart functionality\n"
     "Startup bank = 3 (ROM), RAM disabled\n";
 
-  int xpos = 10,
+  int xpos = 2,
       ypos = addBaseInformation(size, "CompuMate", info) + myLineHeight;
 
   VariantList items;
@@ -52,16 +53,16 @@ CartridgeCMWidget::CartridgeCMWidget(
   VarList::push_back(items, " 3 ");
   myBank =
     new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth(" 0 "),
-                    myLineHeight, items, "Set bank: ",
-                    _font.getStringWidth("Set bank: "), kBankChanged);
+                    myLineHeight, items, "Set bank     ",
+                    0, kBankChanged);
   myBank->setTarget(this);
   addFocusWidget(myBank);
 
   // Raw SWCHA value (this will be broken down further in other UI elements)
-  int lwidth = _font.getStringWidth("Current column: ");
+  int lwidth = _font.getStringWidth("Current column ");
   ypos += myLineHeight + 8;
   new StaticTextWidget(boss, _font, xpos, ypos+2, lwidth, myFontHeight,
-                       "Current SWCHA: ", kTextAlignLeft);
+                       "Current SWCHA ", TextAlign::Left);
   xpos += lwidth;
   mySWCHA = new ToggleBitWidget(boss, _nfont, xpos, ypos, 8, 1);
   mySWCHA->setTarget(this);
@@ -70,10 +71,10 @@ CartridgeCMWidget::CartridgeCMWidget(
   // Current column number
   xpos = 10;  ypos += myLineHeight + 5;
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
-        myFontHeight, "Current column: ", kTextAlignLeft);
+        myFontHeight, "Current column ", TextAlign::Left);
   xpos += lwidth;
 
-  myColumn = new DataGridWidget(boss, _nfont, xpos, ypos-2, 1, 1, 2, 8, Common::Base::F_16);
+  myColumn = new DataGridWidget(boss, _nfont, xpos, ypos-2, 1, 1, 2, 8, Common::Base::Fmt::_16);
   myColumn->setTarget(this);
   myColumn->setEditable(false);
 
@@ -86,7 +87,7 @@ CartridgeCMWidget::CartridgeCMWidget(
   myIncrease->setTarget(this);
   myIncrease->setEditable(false);
 
-  int orig_ypos = ypos;  // save for when we go to the next column
+  const int orig_ypos = ypos;  // save for when we go to the next column
 
   // D5 (column part)
   ypos += myLineHeight + 4;
@@ -138,12 +139,12 @@ CartridgeCMWidget::CartridgeCMWidget(
 
   // Ram state (combination of several bits in SWCHA)
   ypos += myLineHeight + 8;
-  lwidth = _font.getStringWidth("Ram State: ");
+  lwidth = _font.getStringWidth("Ram State ");
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
-        myFontHeight, "Ram State: ", kTextAlignLeft);
-  myRAM = new EditTextWidget(boss, _nfont, xpos+lwidth, ypos-1,
+        myFontHeight, "Ram State ", TextAlign::Left);
+  myRAM = new EditTextWidget(boss, _nfont, xpos+lwidth, ypos-2,
               _nfont.getStringWidth(" Write-only "), myLineHeight, "");
-  myRAM->setEditable(false);
+  myRAM->setEditable(false, true);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -151,21 +152,23 @@ void CartridgeCMWidget::saveOldState()
 {
   myOldState.swcha = myCart.mySWCHA;
   myOldState.column = myCart.column();
-  
+
   myOldState.internalram.clear();
-  for(uInt32 i = 0; i < this->internalRamSize();i++)
+  for(uInt32 i = 0; i < internalRamSize(); ++i)
     myOldState.internalram.push_back(myCart.myRAM[i]);
+
+  myOldState.bank = myCart.getBank();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeCMWidget::loadConfig()
 {
-  myBank->setSelectedIndex(myCart.myCurrentBank);
+  myBank->setSelectedIndex(myCart.getBank(), myCart.getBank() != myOldState.bank);
 
   RiotDebug& riot = Debugger::debugger().riotDebug();
   const RiotState& state = static_cast<const RiotState&>(riot.getState());
 
-  uInt8 swcha = myCart.mySWCHA;
+  const uInt8 swcha = myCart.mySWCHA;
 
   // SWCHA
   BoolArray oldbits, newbits, changed;
@@ -194,8 +197,8 @@ void CartridgeCMWidget::loadConfig()
   myAudOut->setState(swcha & 0x40);
 
   // RAM state (several bits from SWCHA)
-  const string& ram = swcha & 0x10 ? " Inactive" :
-                        swcha & 0x20 ? " Read-only" : " Write-only";
+  const string& ram = (swcha & 0x10) ? " Inactive" :
+                      (swcha & 0x20) ? " Read-only" : " Write-only";
   myRAM->setText(ram, (swcha & 0x30) != (myOldState.swcha & 0x30));
 
   CartDebugWidget::loadConfig();
@@ -207,11 +210,11 @@ void CartridgeCMWidget::handleCommand(CommandSender* sender,
 {
   if(cmd == kBankChanged)
   {
-    myCart.unlockBank();
+    myCart.unlockHotspots();
     myCart.mySWCHA &= 0xFC;
     myCart.mySWCHA |= myBank->getSelected();
     myCart.bank(myCart.mySWCHA & 0x03);
-    myCart.lockBank();
+    myCart.lockHotspots();
     invalidate();
   }
 }
@@ -221,16 +224,16 @@ string CartridgeCMWidget::bankState()
 {
   ostringstream& buf = buffer();
 
-  buf << "Bank = " << std::dec << myCart.myCurrentBank
-      << ", RAM is" << (myCart.mySWCHA & 0x10 ? " Inactive" :
-         myCart.mySWCHA & 0x20 ? " Read-only" : " Write-only");
+  buf << "Bank = " << std::dec << myCart.getBank()
+      << ", RAM is" << ((myCart.mySWCHA & 0x10) ? " Inactive" :
+         (myCart.mySWCHA & 0x20) ? " Read-only" : " Write-only");
 
   return buf.str();
 }
 
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt32 CartridgeCMWidget::internalRamSize() 
+uInt32 CartridgeCMWidget::internalRamSize()
 {
   return 2048;
 }
@@ -242,12 +245,12 @@ uInt32 CartridgeCMWidget::internalRamRPort(int start)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string CartridgeCMWidget::internalRamDescription() 
+string CartridgeCMWidget::internalRamDescription()
 {
   ostringstream desc;
   desc << "$F800 - $FFFF used for Exclusive Read\n"
        << "              or Exclusive Write Access";
-  
+
   return desc.str();
 }
 
@@ -282,8 +285,8 @@ uInt8 CartridgeCMWidget::internalRamGetValue(int addr)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string CartridgeCMWidget::internalRamLabel(int addr) 
+string CartridgeCMWidget::internalRamLabel(int addr)
 {
-  CartDebug& dbg = instance().debugger().cartDebug();
+  const CartDebug& dbg = instance().debugger().cartDebug();
   return dbg.getLabel(addr + 0xF800, false);
 }
