@@ -1,43 +1,43 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: CartDPCPlusWidget.cxx 3308 2016-05-24 16:55:45Z stephena $
 //============================================================================
 
 #include "CartDPCPlus.hxx"
 #include "DataGridWidget.hxx"
 #include "PopUpWidget.hxx"
+#include "EditTextWidget.hxx"
 #include "CartDPCPlusWidget.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeDPCPlusWidget::CartridgeDPCPlusWidget(
       GuiObject* boss, const GUI::Font& lfont, const GUI::Font& nfont,
       int x, int y, int w, int h, CartridgeDPCPlus& cart)
-  : CartDebugWidget(boss, lfont, nfont, x, y, w, h),
-    myCart(cart)
+  : CartridgeARMWidget(boss, lfont, nfont, x, y, w, h, cart),
+    myCart{cart}
 {
-  uInt16 size = cart.mySize;
+  const size_t size = cart.mySize;
 
   ostringstream info;
   info << "Extended DPC cartridge, six 4K banks, 4K display bank, 1K frequency table, "
        << "8K DPC RAM\n"
        << "DPC registers accessible @ $F000 - $F07F\n"
        << "  $F000 - $F03F (R), $F040 - $F07F (W)\n"
-       << "Banks accessible at hotspots $FF6 to $FFB\n"
-       << "Startup bank = " << cart.myStartBank << "\n";
+       << "Banks accessible at hotspots $FFF6 to $FFFB\n"
+       << "Startup bank = " << cart.startBank() << "\n"
+       << "Ver = " << cart.myDriverMD5;
 
 #if 0
   // Eventually, we should query this from the debugger/disassembler
@@ -50,123 +50,123 @@ CartridgeDPCPlusWidget::CartridgeDPCPlusWidget(
   }
 #endif
 
-  int xpos = 10,
+  int xpos = 2,
       ypos = addBaseInformation(size, "Activision (Pitfall II)", info.str()) +
               myLineHeight;
 
   VariantList items;
-  VarList::push_back(items, "0 ($FF6)");
-  VarList::push_back(items, "1 ($FF7)");
-  VarList::push_back(items, "2 ($FF8)");
-  VarList::push_back(items, "3 ($FF9)");
-  VarList::push_back(items, "4 ($FFA)");
-  VarList::push_back(items, "5 ($FFB)");
+  VarList::push_back(items, "0 ($FFF6)");
+  VarList::push_back(items, "1 ($FFF7)");
+  VarList::push_back(items, "2 ($FFF8)");
+  VarList::push_back(items, "3 ($FFF9)");
+  VarList::push_back(items, "4 ($FFFA)");
+  VarList::push_back(items, "5 ($FFFB)");
   myBank =
-    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth("0 ($FFx) "),
-                    myLineHeight, items, "Set bank: ",
-                    _font.getStringWidth("Set bank: "), kBankChanged);
+    new PopUpWidget(boss, _font, xpos, ypos-2, _font.getStringWidth("0 ($FFFx)"),
+                    myLineHeight, items, "Set bank     ",
+                    0, kBankChanged);
   myBank->setTarget(this);
   addFocusWidget(myBank);
 
   // Top registers
-  int lwidth = _font.getStringWidth("Counter Registers: ");
-  xpos = 10;  ypos += myLineHeight + 8;
+  int lwidth = _font.getStringWidth("Counter Registers ");
+  xpos = 2;  ypos += myLineHeight + 8;
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
-        myFontHeight, "Top Registers: ", kTextAlignLeft);
+        myFontHeight, "Top Registers ", TextAlign::Left);
   xpos += lwidth;
 
-  myTops = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 2, 8, Common::Base::F_16);
+  myTops = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 2, 8, Common::Base::Fmt::_16);
   myTops->setTarget(this);
   myTops->setEditable(false);
 
   // Bottom registers
-  xpos = 10;  ypos += myLineHeight + 4;
+  xpos = 2;  ypos += myLineHeight + 4;
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
-        myFontHeight, "Bottom Registers: ", kTextAlignLeft);
+        myFontHeight, "Bottom Registers ", TextAlign::Left);
   xpos += lwidth;
 
-  myBottoms = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 2, 8, Common::Base::F_16);
+  myBottoms = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 2, 8, Common::Base::Fmt::_16);
   myBottoms->setTarget(this);
   myBottoms->setEditable(false);
 
   // Counter registers
-  xpos = 10;  ypos += myLineHeight + 4;
+  xpos = 2;  ypos += myLineHeight + 4;
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
-        myFontHeight, "Counter Registers: ", kTextAlignLeft);
+        myFontHeight, "Counter Registers ", TextAlign::Left);
   xpos += lwidth;
 
-  myCounters = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 4, 16, Common::Base::F_16_4);
+  myCounters = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 4, 16, Common::Base::Fmt::_16_4);
   myCounters->setTarget(this);
   myCounters->setEditable(false);
 
   // Fractional counter registers
-  xpos = 10;  ypos += myLineHeight + 4;
+  xpos = 2;  ypos += myLineHeight + 4;
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
-        myFontHeight, "Frac Counters: ", kTextAlignLeft);
+        myFontHeight, "Frac Counters ", TextAlign::Left);
   xpos += lwidth;
 
-  myFracCounters = new DataGridWidget(boss, _nfont, xpos, ypos-2, 4, 2, 8, 32, Common::Base::F_16_8);
+  myFracCounters = new DataGridWidget(boss, _nfont, xpos, ypos-2, 4, 2, 8, 32, Common::Base::Fmt::_16_8);
   myFracCounters->setTarget(this);
   myFracCounters->setEditable(false);
 
   // Fractional increment registers
-  xpos = 10;  ypos += myFracCounters->getHeight() + 8;
+  xpos = 2;  ypos += myFracCounters->getHeight() + 8;
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
-        myFontHeight, "Frac Increments: ", kTextAlignLeft);
+        myFontHeight, "Frac Increments ", TextAlign::Left);
   xpos += lwidth;
 
-  myFracIncrements = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 2, 8, Common::Base::F_16);
+  myFracIncrements = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 2, 8, Common::Base::Fmt::_16);
   myFracIncrements->setTarget(this);
   myFracIncrements->setEditable(false);
 
   // Special function parameters
-  xpos = 10;  ypos += myLineHeight + 4;
+  xpos = 2;  ypos += myLineHeight + 4;
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
-        myFontHeight, "Function Params: ", kTextAlignLeft);
+        myFontHeight, "Function Params ", TextAlign::Left);
   xpos += lwidth;
 
-  myParameter = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 2, 8, Common::Base::F_16);
+  myParameter = new DataGridWidget(boss, _nfont, xpos, ypos-2, 8, 1, 2, 8, Common::Base::Fmt::_16);
   myParameter->setTarget(this);
   myParameter->setEditable(false);
 
   // Music counters
-  xpos = 10;  ypos += myLineHeight + 4;
+  xpos = 2;  ypos += myLineHeight + 4;
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
-        myFontHeight, "Music Counters: ", kTextAlignLeft);
+        myFontHeight, "Music Counters ", TextAlign::Left);
   xpos += lwidth;
 
-  myMusicCounters = new DataGridWidget(boss, _nfont, xpos, ypos-2, 3, 1, 8, 32, Common::Base::F_16_8);
+  myMusicCounters = new DataGridWidget(boss, _nfont, xpos, ypos-2, 3, 1, 8, 32, Common::Base::Fmt::_16_8);
   myMusicCounters->setTarget(this);
   myMusicCounters->setEditable(false);
 
   // Music frequencies
-  xpos = 10;  ypos += myLineHeight + 4;
+  xpos = 2;  ypos += myLineHeight + 4;
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
-        myFontHeight, "Music Frequencies: ", kTextAlignLeft);
+        myFontHeight, "Music Frequencies ", TextAlign::Left);
   xpos += lwidth;
 
-  myMusicFrequencies = new DataGridWidget(boss, _nfont, xpos, ypos-2, 3, 1, 8, 32, Common::Base::F_16_8);
+  myMusicFrequencies = new DataGridWidget(boss, _nfont, xpos, ypos-2, 3, 1, 8, 32, Common::Base::Fmt::_16_8);
   myMusicFrequencies->setTarget(this);
   myMusicFrequencies->setEditable(false);
 
   // Music waveforms
-  xpos = 10;  ypos += myLineHeight + 4;
+  xpos = 2;  ypos += myLineHeight + 4;
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
-        myFontHeight, "Music Waveforms: ", kTextAlignLeft);
+        myFontHeight, "Music Waveforms ", TextAlign::Left);
   xpos += lwidth;
 
-  myMusicWaveforms = new DataGridWidget(boss, _nfont, xpos, ypos-2, 3, 1, 4, 16, Common::Base::F_16_4);
+  myMusicWaveforms = new DataGridWidget(boss, _nfont, xpos, ypos-2, 3, 1, 4, 16, Common::Base::Fmt::_16_4);
   myMusicWaveforms->setTarget(this);
   myMusicWaveforms->setEditable(false);
 
   // Current random number
-  lwidth = _font.getStringWidth("Current random number: ");
-  xpos = 10;  ypos += myLineHeight + 4;
+  lwidth = _font.getStringWidth("Current random number ");
+  xpos = 2;  ypos += myLineHeight + 4;
   new StaticTextWidget(boss, _font, xpos, ypos, lwidth,
-        myFontHeight, "Current random number: ", kTextAlignLeft);
+        myFontHeight, "Current random number ", TextAlign::Left);
   xpos += lwidth;
 
-  myRandom = new DataGridWidget(boss, _nfont, xpos, ypos-2, 1, 1, 8, 32, Common::Base::F_16_8);
+  myRandom = new DataGridWidget(boss, _nfont, xpos, ypos-2, 1, 1, 8, 32, Common::Base::Fmt::_16_8);
   myRandom->setTarget(this);
   myRandom->setEditable(false);
 
@@ -179,6 +179,9 @@ CartridgeDPCPlusWidget::CartridgeDPCPlusWidget(
   myIMLDA = new CheckboxWidget(boss, _font, xpos, ypos, "Immediate mode LDA");
   myIMLDA->setTarget(this);
   myIMLDA->setEditable(false);
+
+  xpos = 2;  ypos += myLineHeight + 4 * 1;
+  addCycleWidgets(xpos, ypos);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -212,15 +215,19 @@ void CartridgeDPCPlusWidget::saveOldState()
   }
 
   myOldState.random = myCart.myRandomNumber;
-  
+
   for(uInt32 i = 0; i < internalRamSize(); ++i)
     myOldState.internalram.push_back(myCart.myDisplayImage[i]);
+
+  myOldState.bank = myCart.getBank();
+
+  CartridgeARMWidget::saveOldState();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CartridgeDPCPlusWidget::loadConfig()
 {
-  myBank->setSelectedIndex(myCart.myCurrentBank);
+  myBank->setSelectedIndex(myCart.getBank(), myCart.getBank() != myOldState.bank);
 
   // Get registers, using change tracking
   IntArray alist;
@@ -234,7 +241,7 @@ void CartridgeDPCPlusWidget::loadConfig()
     changed.push_back(myCart.myTops[i] != myOldState.tops[i]);
   }
   myTops->setList(alist, vlist, changed);
-  
+
   alist.clear();  vlist.clear();  changed.clear();
   for(int i = 0; i < 8; ++i)
   {
@@ -305,7 +312,7 @@ void CartridgeDPCPlusWidget::loadConfig()
   myFastFetch->setState(myCart.myFastFetch);
   myIMLDA->setState(myCart.myLDAimmediate);
 
-  CartDebugWidget::loadConfig();
+  CartridgeARMWidget::loadConfig();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -314,11 +321,13 @@ void CartridgeDPCPlusWidget::handleCommand(CommandSender* sender,
 {
   if(cmd == kBankChanged)
   {
-    myCart.unlockBank();
+    myCart.unlockHotspots();
     myCart.bank(myBank->getSelected());
-    myCart.lockBank();
+    myCart.lockHotspots();
     invalidate();
   }
+  else
+    CartridgeARMWidget::handleCommand(sender, cmd, data, id);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -326,17 +335,17 @@ string CartridgeDPCPlusWidget::bankState()
 {
   ostringstream& buf = buffer();
 
-  static const char* spot[] = {
-    "$FF6", "$FF7", "$FF8", "$FF9", "$FFA", "$FFB"
+  static constexpr std::array<const char*, 6> spot = {
+    "$FFF6", "$FFF7", "$FFF8", "$FFF9", "$FFFA", "$FFFB"
   };
-  buf << "Bank = " << std::dec << myCart.myCurrentBank
-      << ", hotspot = " << spot[myCart.myCurrentBank];
+  buf << "Bank = " << std::dec << myCart.getBank()
+      << ", hotspot = " << spot[myCart.getBank()];
 
   return buf.str();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt32 CartridgeDPCPlusWidget::internalRamSize() 
+uInt32 CartridgeDPCPlusWidget::internalRamSize()
 {
   return 5*1024;
 }
@@ -348,7 +357,7 @@ uInt32 CartridgeDPCPlusWidget::internalRamRPort(int start)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string CartridgeDPCPlusWidget::internalRamDescription() 
+string CartridgeDPCPlusWidget::internalRamDescription()
 {
   ostringstream desc;
   desc << "$0000 - $0FFF - 4K display data\n"
@@ -357,7 +366,7 @@ string CartridgeDPCPlusWidget::internalRamDescription()
        << "$1000 - $13FF - 1K frequency table,\n"
        << "                C variables and C stack\n"
        << "                not accessible to 6507";
-  
+
   return desc.str();
 }
 
