@@ -1,62 +1,53 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: DataGridWidget.cxx 3304 2016-04-03 00:35:00Z stephena $
 //============================================================================
 
-#include "OSystem.hxx"
 #include "Widget.hxx"
 #include "Dialog.hxx"
+#include "ToolTip.hxx"
+#include "Font.hxx"
+#include "OSystem.hxx"
 #include "Debugger.hxx"
 #include "FrameBuffer.hxx"
+#include "FBSurface.hxx"
 #include "DataGridWidget.hxx"
 #include "DataGridOpsWidget.hxx"
 #include "RamWidget.hxx"
 #include "ScrollBarWidget.hxx"
+#include "StellaKeys.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 DataGridWidget::DataGridWidget(GuiObject* boss, const GUI::Font& font,
                                int x, int y, int cols, int rows,
                                int colchars, int bits,
-                               Common::Base::Format base,
+                               Common::Base::Fmt base,
                                bool useScrollbar)
   : EditableWidget(boss, font, x, y,
                    cols*(colchars * font.getMaxCharWidth() + 8) + 1,
                    font.getLineHeight()*rows + 1),
-    _rows(rows),
-    _cols(cols),
-    _currentRow(0),
-    _currentCol(0),
-    _rowHeight(font.getLineHeight()),
-    _colWidth(colchars * font.getMaxCharWidth() + 8),
-    _bits(bits),
-    _base(base),
-    _selectedItem(0),
-    _currentKeyDown(KBDK_UNKNOWN),
-    _opsWidget(nullptr),
-    _scrollBar(nullptr)
+    _rows{rows},
+    _cols{cols},
+    _rowHeight{font.getLineHeight()},
+    _colWidth{colchars * font.getMaxCharWidth() + 8},
+    _bits{bits},
+    _base{base}
 {
-  _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS |
-           WIDGET_WANTS_RAWDATA;
+  _flags = Widget::FLAG_ENABLED | Widget::FLAG_RETAIN_FOCUS | Widget::FLAG_WANTS_RAWDATA;
   _editMode = false;
 
-  // The item is selected, thus _bgcolor is used to draw the caret and
-  // _textcolorhi to erase it
-  _caretInverse = true;
-
   // Make sure all lists contain some default values
   _hiliteList.clear();
   int size = _rows * _cols;
@@ -64,18 +55,22 @@ DataGridWidget::DataGridWidget(GuiObject* boss, const GUI::Font& font,
   {
     _addrList.push_back(0);
     _valueList.push_back(0);
-    _valueStringList.push_back("");
+    _valueStringList.push_back(EmptyString);
+    _toolTipList.push_back(EmptyString);
     _changedList.push_back(0);
     _hiliteList.push_back(false);
   }
 
   // Set lower and upper bounds to sane values
   setRange(0, 1 << bits);
+  // Limit number of chars allowed
+  setMaxLen(colchars);
 
   // Add a scrollbar if necessary
   if(useScrollbar)
   {
-    _scrollBar = new ScrollBarWidget(boss, font, _x + _w, _y, kScrollBarWidth, _h);
+    _scrollBar = new ScrollBarWidget(boss, font, _x + _w, _y,
+                                     ScrollBarWidget::scrollBarWidth(_font), _h);
     _scrollBar->setTarget(this);
     _scrollBar->_numEntries = 1;
     _scrollBar->_currentPos = 0;
@@ -84,26 +79,25 @@ DataGridWidget::DataGridWidget(GuiObject* boss, const GUI::Font& font,
   }
 
   // Add filtering
-  EditableWidget::TextFilter f;
   switch(base)
   {
-    case Common::Base::F_16:
-    case Common::Base::F_16_1:
-    case Common::Base::F_16_2:
-    case Common::Base::F_16_4:
-    case Common::Base::F_16_8:
+    case Common::Base::Fmt::_16:
+    case Common::Base::Fmt::_16_1:
+    case Common::Base::Fmt::_16_2:
+    case Common::Base::Fmt::_16_4:
+    case Common::Base::Fmt::_16_8:
       setTextFilter([](char c) {
         return (c >= '0' && c <= '9') || (c >= 'a' && c <= 'f');
       });
       break;
-    case Common::Base::F_10:
+    case Common::Base::Fmt::_10:
       setTextFilter([](char c) {
         return (c >= '0' && c <= '9');
       });
       break;
-    case Common::Base::F_2:
-    case Common::Base::F_2_8:
-    case Common::Base::F_2_16:
+    case Common::Base::Fmt::_2:
+    case Common::Base::Fmt::_2_8:
+    case Common::Base::Fmt::_2_16:
       setTextFilter([](char c) { return (c >= '0' && c <= '1'); });
       break;
     default:
@@ -115,14 +109,19 @@ DataGridWidget::DataGridWidget(GuiObject* boss, const GUI::Font& font,
 void DataGridWidget::setList(const IntArray& alist, const IntArray& vlist,
                              const BoolArray& changed)
 {
-/*
-cerr << "alist.size() = "     << alist.size()
-     << ", vlist.size() = "   << vlist.size()
-     << ", changed.size() = " << changed.size()
-     << ", _rows*_cols = "    << _rows * _cols << endl << endl;
-*/
-  int size = int(vlist.size());  // assume the alist is the same size
-  assert(size == _rows * _cols);
+  /*
+  cerr << "alist.size() = "     << alist.size()
+       << ", vlist.size() = "   << vlist.size()
+       << ", changed.size() = " << changed.size()
+       << ", _rows*_cols = "    << _rows * _cols << endl << endl;
+  */
+  const size_t size = vlist.size();  // assume the alist is the same size
+
+  const bool dirty = _editMode
+    || !std::equal(_valueList.begin(), _valueList.end(),
+                   vlist.begin(), vlist.end())
+    || !std::equal(_changedList.begin(), _changedList.end(),
+                   changed.begin(), changed.end());
 
   _addrList.clear();
   _valueList.clear();
@@ -135,22 +134,25 @@ cerr << "alist.size() = "     << alist.size()
 
   // An efficiency thing
   string temp;
-  for(int i = 0; i < size; ++i)
+  for(size_t i = 0; i < size; ++i)
     _valueStringList.push_back(Common::Base::toString(_valueList[i], _base));
 
-/*
-cerr << "_addrList.size() = "     << _addrList.size()
-     << ", _valueList.size() = "   << _valueList.size()
-     << ", _changedList.size() = " << _changedList.size()
-     << ", _valueStringList.size() = " << _valueStringList.size()
-     << ", _rows*_cols = "    << _rows * _cols << endl << endl;
-*/
+  /*
+  cerr << "_addrList.size() = "     << _addrList.size()
+       << ", _valueList.size() = "   << _valueList.size()
+       << ", _changedList.size() = " << _changedList.size()
+       << ", _valueStringList.size() = " << _valueStringList.size()
+       << ", _rows*_cols = "    << _rows * _cols << endl << endl;
+  */
   enableEditMode(false);
 
-  // Send item selected signal for starting with cell 0
-  sendCommand(DataGridWidget::kSelectionChangedCmd, _selectedItem, _id);
+  if(dirty)
+  {
+    // Send item selected signal for starting with cell 0
+    sendCommand(DataGridWidget::kSelectionChangedCmd, _selectedItem, _id);
 
-  setDirty();
+    setDirty();
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -174,12 +176,19 @@ void DataGridWidget::setList(int a, int v)
 
   alist.push_back(a);
   vlist.push_back(v);
-  bool diff = _addrList.size() == 1 ? getSelectedValue() != v : false;
+  const bool diff = _addrList.size() == 1 ? getSelectedValue() != v : false;
   changed.push_back(diff);
 
   setList(alist, vlist, changed);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DataGridWidget::setEditable(bool editable, bool hiliteBG)
+{
+  // Override parent method; enable hilite when widget is not editable
+  EditableWidget::setEditable(editable, hiliteBG);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::setHiliteList(const BoolArray& hilitelist)
 {
@@ -219,7 +228,7 @@ void DataGridWidget::setValueInternal(int position, int value, bool changed)
 void DataGridWidget::setValue(int position, int value, bool changed,
                               bool emitSignal)
 {
-  if(position >= 0 && uInt32(position) < _valueList.size())
+  if(position >= 0 && position < static_cast<int>(_valueList.size()))
   {
     // Correctly format the data for viewing
     editString() = Common::Base::toString(value, _base);
@@ -243,15 +252,16 @@ void DataGridWidget::setRange(int lower, int upper)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DataGridWidget::handleMouseDown(int x, int y, int button, int clickCount)
+void DataGridWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
 {
   if (!isEnabled())
     return;
 
+  resetSelection();
   // First check whether the selection changed
   int newSelectedItem;
   newSelectedItem = findItem(x, y);
-  if (newSelectedItem > int(_valueList.size()) - 1)
+  if (newSelectedItem > static_cast<int>(_valueList.size()) - 1)
     newSelectedItem = -1;
 
   if (_selectedItem != newSelectedItem)
@@ -269,7 +279,7 @@ void DataGridWidget::handleMouseDown(int x, int y, int button, int clickCount)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void DataGridWidget::handleMouseUp(int x, int y, int button, int clickCount)
+void DataGridWidget::handleMouseUp(int x, int y, MouseButton b, int clickCount)
 {
   // If this was a double click and the mouse is still over the selected item,
   // send the double click command
@@ -295,6 +305,7 @@ void DataGridWidget::handleMouseWheel(int x, int y, int direction)
     else if(direction < 0)
       incrementCell();
   }
+  dialog().tooltip().hide();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -324,9 +335,8 @@ bool DataGridWidget::handleText(char text)
 bool DataGridWidget::handleKeyDown(StellaKey key, StellaMod mod)
 {
   // Ignore all mod keys
-  if(instance().eventHandler().kbdControl(mod) ||
-     instance().eventHandler().kbdAlt(mod))
-    return true;
+  if(StellaModTest::isControl(mod) || StellaModTest::isAlt(mod))
+    return false;
 
   bool handled = true;
   bool dirty = false;
@@ -407,7 +417,7 @@ bool DataGridWidget::handleKeyDown(StellaKey key, StellaMod mod)
         break;
 
       case KBDK_PAGEUP:
-        if(instance().eventHandler().kbdShift(mod) && _scrollBar)
+        if(StellaModTest::isShift(mod) && _scrollBar)
           handleMouseWheel(0, 0, -1);
         else if (_currentRow > 0)
         {
@@ -417,7 +427,7 @@ bool DataGridWidget::handleKeyDown(StellaKey key, StellaMod mod)
         break;
 
       case KBDK_PAGEDOWN:
-        if(instance().eventHandler().kbdShift(mod) && _scrollBar)
+        if(StellaModTest::isShift(mod) && _scrollBar)
           handleMouseWheel(0, 0, +1);
         else if (_currentRow < int(_rows) - 1)
         {
@@ -486,13 +496,14 @@ bool DataGridWidget::handleKeyDown(StellaKey key, StellaMod mod)
 
   if (dirty)
   {
-    int oldItem = _selectedItem;
+    const int oldItem = _selectedItem;
     _selectedItem = _currentRow*_cols + _currentCol;
 
     if(_selectedItem != oldItem)
       sendCommand(DataGridWidget::kSelectionChangedCmd, _selectedItem, _id);
 
     setDirty();
+    dialog().tooltip().hide();
   }
 
   _currentKeyDown = key;
@@ -534,9 +545,9 @@ void DataGridWidget::handleCommand(CommandSender* sender, int cmd,
 {
   switch(cmd)
   {
-    case kSetPositionCmd:
+    case GuiObject::kSetPositionCmd:
       // Chain access; pass to parent
-      sendCommand(kSetPositionCmd, data, _id);
+      sendCommand(GuiObject::kSetPositionCmd, data, _id);
       break;
 
     case kDGZeroCmd:
@@ -566,90 +577,164 @@ void DataGridWidget::handleCommand(CommandSender* sender, int cmd,
     case kDGShiftRCmd:
       rshiftCell();
       break;
+
+    default:
+      return;
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DataGridWidget::setToolTip(int column, int row, const string& text)
+{
+  if(row >= 0 && row < _rows && column >= 0 && column < _cols)
+    _toolTipList[row * _cols + column] = text;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int DataGridWidget::getToolTipIndex(const Common::Point& pos) const
+{
+  const int col = (pos.x - getAbsX()) / _colWidth;
+  const int row = (pos.y - getAbsY()) / _rowHeight;
+
+  if(row >= 0 && row < _rows && col >= 0 && col < _cols)
+    return row * _cols + col;
+  else
+    return -1;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string DataGridWidget::getToolTip(const Common::Point& pos) const
+{
+  const int idx = getToolTipIndex(pos);
+
+  if(idx < 0)
+    return EmptyString;
+
+  const Int32 val = _valueList[idx];
+  ostringstream buf;
+
+  if(_toolTipList[idx] != EmptyString)
+    buf << _toolTipList[idx];
+  else
+    buf << _toolTipText;
+   buf << "$" << Common::Base::toString(val, Common::Base::Fmt::_16)
+       << " = #" << val;
+  if(val < 0x100)
+  {
+    if(val >= 0x80)
+      buf << '/' << -(0x100 - val);
+    buf << " = %" << Common::Base::toString(val, Common::Base::Fmt::_2);
+  }
+
+  return buf.str();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool DataGridWidget::changedToolTip(const Common::Point& oldPos,
+                                    const Common::Point& newPos) const
+{
+  return getToolTipIndex(oldPos) != getToolTipIndex(newPos);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::drawWidget(bool hilite)
 {
   FBSurface& s = _boss->dialog().surface();
-  int row, col;
 
+  s.fillRect(_x, _y, _w, _h, hilite && isEnabled() && isEditable() ? _bgcolorhi : _bgcolor);
   // Draw the internal grid and labels
-  int linewidth = _cols * _colWidth;
-  for (row = 0; row <= _rows; row++)
-    s.hLine(_x, _y + (row * _rowHeight), _x + linewidth, kColor);
-  int lineheight = _rows * _rowHeight;
-  for (col = 0; col <= _cols; col++)
-    s.vLine(_x + (col * _colWidth), _y, _y + lineheight, kColor);
+  const int linewidth = _cols * _colWidth;
+  s.frameRect(_x, _y, _w, _h, hilite && isEnabled() && isEditable() ? kWidColorHi : kColor);
+  for(int row = 1; row <= _rows-1; row++)
+    s.hLine(_x+1, _y + (row * _rowHeight), _x + linewidth-1, kBGColorLo);
+
+  const int lineheight = _rows * _rowHeight;
+  for(int col = 1; col <= _cols-1; col++)
+    s.vLine(_x + (col * _colWidth), _y+1, _y + lineheight-1, kBGColorLo);
 
   // Draw the list items
-  for (row = 0; row < _rows; row++)
+  for(int row = 0; row < _rows; row++)
   {
-    for (col = 0; col < _cols; col++)
+    for(int col = 0; col < _cols; col++)
     {
-      int x = _x + 4 + (col * _colWidth);
-      int y = _y + 2 + (row * _rowHeight);
-      int pos = row*_cols + col;
+      const int x = _x + 4 + (col * _colWidth);
+      const int y = _y + 2 + (row * _rowHeight);
+      const int pos = row*_cols + col;
+      ColorId textColor = kTextColor;
 
       // Draw the selected item inverted, on a highlighted background.
       if (_currentRow == row && _currentCol == col &&
           _hasFocus && !_editMode)
+      {
         s.fillRect(x - 4, y - 2, _colWidth+1, _rowHeight+1, kTextColorHi);
+        textColor = kTextColorInv;
+      }
 
       if (_selectedItem == pos && _editMode)
       {
         adjustOffset();
-        s.drawString(_font, editString(), x, y, _colWidth, kTextColor,
-                     kTextAlignLeft, -_editScrollOffset, false);
+        s.drawString(_font, editString(), x, y, _colWidth, textColor,
+                     TextAlign::Left, -_editScrollOffset, false);
       }
       else
       {
-        uInt32 color = kTextColor;
         if(_changedList[pos])
         {
           s.fillRect(x - 3, y - 1, _colWidth-1, _rowHeight-1, kDbgChangedColor);
 
           if(_hiliteList[pos])
-            color = kDbgColorHi;
+            textColor = kDbgColorHi;
           else
-            color = kDbgChangedTextColor;
+            textColor = kDbgChangedTextColor;
         }
         else if(_hiliteList[pos])
-          color = kDbgColorHi;
+          textColor = kDbgColorHi;
 
-        s.drawString(_font, _valueStringList[pos], x, y, _colWidth, color);
+        s.drawString(_font, _valueStringList[pos], x, y, _colWidth, textColor);
       }
     }
   }
 
   // Only draw the caret while editing, and if it's in the current viewport
   if(_editMode)
-    drawCaret();
+    drawCaretSelection();
 
   // Draw the scrollbar
   if(_scrollBar)
     _scrollBar->recalc();  // takes care of the draw
+
+  // Cross out the grid?
+  if(_crossGrid)
+  {
+    s.line(_x + 1, _y + 1, _x + _w - 2, _y + _h - 1, kColor);
+    s.line(_x + _w - 2, _y + 1, _x + 1, _y + _h - 1, kColor);
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect DataGridWidget::getEditRect() const
+Common::Rect DataGridWidget::getEditRect() const
 {
-  GUI::Rect r(1, 0, _colWidth, _rowHeight);
   const int rowoffset = _currentRow * _rowHeight;
   const int coloffset = _currentCol * _colWidth + 4;
-  r.top += rowoffset;
-  r.bottom += rowoffset;
-  r.left += coloffset;
-  r.right += coloffset - 5;
 
-  return r;
+  return Common::Rect(1 + coloffset, rowoffset,
+                      _colWidth + coloffset - 5, _rowHeight + rowoffset);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int DataGridWidget::getWidth() const
 {
-  return _w + (_scrollBar ? kScrollBarWidth : 0);
+  return _w + (_scrollBar ? ScrollBarWidget::scrollBarWidth(_font) : 0);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void DataGridWidget::setCrossed(bool enable)
+{
+  if(_crossGrid != enable)
+  {
+    _crossGrid = enable;
+    setDirty();
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -657,8 +742,10 @@ void DataGridWidget::startEditMode()
 {
   if (isEditable() && !_editMode && _selectedItem >= 0)
   {
+    dialog().tooltip().hide();
     enableEditMode(true);
     setText("", true);  // Erase current entry when starting editing
+    backupString() = "@@"; // dummy value to process Escape correctly key when nothing is entered
   }
 }
 
@@ -676,26 +763,27 @@ void DataGridWidget::endEditMode()
   {
     switch(_base)
     {
-      case Common::Base::F_16:
-      case Common::Base::F_16_1:
-      case Common::Base::F_16_2:
-      case Common::Base::F_16_4:
-      case Common::Base::F_16_8:
+      case Common::Base::Fmt::_16:
+      case Common::Base::Fmt::_16_1:
+      case Common::Base::Fmt::_16_2:
+      case Common::Base::Fmt::_16_4:
+      case Common::Base::Fmt::_16_8:
         editString().insert(0, 1, '$');
         break;
-      case Common::Base::F_2:
-      case Common::Base::F_2_8:
-      case Common::Base::F_2_16:
+      case Common::Base::Fmt::_2:
+      case Common::Base::Fmt::_2_8:
+      case Common::Base::Fmt::_2_16:
         editString().insert(0, 1, '\\');
         break;
-      case Common::Base::F_10:
+      case Common::Base::Fmt::_10:
         editString().insert(0, 1, '#');
         break;
-      case Common::Base::F_DEFAULT:
+      case Common::Base::Fmt::_DEFAULT:
+      default:  // TODO - properly handle all other cases
         break;
     }
   }
-  int value = instance().debugger().stringToValue(editString());
+  const int value = instance().debugger().stringToValue(editString());
   if(value < _lowerBound || value >= _upperBound)
   {
     abortEditMode();
@@ -703,6 +791,7 @@ void DataGridWidget::endEditMode()
   }
 
   setSelectedValue(value);
+  commit();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -710,6 +799,7 @@ void DataGridWidget::abortEditMode()
 {
   if(_editMode)
   {
+    abort();
     // Undo any changes made
     assert(_selectedItem >= 0);
     enableEditMode(false);
@@ -719,7 +809,7 @@ void DataGridWidget::abortEditMode()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::negateCell()
 {
-  int mask  = (1 << _bits) - 1;
+  const int mask  = (1 << _bits) - 1;
   int value = getSelectedValue();
   if(mask != _upperBound - 1)     // ignore when values aren't byte-aligned
     return;
@@ -731,7 +821,7 @@ void DataGridWidget::negateCell()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::invertCell()
 {
-  int mask  = (1 << _bits) - 1;
+  const int mask  = (1 << _bits) - 1;
   int value = getSelectedValue();
   if(mask != _upperBound - 1)     // ignore when values aren't byte-aligned
     return;
@@ -743,7 +833,7 @@ void DataGridWidget::invertCell()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::decrementCell()
 {
-  int mask  = (1 << _bits) - 1;
+  const int mask  = (1 << _bits) - 1;
   int value = getSelectedValue();
   if(value <= _lowerBound)        // take care of wrap-around
     value = _upperBound;
@@ -755,7 +845,7 @@ void DataGridWidget::decrementCell()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::incrementCell()
 {
-  int mask  = (1 << _bits) - 1;
+  const int mask  = (1 << _bits) - 1;
   int value = getSelectedValue();
   if(value >= _upperBound - 1)    // take care of wrap-around
     value = _lowerBound - 1;
@@ -767,7 +857,7 @@ void DataGridWidget::incrementCell()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::lshiftCell()
 {
-  int mask  = (1 << _bits) - 1;
+  const int mask  = (1 << _bits) - 1;
   int value = getSelectedValue();
   if(mask != _upperBound - 1)     // ignore when values aren't byte-aligned
     return;
@@ -779,7 +869,7 @@ void DataGridWidget::lshiftCell()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void DataGridWidget::rshiftCell()
 {
-  int mask  = (1 << _bits) - 1;
+  const int mask  = (1 << _bits) - 1;
   int value = getSelectedValue();
   if(mask != _upperBound - 1)     // ignore when values aren't byte-aligned
     return;
