@@ -1,100 +1,98 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: PromptWidget.cxx 3304 2016-04-03 00:35:00Z stephena $
 //============================================================================
 
-#include <iostream>
-#include <fstream>
-#include <algorithm>
-
 #include "ScrollBarWidget.hxx"
-#include "FrameBuffer.hxx"
-#include "EventHandler.hxx"
+#include "FBSurface.hxx"
+#include "Font.hxx"
+#include "StellaKeys.hxx"
 #include "Version.hxx"
+#include "OSystem.hxx"
 #include "Debugger.hxx"
 #include "DebuggerDialog.hxx"
 #include "DebuggerParser.hxx"
+#include "EventHandler.hxx"
 
 #include "PromptWidget.hxx"
 #include "CartDebug.hxx"
 
 #define PROMPT  "> "
 
+// Uncomment the following to give full-line cut/copy/paste
+// Note that this will be removed eventually, when we implement proper cut/copy/paste
+#define PSEUDO_CUT_COPY_PASTE
+
+// TODO: Github issue #361
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PromptWidget::PromptWidget(GuiObject* boss, const GUI::Font& font,
                            int x, int y, int w, int h)
-  : Widget(boss, font, x, y, w - kScrollBarWidth, h),
-    CommandSender(boss),
-    _makeDirty(false),
-    _firstTime(true),
-    _exitedEarly(false)
+  : Widget(boss, font, x, y, w - ScrollBarWidget::scrollBarWidth(font), h),
+    CommandSender(boss)
 {
-  _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS |
-           WIDGET_WANTS_TAB | WIDGET_WANTS_RAWDATA;
+  _flags = Widget::FLAG_ENABLED | Widget::FLAG_CLEARBG | Widget::FLAG_RETAIN_FOCUS |
+           Widget::FLAG_WANTS_TAB | Widget::FLAG_WANTS_RAWDATA;
   _textcolor = kTextColor;
   _bgcolor = kWidColor;
+  _bgcolorlo = kDlgColor;
 
   _kConsoleCharWidth  = font.getMaxCharWidth();
   _kConsoleCharHeight = font.getFontHeight();
   _kConsoleLineHeight = _kConsoleCharHeight + 2;
 
   // Calculate depending values
-  _lineWidth = (_w - kScrollBarWidth - 2) / _kConsoleCharWidth;
+  _lineWidth = (_w - ScrollBarWidget::scrollBarWidth(_font) - 2) / _kConsoleCharWidth;
   _linesPerPage = (_h - 2) / _kConsoleLineHeight;
   _linesInBuffer = kBufferSize / _lineWidth;
 
   // Add scrollbar
-  _scrollBar = new ScrollBarWidget(boss, font, _x + _w, _y, kScrollBarWidth, _h);
+  _scrollBar = new ScrollBarWidget(boss, font, _x + _w, _y,
+                                   ScrollBarWidget::scrollBarWidth(_font), _h);
   _scrollBar->setTarget(this);
 
-  // Init colors
-  _inverse = false;
-
   clearScreen();
 
   addFocusWidget(this);
+  setHelpAnchor("PromptTab", true);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PromptWidget::drawWidget(bool hilite)
 {
 //cerr << "PromptWidget::drawWidget\n";
-  uInt32 fgcolor, bgcolor;
-
+  ColorId fgcolor{}, bgcolor{};
   FBSurface& s = _boss->dialog().surface();
 
   // Draw text
-  int start = _scrollLine - _linesPerPage + 1;
+  const int start = _scrollLine - _linesPerPage + 1;
   int y = _y + 2;
 
   for (int line = 0; line < _linesPerPage; ++line)
   {
     int x = _x + 1;
     for (int column = 0; column < _lineWidth; ++column) {
-      int c = buffer((start + line) * _lineWidth + column);
+      const int c = buffer((start + line) * _lineWidth + column);
 
       if(c & (1 << 17))  // inverse video flag
       {
         fgcolor = _bgcolor;
-        bgcolor = (c & 0x1ffff) >> 8;
+        bgcolor = static_cast<ColorId>((c & 0x1ffff) >> 8);
         s.fillRect(x, y, _kConsoleCharWidth, _kConsoleCharHeight, bgcolor);
       }
       else
-        fgcolor = c >> 8;
+        fgcolor = static_cast<ColorId>(c >> 8);
 
       s.drawChar(_font, c & 0x7f, x, y, fgcolor);
       x += _kConsoleCharWidth;
@@ -110,7 +108,7 @@ void PromptWidget::drawWidget(bool hilite)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PromptWidget::handleMouseDown(int x, int y, int button, int clickCount)
+void PromptWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
 {
 //  cerr << "PromptWidget::handleMouseDown\n";
 }
@@ -130,6 +128,8 @@ void PromptWidget::printPrompt()
 
   print(PROMPT);
   _promptStartPos = _promptEndPos = _currentPos;
+
+  resetFunctions();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -143,6 +143,8 @@ bool PromptWidget::handleText(char text)
     _promptEndPos++;
     putcharIntern(text);
     scrollToCurrent();
+
+    resetFunctions();
   }
   return true;
 }
@@ -150,278 +152,171 @@ bool PromptWidget::handleText(char text)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
 {
-  bool handled = true;
-  bool dirty = false;
-  
-  switch(key)
+  bool handled = true,
+    dirty = true,
+    changeInput = false,
+    resetAutoComplete = true,
+    resetHistoryScroll = true;
+
+  // Uses normal edit events + special prompt events
+  Event::Type event = instance().eventHandler().eventForKey(EventMode::kEditMode, key, mod);
+  if(event == Event::NoType)
+    event = instance().eventHandler().eventForKey(EventMode::kPromptMode, key, mod);
+
+  switch(event)
   {
-    case KBDK_RETURN:
-    case KBDK_KP_ENTER:
+    case Event::EndEdit:
     {
-      nextLine();
-
-      assert(_promptEndPos >= _promptStartPos);
-      int len = _promptEndPos - _promptStartPos;
-
-      if (len > 0)
-      {
-        // Copy the user input to command
-        string command;
-        for (int i = 0; i < len; i++)
-          command += buffer(_promptStartPos + i) & 0x7f;
-
-        // Add the input to the history
-        addToHistory(command.c_str());
-
-        // Pass the command to the debugger, and print the result
-        string result = instance().debugger().run(command);
-
-        // This is a bit of a hack
-        // Certain commands remove the debugger dialog from underneath us,
-        // so we shouldn't print any messages
-        // Those commands will return '_EXIT_DEBUGGER' as their result
-        if(result == "_EXIT_DEBUGGER")
-        {
-          _exitedEarly = true;
-          return true;
-        }
-        else if(result != "")
-          print(result + "\n");
-      }
+      if(execute())
+        return true;
 
       printPrompt();
-      dirty = true;
       break;
     }
 
-    case KBDK_TAB:
-    {
-      // Tab completion: we complete either commands or labels, but not
-      // both at once.
+    // special events (auto complete & history scrolling)
+    case Event::UINavNext:
+      dirty = changeInput = autoComplete(+1);
+      resetAutoComplete = false;
+      break;
 
-      if(_currentPos <= _promptStartPos)
+    case Event::UINavPrev:
+      dirty = changeInput = autoComplete(-1);
+      resetAutoComplete = false;
+      break;
+
+    case Event::UILeft: // mapped to KBDK_DOWN by default
+      dirty = changeInput = historyScroll(-1);
+      resetHistoryScroll = false;
+      break;
+
+    case Event::UIRight: // mapped to KBDK_UP by default
+      dirty = changeInput = historyScroll(+1);
+      resetHistoryScroll = false;
+      break;
+
+    // input modifying events
+    case Event::Backspace:
+      if(_currentPos > _promptStartPos)
+      {
+        killChar(-1);
+        changeInput = true;
+      }
+      scrollToCurrent();
+      break;
+
+    case Event::Delete:
+      killChar(+1);
+      changeInput = true;
+      break;
+
+    case Event::DeleteEnd:
+      killLine(+1);
+      changeInput = true;
+      break;
+
+    case Event::DeleteHome:
+      killLine(-1);
+      changeInput = true;
+      break;
+
+    case Event::DeleteLeftWord:
+      killWord();
+      changeInput = true;
+      break;
+
+    case Event::Cut:
+      textCut();
+      changeInput = true;
+      break;
+
+    case Event::Copy:
+      textCopy();
+      break;
+
+    case Event::Paste:
+      textPaste();
+      changeInput = true;
+      break;
+
+    // cursor events
+    case Event::MoveHome:
+      _currentPos = _promptStartPos;
+      break;
+
+    case Event::MoveEnd:
+      _currentPos = _promptEndPos;
+      break;
+
+    case Event::MoveRightChar:
+      if(_currentPos < _promptEndPos)
+        _currentPos++;
+      else
+        handled = false;
+      break;
+
+    case Event::MoveLeftChar:
+      if(_currentPos > _promptStartPos)
+        _currentPos--;
+      else
+        handled = false;
+      break;
+
+    // scrolling events
+    case Event::UIUp:
+      if(_scrollLine <= _firstLineInBuffer + _linesPerPage - 1)
         break;
 
-      scrollToCurrent();
-      int len = _promptEndPos - _promptStartPos;
-      if(len > 255) len = 255;
-
-      int lastDelimPos = -1;
-      char delimiter = '\0';
-
-      char str[256];
-      for (int i = 0; i < len; i++)
-      {
-        str[i] = buffer(_promptStartPos + i) & 0x7f;
-        if(strchr("{*@<> ", str[i]) != NULL )
-        {
-          lastDelimPos = i;
-          delimiter = str[i];
-        }
-      }
-      str[len] = '\0';
-
-      StringList list;
-      string completionList;
-      string prefix;
-
-      if(lastDelimPos < 0)
-      {
-        // no delimiters, do command completion:
-        const DebuggerParser& parser = instance().debugger().parser();
-        parser.getCompletions(str, list);
-
-        if(list.size() < 1)
-          break;
-
-        sort(list.begin(), list.end());
-        completionList = list[0];
-        for(uInt32 i = 1; i < list.size(); ++i)
-          completionList += " " + list[i];
-        prefix = getCompletionPrefix(list, str);
-      }
-      else
-      {
-        // we got a delimiter, so this must be a label or a function
-        const Debugger& dbg = instance().debugger();
-
-        dbg.cartDebug().getCompletions(str + lastDelimPos + 1, list);
-        dbg.getCompletions(str + lastDelimPos + 1, list);
-
-        if(list.size() < 1)
-          break;
-
-        sort(list.begin(), list.end());
-        completionList = list[0];
-        for(uInt32 i = 1; i < list.size(); ++i)
-          completionList += " " + list[i];
-        prefix = getCompletionPrefix(list, str + lastDelimPos + 1);
-      }
-
-      if(list.size() == 1)
-      {
-        // add to buffer as though user typed it (plus a space)
-        _currentPos = _promptStartPos + lastDelimPos + 1;
-        const char* clptr = completionList.c_str();
-        while(*clptr != '\0')
-          putcharIntern(*clptr++);
-
-        putcharIntern(' ');
-        _promptEndPos = _currentPos;
-      }
-      else
-      {
-        nextLine();
-        // add to buffer as-is, then add PROMPT plus whatever we have so far
-        _currentPos = _promptStartPos + lastDelimPos + 1;
-
-        print("\n");
-        print(completionList);
-        print("\n");
-        print(PROMPT);
-
-        _promptStartPos = _currentPos;
-
-        for(int i = 0; i < lastDelimPos; i++)
-          putcharIntern(str[i]);
-
-        if(lastDelimPos > 0)
-          putcharIntern(delimiter);
-
-        print(prefix);
-        _promptEndPos = _currentPos;
-      }
-      dirty = true;
-      break;
-    }
-
-    case KBDK_BACKSPACE:
-      if (_currentPos > _promptStartPos)
-        killChar(-1);
-
-      scrollToCurrent();
-      dirty = true;
+      _scrollLine -= 1;
+      updateScrollBuffer();
       break;
 
-    case KBDK_DELETE:
-      killChar(+1);
-      dirty = true;
+    case Event::UIDown:
+      // Don't scroll down when at bottom of buffer
+      if(_scrollLine >= _promptEndPos / _lineWidth)
+        break;
+
+      _scrollLine += 1;
+      updateScrollBuffer();
       break;
 
-    case KBDK_PAGEUP:
-      if (instance().eventHandler().kbdShift(mod))
-      {
-        // Don't scroll up when at top of buffer
-        if(_scrollLine < _linesPerPage)
-          break;
+    case Event::UIPgUp:
+      // Don't scroll up when at top of buffer
+      if(_scrollLine < _linesPerPage)
+        break;
 
-        _scrollLine -= _linesPerPage - 1;
-        if (_scrollLine < _firstLineInBuffer + _linesPerPage - 1)
-          _scrollLine = _firstLineInBuffer + _linesPerPage - 1;
-        updateScrollBuffer();
-
-        dirty = true;
-      }
-      break;
-
-    case KBDK_PAGEDOWN:
-      if (instance().eventHandler().kbdShift(mod))
-      {
-        // Don't scroll down when at bottom of buffer
-        if(_scrollLine >= _promptEndPos / _lineWidth)
-          break;
-
-        _scrollLine += _linesPerPage - 1;
-        if (_scrollLine > _promptEndPos / _lineWidth)
-          _scrollLine = _promptEndPos / _lineWidth;
-        updateScrollBuffer();
-
-        dirty = true;
-      }
-      break;
-
-    case KBDK_HOME:
-      if (instance().eventHandler().kbdShift(mod))
-      {
+      _scrollLine -= _linesPerPage - 1;
+      if(_scrollLine < _firstLineInBuffer + _linesPerPage - 1)
         _scrollLine = _firstLineInBuffer + _linesPerPage - 1;
-        updateScrollBuffer();
-      }
-      else
-        _currentPos = _promptStartPos;
-
-      dirty = true;
+      updateScrollBuffer();
       break;
 
-    case KBDK_END:
-      if (instance().eventHandler().kbdShift(mod))
-      {
+    case Event::UIPgDown:
+      // Don't scroll down when at bottom of buffer
+      if(_scrollLine >= _promptEndPos / _lineWidth)
+        break;
+
+      _scrollLine += _linesPerPage - 1;
+      if(_scrollLine > _promptEndPos / _lineWidth)
         _scrollLine = _promptEndPos / _lineWidth;
-        if (_scrollLine < _linesPerPage - 1)
-          _scrollLine = _linesPerPage - 1;
-        updateScrollBuffer();
-      }
-      else
-        _currentPos = _promptEndPos;
-
-      dirty = true;
+      updateScrollBuffer();
       break;
 
-    case KBDK_UP:
-      if (instance().eventHandler().kbdShift(mod))
-      {
-        if(_scrollLine <= _firstLineInBuffer + _linesPerPage - 1)
-          break;
-
-        _scrollLine -= 1;
-        updateScrollBuffer();
-
-        dirty = true;
-      }
-      else
-        historyScroll(+1);
+    case Event::UIHome:
+      _scrollLine = _firstLineInBuffer + _linesPerPage - 1;
+      updateScrollBuffer();
       break;
 
-    case KBDK_DOWN:
-      if (instance().eventHandler().kbdShift(mod))
-      {
-        // Don't scroll down when at bottom of buffer
-        if(_scrollLine >= _promptEndPos / _lineWidth)
-          break;
-
-        _scrollLine += 1;
-        updateScrollBuffer();
-
-        dirty = true;
-      }
-      else
-        historyScroll(-1);
-      break;
-
-    case KBDK_RIGHT:
-      if (_currentPos < _promptEndPos)
-        _currentPos++;
-
-      dirty = true;
-      break;
-
-    case KBDK_LEFT:
-      if (_currentPos > _promptStartPos)
-        _currentPos--;
-
-      dirty = true;
+    case Event::UIEnd:
+      _scrollLine = _promptEndPos / _lineWidth;
+      if(_scrollLine < _linesPerPage - 1)
+        _scrollLine = _linesPerPage - 1;
+      updateScrollBuffer();
       break;
 
     default:
-      if (instance().eventHandler().kbdControl(mod))
-      {
-        specialKeys(key);
-      }
-      else if (instance().eventHandler().kbdAlt(mod))
-      {
-        // Placeholder only - this will never be reached
-      }
-      else
-        handled = false;
+      handled = false;
+      dirty = false;
       break;
   }
 
@@ -429,19 +324,12 @@ bool PromptWidget::handleKeyDown(StellaKey key, StellaMod mod)
   if(dirty)
     setDirty();
 
-  // There are times when we want the prompt and scrollbar to be marked
-  // as dirty *after* they've been drawn above.  One such occurrence is
-  // when we issue a command that indirectly redraws the entire parent
-  // dialog (such as 'scanline' or 'frame').
-  // In those cases, the return code of the command must be shown, but the
-  // entire dialog contents are redrawn at a later time.  So the prompt and
-  // scrollbar won't be redrawn unless they're dirty again.
-  if(_makeDirty)
-  {
-    setDirty();
-    _scrollBar->setDirty();
-    _makeDirty = false;
-  }
+  // Reset special event handling if input has changed
+  // We assume that non-handled events will modify the input too
+  if(!handled || (resetAutoComplete && changeInput))
+    _tabCount = -1;
+  if(!handled || (resetHistoryScroll && changeInput))
+    _historyLine = 0;
 
   return handled;
 }
@@ -466,42 +354,61 @@ void PromptWidget::insertIntoPrompt(const char* str)
 void PromptWidget::handleCommand(CommandSender* sender, int cmd,
                                  int data, int id)
 {
-  switch (cmd)
+  if(cmd == GuiObject::kSetPositionCmd)
   {
-    case kSetPositionCmd:
-      int newPos = int(data) + _linesPerPage - 1 + _firstLineInBuffer;
-      if (newPos != _scrollLine)
-      {
-        _scrollLine = newPos;
-        setDirty();
-      }
-      break;
+    const int newPos = data + _linesPerPage - 1 + _firstLineInBuffer;
+    if (newPos != _scrollLine)
+    {
+      _scrollLine = newPos;
+      setDirty();
+    }
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PromptWidget::loadConfig()
 {
-  // See logic at the end of handleKeyDown for an explanation of this
-  _makeDirty = true;
-
   // Show the prompt the first time we draw this widget
   if(_firstTime)
   {
+    _firstTime = false;
+
     // Display greetings & prompt
     string version = string("Stella ") + STELLA_VERSION + "\n";
     print(version);
     print(PROMPT);
 
     // Take care of one-time debugger stuff
-    print(instance().debugger().autoExec());
+    // fill the history from the saved breaks, traps and watches commands
+    StringList history;
+    print(instance().debugger().autoExec(&history));
+    for(uInt32 i = 0; i < history.size(); ++i)
+    {
+      addToHistory(history[i].c_str());
+    }
+    history.clear();
     print(instance().debugger().cartDebug().loadConfigFile() + "\n");
     print(instance().debugger().cartDebug().loadListFile() + "\n");
     print(instance().debugger().cartDebug().loadSymbolFile() + "\n");
+
+    bool extra = false;
+    if(instance().settings().getBool("dbg.autosave"))
+    {
+      print(DebuggerParser::inverse(" autoSave enabled "));
+      print("\177 "); // must switch inverse here!
+      extra = true;
+    }
+    if(instance().settings().getBool("dbg.logbreaks"))
+    {
+      print(DebuggerParser::inverse(" logBreaks enabled "));
+      extra = true;
+    }
+    if(extra)
+      print("\n");
+
     print(PROMPT);
 
     _promptStartPos = _promptEndPos = _currentPos;
-    _firstTime = false;
     _exitedEarly = false;
   }
   else if(_exitedEarly)
@@ -514,47 +421,7 @@ void PromptWidget::loadConfig()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int PromptWidget::getWidth() const
 {
-  return _w + kScrollBarWidth;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PromptWidget::specialKeys(StellaKey key)
-{
-  bool handled = true;
-
-  switch(key)
-  {
-    case KBDK_D:
-      killChar(+1);
-      break;
-    case KBDK_K:
-      killLine(+1);
-      break;
-    case KBDK_U:
-      killLine(-1);
-      break;
-    case KBDK_W:
-      killLastWord();
-      break;
-    case KBDK_A:
-      textSelectAll();
-      break;
-    case KBDK_X:
-      textCut();
-      break;
-    case KBDK_C:
-      textCopy();
-      break;
-    case KBDK_V:
-      textPaste();
-      break;
-    default:
-      handled = false;
-      break;
-  }
-
-  if(handled)
-    setDirty();
+  return _w + ScrollBarWidget::scrollBarWidth(_font);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -594,7 +461,7 @@ void PromptWidget::killLine(int direction)
 {
   if(direction == -1)  // erase from current position to beginning of line
   {
-    int count = _currentPos - _promptStartPos;
+    const int count = _currentPos - _promptStartPos;
     if(count > 0)
       for (int i = 0; i < count; i++)
        killChar(-1);
@@ -609,7 +476,7 @@ void PromptWidget::killLine(int direction)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PromptWidget::killLastWord()
+void PromptWidget::killWord()
 {
   int cnt = 0;
   bool space = true;
@@ -635,76 +502,147 @@ void PromptWidget::killLastWord()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PromptWidget::textSelectAll()
+string PromptWidget::getLine()
 {
+#if defined(PSEUDO_CUT_COPY_PASTE)
+  assert(_promptEndPos >= _promptStartPos);
+  string text;
+
+  // Copy current line to text
+  for(int i = _promptStartPos; i < _promptEndPos; i++)
+    text += buffer(i) & 0x7f;
+
+  return text;
+#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PromptWidget::textCut()
 {
+#if defined(PSEUDO_CUT_COPY_PASTE)
+  textCopy();
+
+  // Remove the current line
+  _currentPos = _promptStartPos;
+  killLine(1);  // to end of line
+  _promptEndPos = _currentPos;
+
+  resetFunctions();
+#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PromptWidget::textCopy()
 {
+#if defined(PSEUDO_CUT_COPY_PASTE)
+  string text = getLine();
+
+  instance().eventHandler().copyText(text);
+#endif
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PromptWidget::textPaste()
 {
+#if defined(PSEUDO_CUT_COPY_PASTE)
+  string text;
+
+  // Remove the current line
+  _currentPos = _promptStartPos;
+  killLine(1);  // to end of line
+
+  instance().eventHandler().pasteText(text);
+  print(text);
+  _promptEndPos = _currentPos;
+
+  resetFunctions();
+#endif
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int PromptWidget::historyDir(int& index, int direction)
+{
+  index += direction;
+  if(index < 0)
+    index += static_cast<int>(_history.size());
+  else
+    index %= static_cast<int>(_history.size());
+
+  return index;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PromptWidget::historyAdd(const string& entry)
+{
+  if(_historyIndex >= static_cast<int>(_history.size()))
+    _history.push_back(entry);
+  else
+    _history[_historyIndex] = entry;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PromptWidget::addToHistory(const char* str)
 {
-  strncpy(_history[_historyIndex], str, kLineBufferSize-1);
-  _historyIndex = (_historyIndex + 1) % kHistorySize;
-  _historyLine = 0;
-
-  if (_historySize < kHistorySize)
-    _historySize++;
-}
-
-#if 0 // FIXME
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int PromptWidget::compareHistory(const char *histLine)
-{
-  return 1;
-}
-#endif
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PromptWidget::historyScroll(int direction)
-{
-  if (_historySize == 0)
-    return;
-
-  if (_historyLine == 0 && direction > 0)
+  // Do not add duplicates, remove old duplicate
+  if(_history.size())
   {
-    int i;
-    for (i = 0; i < _promptEndPos - _promptStartPos; i++)
-      _history[_historyIndex][i] = buffer(_promptStartPos + i);
+    int i = _historyIndex;
+    const int historyEnd = _historyIndex % _history.size();
 
-    _history[_historyIndex][i] = '\0';
-  }
+    do
+    {
+      historyDir(i, -1);
 
-  // Advance to the next line in the history
-  int line = _historyLine + direction;
-  if ((direction < 0 && line < 0) || (direction > 0 && line > _historySize))
-    return;
+      if(!BSPF::compareIgnoreCase(_history[i], str))
+      {
+        int j = i, prevJ;
 
-  // If they press arrow-up with anything in the buffer, search backwards
-  // in the history.
-  /*
-  if(direction < 0 && _currentPos > _promptStartPos) {
-    for(;line > 0; line--) {
-      if(compareHistory(_history[line]) == 0)
+        do
+        {
+          prevJ = j;
+          historyDir(j, +1);
+          _history[prevJ] = _history[j];
+        }
+        while(j != historyEnd);
+
+        historyDir(_historyIndex, -1);
         break;
+      }
     }
+    while(i != historyEnd);
   }
-  */
+  historyAdd(str);
+  _historyLine = 0; // reset history scroll
+  _historyIndex = (_historyIndex + 1) % kHistorySize;
+}
 
-  _historyLine = line;
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool PromptWidget::historyScroll(int direction)
+{
+  if(_history.size() == 0)
+    return false;
+
+  // add current input temporarily to history
+  if(_historyLine == 0)
+    historyAdd(getLine());
+
+  // Advance to the next/prev line in the history
+  historyDir(_historyLine, direction);
+
+  // Search the history using the original input
+  do
+  {
+    const int idx = _historyLine
+      ? (_historyIndex - _historyLine + _history.size()) % static_cast<int>(_history.size())
+      : _historyIndex;
+
+    if(BSPF::startsWithIgnoreCase(_history[idx], _history[_historyIndex]))
+      break;
+
+    // Advance to the next/prev line in the history
+    historyDir(_historyLine, direction);
+  }
+  while(_historyLine); // If _historyLine == 0, nothing was found
 
   // Remove the current user text
   _currentPos = _promptStartPos;
@@ -714,21 +652,145 @@ void PromptWidget::historyScroll(int direction)
   scrollToCurrent();
 
   // Print the text from the history
-  int idx;
-  if (_historyLine > 0)
-    idx = (_historyIndex - _historyLine + _historySize) % _historySize;
-  else
-    idx = _historyIndex;
+  const int idx = _historyLine
+    ? (_historyIndex - _historyLine + _history.size()) % static_cast<int>(_history.size())
+    : _historyIndex;
 
-  for (int i = 0; i < kLineBufferSize && _history[idx][i] != '\0'; i++)
+  for(int i = 0; i < kLineBufferSize && _history[idx][i] != '\0'; i++)
     putcharIntern(_history[idx][i]);
-
   _promptEndPos = _currentPos;
 
   // Ensure once more the caret is visible (in case of very long history entries)
   scrollToCurrent();
 
-  setDirty();
+  return true;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool PromptWidget::execute()
+{
+  nextLine();
+
+  assert(_promptEndPos >= _promptStartPos);
+  const int len = _promptEndPos - _promptStartPos;
+
+  if(len > 0)
+  {
+    // Copy the user input to command
+    string command = getLine();
+
+    // Add the input to the history
+    addToHistory(command.c_str());
+
+    // Pass the command to the debugger, and print the result
+    string result = instance().debugger().run(command);
+
+    // This is a bit of a hack
+    // Certain commands remove the debugger dialog from underneath us,
+    // so we shouldn't print any messages
+    // Those commands will return '_EXIT_DEBUGGER' as their result
+    if(result == "_EXIT_DEBUGGER")
+    {
+      _exitedEarly = true;
+      return true;
+    }
+    else if(result == "_NO_PROMPT")
+      return true;
+    else if(result != "")
+      print(result + "\n");
+  }
+  return false;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool PromptWidget::autoComplete(int direction)
+{
+  // Tab completion: we complete either commands or labels, but not
+  // both at once.
+
+  if(_currentPos <= _promptStartPos)
+    return false; // no input
+
+  scrollToCurrent();
+
+  int len = _promptEndPos - _promptStartPos;
+
+  if(_tabCount != -1)
+    len = static_cast<int>(strlen(_inputStr));
+  if(len > kLineBufferSize - 1)
+    len = kLineBufferSize - 1;
+
+  int lastDelimPos = -1;
+  char delimiter = '\0';
+
+  for(int i = 0; i < len; i++)
+  {
+    // copy the input at first tab press only
+    if(_tabCount == -1)
+      _inputStr[i] = buffer(_promptStartPos + i) & 0x7f;
+    // whitespace characters
+    if(strchr("{*@<> =[]()+-/&|!^~%", _inputStr[i]))
+    {
+      lastDelimPos = i;
+      delimiter = _inputStr[i];
+    }
+}
+  if(_tabCount == -1)
+    _inputStr[len] = '\0';
+
+  StringList list;
+
+  if(lastDelimPos == -1)
+    // no delimiters, do only command completion:
+    instance().debugger().parser().getCompletions(_inputStr, list);
+  else
+  {
+    const size_t strLen = len - lastDelimPos - 1;
+    // do not show ALL commands/labels without any filter as it makes no sense
+    if(strLen > 0)
+    {
+      // Special case for 'help' command
+      if(BSPF::startsWithIgnoreCase(_inputStr, "help"))
+        instance().debugger().parser().getCompletions(_inputStr + lastDelimPos + 1, list);
+      else
+      {
+        // we got a delimiter, so this must be a label or a function
+        const Debugger& dbg = instance().debugger();
+
+        dbg.cartDebug().getCompletions(_inputStr + lastDelimPos + 1, list);
+        dbg.getCompletions(_inputStr + lastDelimPos + 1, list);
+      }
+    }
+
+  }
+  if(list.size() < 1)
+    return false;
+  sort(list.begin(), list.end());
+
+  if(direction < 0)
+  {
+    if(--_tabCount < 0)
+      _tabCount = static_cast<int>(list.size()) - 1;
+  }
+  else
+    _tabCount = (_tabCount + 1) % list.size();
+
+  nextLine();
+  _currentPos = _promptStartPos;
+  killLine(1);  // kill whole line
+
+  // start with-autocompleted, fixed string...
+  for(int i = 0; i < lastDelimPos; i++)
+    putcharIntern(_inputStr[i]);
+  if(lastDelimPos > 0)
+    putcharIntern(delimiter);
+
+  // ...and add current autocompletion string
+  print(list[_tabCount]);
+  putcharIntern(' ');
+  _promptEndPos = _currentPos;
+
+  return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -738,7 +800,7 @@ void PromptWidget::nextLine()
   _textcolor = kTextColor;
   _inverse = false;
 
-  int line = _currentPos / _lineWidth;
+  const int line = _currentPos / _lineWidth;
   if (line == _scrollLine)
     _scrollLine++;
 
@@ -747,15 +809,14 @@ void PromptWidget::nextLine()
   updateScrollBuffer();
 }
 
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // Call this (at least) when the current line changes or when a new line is added
 void PromptWidget::updateScrollBuffer()
 {
-  int lastchar = std::max(_promptEndPos, _currentPos);
-  int line = lastchar / _lineWidth;
-  int numlines = (line < _linesInBuffer) ? line + 1 : _linesInBuffer;
-  int firstline = line - numlines + 1;
+  const int lastchar = std::max(_promptEndPos, _currentPos),
+            line = lastchar / _lineWidth,
+            numlines = (line < _linesInBuffer) ? line + 1 : _linesInBuffer,
+            firstline = line - numlines + 1;
 
   if (firstline > _firstLineInBuffer)
   {
@@ -773,12 +834,13 @@ void PromptWidget::updateScrollBuffer()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-int PromptWidget::printf(const char* format, ...)
+// TODO: rewrite this (cert-dcl50-cpp)
+int PromptWidget::printf(const char* format, ...)  // NOLINT
 {
   va_list argptr;
 
   va_start(argptr, format);
-  int count = this->vprintf(format, argptr);
+  const int count = this->vprintf(format, argptr);
   va_end (argptr);
   return count;
 }
@@ -786,8 +848,8 @@ int PromptWidget::printf(const char* format, ...)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int PromptWidget::vprintf(const char* format, va_list argptr)
 {
-  char buf[2048];  // Note: generates warnings about 'nonliteral' format
-  int count = std::vsnprintf(buf, sizeof(buf), format, argptr);
+  char buf[2048];  // NOLINT  (will be rewritten soon)
+  const int count = std::vsnprintf(buf, sizeof(buf), format, argptr);
 
   print(buf);
   return count;
@@ -800,13 +862,11 @@ void PromptWidget::putcharIntern(int c)
     nextLine();
   else if(c & 0x80) { // set foreground color to TIA color
                       // don't print or advance cursor
-                      // there are only 128 TIA colors, but
-                      // OverlayColor contains 256 of them
-    _textcolor = (c & 0x7f) << 1;
+    _textcolor = static_cast<ColorId>((c & 0x7f) << 1);
   }
   else if(c && c < 0x1e) { // first actual character is large dash
     // More colors (the regular GUI ones)
-    _textcolor = c + 0x100;
+    _textcolor = static_cast<ColorId>(c + 0x100);
   }
   else if(c == 0x7f) { // toggle inverse video (DEL char)
     _inverse = !_inverse;
@@ -827,7 +887,7 @@ void PromptWidget::putcharIntern(int c)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PromptWidget::print(const string& str)
 {
-  for(char c: str)
+  for(const auto c: str)
     putcharIntern(c);
 }
 
@@ -836,18 +896,17 @@ void PromptWidget::drawCaret()
 {
 //cerr << "PromptWidget::drawCaret()\n";
   FBSurface& s = _boss->dialog().surface();
-
-  int line = _currentPos / _lineWidth;
+  const int line = _currentPos / _lineWidth;
 
   // Don't draw the cursor if it's not in the current view
   if(_scrollLine < line)
     return;
 
-  int displayLine = line - _scrollLine + _linesPerPage - 1;
-  int x = _x + 1 + (_currentPos % _lineWidth) * _kConsoleCharWidth;
-  int y = _y + displayLine * _kConsoleLineHeight;
+  const int displayLine = line - _scrollLine + _linesPerPage - 1,
+                          x = _x + 1 + (_currentPos % _lineWidth) * _kConsoleCharWidth,
+                          y = _y + displayLine * _kConsoleLineHeight;
 
-  char c = buffer(_currentPos);
+  const char c = buffer(_currentPos); //FIXME: int to char??
   s.fillRect(x, y, _kConsoleCharWidth, _kConsoleLineHeight, kTextColor);
   s.drawChar(_font, c, x, y + 2, kBGColor);
 }
@@ -855,7 +914,7 @@ void PromptWidget::drawCaret()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PromptWidget::scrollToCurrent()
 {
-  int line = _promptEndPos / _lineWidth;
+  const int line = _promptEndPos / _lineWidth;
 
   if (line + _linesPerPage <= _scrollLine)
   {
@@ -869,74 +928,62 @@ void PromptWidget::scrollToCurrent()
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool PromptWidget::saveBuffer(string& filename)
+string PromptWidget::saveBuffer(const FilesystemNode& file)
 {
-  ofstream out(filename);
-  if(!out.is_open())
-    return false;
-
-  for(int start=0; start<_promptStartPos; start+=_lineWidth)
+  stringstream out;
+  for(int start = 0; start < _promptStartPos; start += _lineWidth)
   {
-    int end = start+_lineWidth-1;
+    int end = start + _lineWidth - 1;
 
-    // look for first non-space, printing char from end of line
-    while( char(_buffer[end] & 0xff) <= ' ' && end >= start)
+    // Look for first non-space, printing char from end of line
+    while(static_cast<char>(_buffer[end] & 0xff) <= ' ' && end >= start)
       end--;
 
-    // spit out the line minus its trailing junk.
+    // Spit out the line minus its trailing junk
     // Strip off any color/inverse bits
     for(int j = start; j <= end; ++j)
-      out << char(_buffer[j] & 0xff);
+      out << static_cast<char>(_buffer[j] & 0xff);
 
     // add a \n
     out << endl;
   }
 
-  return true;
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-string PromptWidget::getCompletionPrefix(const StringList& completions, string prefix)
-{
-  // Search for prefix in every string, progressively growing it
-  // Once a mismatch is found or length is past one of the strings, we're done
-  // We *could* use the longest common string algorithm, but for the lengths
-  // of the strings we're dealing with, it's probably not worth it
-  for(;;)
-  {
-    for(const auto& s: completions)
-    {
-      if(s.length() < prefix.length())
-        return prefix;  // current prefix is the best we're going to get
-      else if(!BSPF::startsWithIgnoreCase(s, prefix))
-      {
-        prefix.erase(prefix.length()-1);
-        return prefix;
-      }
-    }
-    if(completions[0].length() > prefix.length())
-      prefix = completions[0].substr(0, prefix.length() + 1);
-    else
-      return prefix;
+  try {
+    if(file.write(out) > 0)
+      return "saved " + file.getShortPath() + " OK";
   }
+  catch(...) { }
+
+  return "unable to save session";
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PromptWidget::clearScreen()
 {
-  // Initialize start position and history
+  // Initialize start position
   _currentPos = 0;
   _scrollLine = _linesPerPage - 1;
   _firstLineInBuffer = 0;
   _promptStartPos = _promptEndPos = -1;
   memset(_buffer, 0, kBufferSize * sizeof(int));
 
-  _historyIndex = 0;
-  _historyLine = 0;
-  _historySize = 0;
-  for (int i = 0; i < kHistorySize; i++)
-    _history[i][0] = '\0';
-
   if(!_firstTime)
     updateScrollBuffer();
+
+  resetFunctions();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PromptWidget::clearHistory()
+{
+  _history.clear();
+  _historyIndex = 0;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void PromptWidget::resetFunctions()
+{
+  // reset special functions
+  _tabCount = -1;
+  _historyLine = 0;
 }
