@@ -1,27 +1,30 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: RomListWidget.cxx 3304 2016-04-03 00:35:00Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
+#include "OSystem.hxx"
 #include "Debugger.hxx"
 #include "DiStella.hxx"
-#include "PackedBitArray.hxx"
 #include "Widget.hxx"
+#include "Dialog.hxx"
+#include "ToolTip.hxx"
+#include "StellaKeys.hxx"
+#include "FBSurface.hxx"
+#include "Font.hxx"
 #include "ScrollBarWidget.hxx"
 #include "RomListSettings.hxx"
 #include "RomListWidget.hxx"
@@ -30,64 +33,58 @@
 RomListWidget::RomListWidget(GuiObject* boss, const GUI::Font& lfont,
                              const GUI::Font& nfont,
                              int x, int y, int w, int h)
-  : EditableWidget(boss, nfont, x, y, 16, 16),
-    _rows(0),
-    _cols(0),
-    _currentPos(0),
-    _selectedItem(-1),
-    _highlightedItem(-1),
-    _editMode(false),
-    _currentKeyDown(KBDK_UNKNOWN),
-    _base(Common::Base::F_DEFAULT),
-    myDisasm(nullptr),
-    myBPState(nullptr)
+  : EditableWidget(boss, nfont, x, y, 16, 16)
 {
-  _flags = WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS;
+  _flags = Widget::FLAG_ENABLED | Widget::FLAG_CLEARBG | Widget::FLAG_RETAIN_FOCUS;
   _bgcolor = kWidColor;
   _bgcolorhi = kWidColor;
   _textcolor = kTextColor;
   _textcolorhi = kTextColor;
 
+  _editMode = false;
+  _dyText = -1; // fixes the vertical position of selected text
+
   _cols = w / _fontWidth;
-  _rows = h / _fontHeight;
+  _rows = h / _lineHeight;
 
   // Set real dimensions
-  _w = w - kScrollBarWidth;
+  _w = w - ScrollBarWidget::scrollBarWidth(_font);
   _h = h + 2;
 
   // Create scrollbar and attach to the list
-  myScrollBar = new ScrollBarWidget(boss, lfont, _x + _w, _y, kScrollBarWidth, _h);
+  myScrollBar = new ScrollBarWidget(boss, lfont, _x + _w, _y,
+                                    ScrollBarWidget::scrollBarWidth(_font), _h);
   myScrollBar->setTarget(this);
 
   // Add settings menu
-  myMenu = make_ptr<RomListSettings>(this, lfont);
+  myMenu = make_unique<RomListSettings>(this, lfont);
 
   // Take advantage of a wide debugger window when possible
   const int fontWidth = lfont.getMaxCharWidth(),
             numchars = w / fontWidth;
 
-  _labelWidth = std::max(16, int(0.20 * (numchars - 12))) * fontWidth - 1;
-  _bytesWidth = 12 * fontWidth;
+  _labelWidth = std::max(14, int(0.45 * (numchars - 8 - 8 - 9 - 2))) * fontWidth - 1;
+  _bytesWidth = 9 * fontWidth;
 
-  //////////////////////////////////////////////////////
+  ///////////////////////////////////////////////////////
   // Add checkboxes
   int ypos = _y + 2;
 
   // rowheight is determined by largest item on a line,
   // possibly meaning that number of rows will change
-  _fontHeight = std::max(_fontHeight, CheckboxWidget::boxSize());
-  _rows = h / _fontHeight;
+  _lineHeight = std::max(_lineHeight, CheckboxWidget::boxSize(_font));
+  _rows = h / _lineHeight;
 
   // Create a CheckboxWidget for each row in the list
-  CheckboxWidget* t = nullptr;
   for(int i = 0; i < _rows; ++i)
   {
-    t = new CheckboxWidget(boss, lfont, _x + 2, ypos, "", kCheckActionCmd);
+    CheckboxWidget* t = new CheckboxWidget(boss, lfont, _x + 2, ypos, "",
+        CheckboxWidget::kCheckActionCmd);
     t->setTarget(this);
     t->setID(i);
-    t->setFill(CheckboxWidget::Circle);
+    t->setFill(CheckboxWidget::FillType::Circle);
     t->setTextColor(kTextColorEm);
-    ypos += _fontHeight;
+    ypos += _lineHeight;
 
     myCheckList.push_back(t);
   }
@@ -97,44 +94,42 @@ RomListWidget::RomListWidget(GuiObject* boss, const GUI::Font& lfont,
   {
     switch(_base)
     {
-      case Common::Base::F_16:
-      case Common::Base::F_16_1:
-      case Common::Base::F_16_2:
-      case Common::Base::F_16_4:
-      case Common::Base::F_16_8:
+      case Common::Base::Fmt::_16:
+      case Common::Base::Fmt::_16_1:
+      case Common::Base::Fmt::_16_2:
+      case Common::Base::Fmt::_16_4:
+      case Common::Base::Fmt::_16_8:
         return (c >= '0' && c <= '9') || (c >= 'a' && c <= 'f') || c == ' ';
 
-      case Common::Base::F_2:
-      case Common::Base::F_2_8:
-      case Common::Base::F_2_16:
+      case Common::Base::Fmt::_2:
+      case Common::Base::Fmt::_2_8:
+      case Common::Base::Fmt::_2_16:
         return c == '0' || c == '1' || c == ' ';
 
-      case Common::Base::F_10:
+      case Common::Base::Fmt::_10:
         return (c >= '0' && c <= '9') || c == ' ';
 
-      case Common::Base::F_DEFAULT:
+      case Common::Base::Fmt::_DEFAULT:
+      default:  // TODO - properly handle all other cases
         return false;
     }
-    return false;
   };
   setTextFilter(f);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void RomListWidget::setList(const CartDebug::Disassembly& disasm,
-                            const PackedBitArray& state)
+void RomListWidget::setList(const CartDebug::Disassembly& disasm)
 {
   myDisasm = &disasm;
-  myBPState = &state;
 
   // Enable all checkboxes
   for(int i = 0; i < _rows; ++i)
-    myCheckList[i]->setFlags(WIDGET_ENABLED);
+    myCheckList[i]->setFlags(Widget::FLAG_ENABLED);
 
   // Then turn off any extras
-  if(int(myDisasm->list.size()) < _rows)
-    for(int i = int(myDisasm->list.size()); i < _rows; ++i)
-      myCheckList[i]->clearFlags(WIDGET_ENABLED);
+  if(static_cast<int>(myDisasm->list.size()) < _rows)
+    for(int i = static_cast<int>(myDisasm->list.size()); i < _rows; ++i)
+      myCheckList[i]->clearFlags(Widget::FLAG_ENABLED);
 
   recalc();
 }
@@ -142,7 +137,7 @@ void RomListWidget::setList(const CartDebug::Disassembly& disasm,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomListWidget::setSelected(int item)
 {
-  if(item < -1 || item >= int(myDisasm->list.size()))
+  if(item < -1 || item >= static_cast<int>(myDisasm->list.size()))
     return;
 
   if(isEnabled())
@@ -158,7 +153,7 @@ void RomListWidget::setSelected(int item)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomListWidget::setHighlighted(int item)
 {
-  if(item < -1 || item >= int(myDisasm->list.size()))
+  if(item < -1 || item >= static_cast<int>(myDisasm->list.size()))
     return;
 
   if(isEnabled())
@@ -169,8 +164,12 @@ void RomListWidget::setHighlighted(int item)
     _highlightedItem = item;
 
     // Only scroll the list if we're about to pass the page boundary
-    if(_currentPos == 0)
-      _currentPos = _highlightedItem;
+    if (_highlightedItem < _currentPos)
+    {
+      _currentPos -= _rows;
+      if (_currentPos < 0)
+        _currentPos = 0;
+    }
     else if(_highlightedItem == _currentPos + _rows)
       _currentPos += _rows;
 
@@ -181,13 +180,13 @@ void RomListWidget::setHighlighted(int item)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int RomListWidget::findItem(int x, int y) const
 {
-  return (y - 1) / _fontHeight + _currentPos;
+  return (y - 1) / _lineHeight + _currentPos;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomListWidget::recalc()
 {
-  int size = int(myDisasm->list.size());
+  const int size = static_cast<int>(myDisasm->list.size());
 
   if (_currentPos >= size)
     _currentPos = size - 1;
@@ -199,7 +198,7 @@ void RomListWidget::recalc()
 
   _editMode = false;
 
-  myScrollBar->_numEntries     = int(myDisasm->list.size());
+  myScrollBar->_numEntries     = static_cast<int>(myDisasm->list.size());
   myScrollBar->_entriesPerPage = _rows;
 
   // Reset to normal data entry
@@ -223,7 +222,7 @@ void RomListWidget::scrollToCurrent(int item)
     _currentPos = item - _rows + 1;
   }
 
-  int size = int(myDisasm->list.size());
+  const int size = static_cast<int>(myDisasm->list.size());
   if (_currentPos < 0 || _rows > size)
     _currentPos = 0;
   else if (_currentPos + _rows > size)
@@ -236,25 +235,28 @@ void RomListWidget::scrollToCurrent(int item)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void RomListWidget::handleMouseDown(int x, int y, int button, int clickCount)
+void RomListWidget::handleMouseDown(int x, int y, MouseButton b, int clickCount)
 {
   if (!isEnabled())
     return;
 
+  resetSelection();
   // Grab right mouse button for context menu, left for selection/edit mode
-  if(button == 2)
+  if(b == MouseButton::RIGHT)
   {
     // Set selected and add menu at current x,y mouse location
     _selectedItem = findItem(x, y);
     scrollToSelected();
-    myMenu->show(x + getAbsX(), y + getAbsY(), _selectedItem);
+    dialog().tooltip().hide();
+    myMenu->show(x + getAbsX(), y + getAbsY(),
+                 dialog().surface().dstRect(), _selectedItem);
   }
   else
   {
     // First check whether the selection changed
     int newSelectedItem;
     newSelectedItem = findItem(x, y);
-    if (newSelectedItem > int(myDisasm->list.size()) - 1)
+    if (newSelectedItem > static_cast<int>(myDisasm->list.size()) - 1)
       newSelectedItem = -1;
 
     if (_selectedItem != newSelectedItem)
@@ -268,7 +270,7 @@ void RomListWidget::handleMouseDown(int x, int y, int button, int clickCount)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void RomListWidget::handleMouseUp(int x, int y, int button, int clickCount)
+void RomListWidget::handleMouseUp(int x, int y, MouseButton b, int clickCount)
 {
   // If this was a double click and the mouse is still over the selected item,
   // send the double click command
@@ -301,11 +303,11 @@ bool RomListWidget::handleText(char text)
 bool RomListWidget::handleKeyDown(StellaKey key, StellaMod mod)
 {
   // Ignore all Alt-mod keys
-  if(instance().eventHandler().kbdAlt(mod))
-    return true;
+  if(StellaModTest::isAlt(mod))
+    return false;
 
   bool handled = true;
-  int oldSelectedItem = _selectedItem;
+  const int oldSelectedItem = _selectedItem;
 
   if (_editMode)
   {
@@ -355,7 +357,7 @@ bool RomListWidget::handleEvent(Event::Type e)
     return false;
 
   bool handled = true;
-  int oldSelectedItem = _selectedItem;
+  const int oldSelectedItem = _selectedItem;
 
   switch(e)
   {
@@ -373,7 +375,7 @@ bool RomListWidget::handleEvent(Event::Type e)
       break;
 
     case Event::UIDown:
-      if (_selectedItem < int(myDisasm->list.size()) - 1)
+      if (_selectedItem < static_cast<int>(myDisasm->list.size()) - 1)
         _selectedItem++;
       break;
 
@@ -385,8 +387,8 @@ bool RomListWidget::handleEvent(Event::Type e)
 
     case Event::UIPgDown:
       _selectedItem += _rows - 1;
-      if (_selectedItem >= int(myDisasm->list.size()))
-        _selectedItem = int(myDisasm->list.size()) - 1;
+      if (_selectedItem >= static_cast<int>(myDisasm->list.size()))
+        _selectedItem = static_cast<int>(myDisasm->list.size()) - 1;
       break;
 
     case Event::UIHome:
@@ -394,7 +396,7 @@ bool RomListWidget::handleEvent(Event::Type e)
       break;
 
     case Event::UIEnd:
-      _selectedItem = int(myDisasm->list.size()) - 1;
+      _selectedItem = static_cast<int>(myDisasm->list.size()) - 1;
       break;
 
     default:
@@ -415,14 +417,13 @@ void RomListWidget::handleCommand(CommandSender* sender, int cmd, int data, int
 {
   switch (cmd)
   {
-    case kCheckActionCmd:
+    case CheckboxWidget::kCheckActionCmd:
       // We let the parent class handle this
       // Pass it as a kRLBreakpointChangedCmd command, since that's the intent
-      sendCommand(RomListWidget::kBPointChangedCmd, _currentPos+id,
-                  myCheckList[id]->getState());
+      sendCommand(RomListWidget::kBPointChangedCmd, _currentPos+id, 0);
       break;
 
-    case kSetPositionCmd:
+    case GuiObject::kSetPositionCmd:
       if (_currentPos != data)
       {
         _currentPos = data;
@@ -445,91 +446,177 @@ void RomListWidget::lostFocusWidget()
   abortEditMode();
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Common::Point RomListWidget::getToolTipIndex(const Common::Point& pos) const
+{
+  const Common::Rect& r = getEditRect();
+  const int col = (pos.x - r.x() - getAbsX()) / _font.getMaxCharWidth();
+  const int row = (pos.y - getAbsY()) / _lineHeight;
+
+  if(col < 0 || col >= 8
+     || row < 0 || row + _currentPos >= static_cast<int>(myDisasm->list.size()))
+    return Common::Point(-1, -1);
+  else
+    return Common::Point(col, row + _currentPos);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+string RomListWidget::getToolTip(const Common::Point& pos) const
+{
+  const Common::Point& idx = getToolTipIndex(pos);
+
+  if(idx.y < 0)
+    return EmptyString;
+
+  const string bytes = myDisasm->list[idx.y].bytes;
+
+  if(static_cast<Int32>(bytes.length()) < idx.x + 1)
+    return EmptyString;
+
+  Int32 val = 0;
+  if(bytes.length() == 8 && bytes[2] != ' ')
+  {
+    // Binary value
+    val = static_cast<Int32>(stol(bytes, nullptr, 2));
+  }
+  else
+  {
+    // 1..3 hex values
+    if(idx.x == 2)
+      // Skip gap after first byte
+      return EmptyString;
+
+    string valStr;
+
+    if(idx.x < 2 || bytes.length() < 8)
+      // 1 or 2 hex bytes, get one hex byte
+      valStr = bytes.substr((idx.x / 3) * 3, 2);
+    else
+      // 3 hex bytes, get two rightmost hex bytes
+      valStr = bytes.substr(6, 2) + bytes.substr(3, 2);
+
+    val = static_cast<Int32>(stol(valStr, nullptr, 16));
+  }
+  ostringstream buf;
+
+  buf << _toolTipText
+    << "$" << Common::Base::toString(val, Common::Base::Fmt::_16)
+    << " = #" << val;
+  if(val < 0x100)
+  {
+    if(val >= 0x80)
+      buf << '/' << -(0x100 - val);
+    buf << " = %" << Common::Base::toString(val, Common::Base::Fmt::_2);
+  }
+
+  return buf.str();
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool RomListWidget::changedToolTip(const Common::Point& oldPos,
+                                   const Common::Point& newPos) const
+{
+  return getToolTipIndex(oldPos) != getToolTipIndex(newPos);
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void RomListWidget::drawWidget(bool hilite)
 {
   FBSurface& s = _boss->dialog().surface();
   const CartDebug::DisassemblyList& dlist = myDisasm->list;
-  int i, pos, xpos, ypos, len = int(dlist.size());
+  constexpr ColorId textColor = kTextColor;
 
-  const GUI::Rect& r = getEditRect();
-  const GUI::Rect& l = getLineRect();
+  const Common::Rect& r = getEditRect();
+  const Common::Rect& l = getLineRect();
 
   // Draw a thin frame around the list and to separate columns
-  s.hLine(_x, _y, _x + _w - 1, kColor);
-  s.hLine(_x, _y + _h - 1, _x + _w - 1, kShadowColor);
-  s.vLine(_x, _y, _y + _h - 1, kColor);
-  s.vLine(_x + CheckboxWidget::boxSize() + 5, _y, _y + _h - 1, kColor);
+  s.frameRect(_x, _y, _w + 1, _h, hilite ? kWidColorHi : kColor);
+  s.vLine(_x + CheckboxWidget::boxSize(_font) + 5, _y, _y + _h - 1, kColor);
 
   // Draw the list items
-  int ccountw = _fontWidth << 2,
-      large_disasmw = _w - l.x() - _labelWidth,
-      medium_disasmw = large_disasmw - r.width(),
-      small_disasmw = medium_disasmw - (ccountw << 1),
-      actualwidth = myDisasm->fieldwidth * _fontWidth;
-  if(actualwidth < small_disasmw)
-    small_disasmw = actualwidth;
+  const int cycleCountW = _fontWidth * 8,
+      noTypeDisasmW = _w - l.x() - _labelWidth,
+      noCodeDisasmW = noTypeDisasmW - r.w(),
+      actualWidth = myDisasm->fieldwidth * _fontWidth,
+      codeDisasmW = std::min(actualWidth, noCodeDisasmW - cycleCountW);
 
-  xpos = _x + CheckboxWidget::boxSize() + 10;  ypos = _y + 2;
-  for (i = 0, pos = _currentPos; i < _rows && pos < len; i++, pos++, ypos += _fontHeight)
+  const int len = static_cast<int>(dlist.size());
+  const int xpos = _x + CheckboxWidget::boxSize(_font) + 10;
+  int ypos = _y + 2;
+  for(int i = 0, pos = _currentPos; i < _rows && pos < len; i++, pos++, ypos += _lineHeight)
   {
-    // Draw checkboxes for correct lines (takes scrolling into account)
-    myCheckList[i]->setState(myBPState->isSet(dlist[pos].address));
+    ColorId bytesColor = textColor;
+
+    // Mark checkboxes dirty for correct lines (takes scrolling into account)
+    myCheckList[i]->setState(instance().debugger().
+                             checkBreakPoint(dlist[pos].address,
+                             instance().debugger().cartDebug().getBank(dlist[pos].address)));
     myCheckList[i]->setDirty();
+    // All checkboxes have to be redrawn because RomListWidget clears its whole area
+    // Also draw immediately, because chain order is not deterministic
     myCheckList[i]->draw();
 
     // Draw highlighted item in a frame
-    if (_highlightedItem == pos)
-      s.frameRect(_x + l.x() - 3, ypos - 1, _w - l.x(), _fontHeight, kTextColorHi);
+    if(_highlightedItem == pos)
+      s.frameRect(_x + l.x() - 3, ypos - 1, _w - l.x(), _lineHeight, kWidColorHi);
 
     // Draw the selected item inverted, on a highlighted background.
-    if (_selectedItem == pos && _hasFocus)
+    if(_selectedItem == pos && _hasFocus)
     {
-      if (!_editMode)
-        s.fillRect(_x + r.x() - 3, ypos - 1, r.width(), _fontHeight, kTextColorHi);
+      if(!_editMode)
+      {
+        s.fillRect(_x + r.x() - 3, ypos - 1, r.w(), _lineHeight, kTextColorHi);
+        bytesColor = kTextColorInv;
+      }
       else
-        s.frameRect(_x + r.x() - 3, ypos - 1, r.width(), _fontHeight, kTextColorHi);
+        s.frameRect(_x + r.x() - 3, ypos - 1, r.w(), _lineHeight, kWidColorHi);
     }
 
     // Draw labels
     s.drawString(_font, dlist[pos].label, xpos, ypos, _labelWidth,
-                 dlist[pos].hllabel ? kTextColor : kColor);
+                  dlist[pos].hllabel ? textColor : kColor);
 
     // Bytes are only editable if they represent code, graphics, or accessible data
     // Otherwise, the disassembly should get all remaining space
-    if(dlist[pos].type & (CartDebug::CODE|CartDebug::GFX|CartDebug::PGFX|CartDebug::DATA))
+    if(dlist[pos].type & (Device::CODE | Device::GFX | Device::PGFX |
+       Device::COL | Device::PCOL | Device::BCOL | Device::DATA))
     {
-      if(dlist[pos].type == CartDebug::CODE)
+      if(dlist[pos].type == Device::CODE)
       {
-        // Draw disassembly and cycle count
-        s.drawString(_font, dlist[pos].disasm, xpos + _labelWidth, ypos,
-                     small_disasmw, kTextColor);
-        s.drawString(_font, dlist[pos].ccount, xpos + _labelWidth + small_disasmw, ypos,
-                     ccountw, kTextColor);
+        // Draw mnemonic
+        s.drawString(_font, dlist[pos].disasm.substr(0, 7), xpos + _labelWidth, ypos,
+                      7 * _fontWidth, textColor);
+        // Draw operand
+        if(dlist[pos].disasm.length() > 8)
+          s.drawString(_font, dlist[pos].disasm.substr(8), xpos + _labelWidth + 7 * _fontWidth, ypos,
+                        codeDisasmW - 7 * _fontWidth, textColor);
+        // Draw cycle count
+        s.drawString(_font, dlist[pos].ccount, xpos + _labelWidth + codeDisasmW, ypos,
+                      cycleCountW, textColor);
       }
       else
       {
         // Draw disassembly only
         s.drawString(_font, dlist[pos].disasm, xpos + _labelWidth, ypos,
-                     medium_disasmw, kTextColor);
+                      noCodeDisasmW - 4, kTextColor);
       }
 
       // Draw separator
-      s.vLine(_x + r.x() - 7, ypos, ypos + _fontHeight - 1, kColor);
+      s.vLine(_x + r.x() - 7, ypos, ypos + _lineHeight - 1, kColor);
 
       // Draw bytes
       {
-        if (_selectedItem == pos && _editMode)
+        if(_selectedItem == pos && _editMode)
         {
           adjustOffset();
-          s.drawString(_font, editString(), _x + r.x(), ypos, r.width(), kTextColor,
-                       kTextAlignLeft, -_editScrollOffset, false);
+          s.drawString(_font, editString(), _x + r.x(), ypos, r.w(), textColor,
+                        TextAlign::Left, -_editScrollOffset, false);
 
-          drawCaret();
+          drawCaretSelection();
         }
         else
         {
-          s.drawString(_font, dlist[pos].bytes, _x + r.x(), ypos, r.width(), kTextColor);
+          s.drawString(_font, dlist[pos].bytes, _x + r.x(), ypos, r.w(), bytesColor);
         }
       }
     }
@@ -537,36 +624,28 @@ void RomListWidget::drawWidget(bool hilite)
     {
       // Draw disassembly, giving it all remaining horizontal space
       s.drawString(_font, dlist[pos].disasm, xpos + _labelWidth, ypos,
-                   large_disasmw, kTextColor);
+                    noTypeDisasmW, textColor);
     }
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect RomListWidget::getLineRect() const
+Common::Rect RomListWidget::getLineRect() const
 {
-  GUI::Rect r(2, 1, _w, _fontHeight);
-  const int yoffset = (_selectedItem - _currentPos) * _fontHeight,
-            xoffset = CheckboxWidget::boxSize() + 10;
-  r.top    += yoffset;
-  r.bottom += yoffset;
-  r.left   += xoffset;
-  r.right  -= xoffset - 15;
-	
-  return r;
+  const int yoffset = std::max(0, (_selectedItem - _currentPos) * _lineHeight),
+            xoffset = CheckboxWidget::boxSize(_font) + 10;
+
+  return Common::Rect(2 + xoffset, 1 + yoffset,
+                      _w - (xoffset - 15), _lineHeight + yoffset);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-GUI::Rect RomListWidget::getEditRect() const
+Common::Rect RomListWidget::getEditRect() const
 {
-  GUI::Rect r(2, 1, _w, _fontHeight);
-  const int yoffset = (_selectedItem - _currentPos) * _fontHeight;
-  r.top    += yoffset;
-  r.bottom += yoffset;
-  r.left   += _w - _bytesWidth;
-  r.right   = _w;
-	
-  return r;
+  const int yoffset = std::max(0, (_selectedItem - _currentPos) * _lineHeight);
+
+  return Common::Rect(2 + _w - _bytesWidth, 1 + yoffset + 1,
+                      _w, _lineHeight + yoffset + 1);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -579,11 +658,12 @@ void RomListWidget::startEditMode()
       return;
 
     _editMode = true;
+    dialog().tooltip().hide();
     switch(myDisasm->list[_selectedItem].type)
     {
-      case CartDebug::GFX:
-      case CartDebug::PGFX:
-        _base = DiStella::settings.gfx_format;
+      case Device::GFX:
+      case Device::PGFX:
+        _base = DiStella::settings.gfxFormat;
         break;
       default:
         _base = Common::Base::format();
@@ -604,7 +684,7 @@ void RomListWidget::endEditMode()
   // Send a message that editing finished with a return/enter key press
   // The parent then calls getText() to get the newly entered data
   _editMode = false;
-  sendCommand(RomListWidget::kRomChangedCmd, _selectedItem, _base);
+  sendCommand(RomListWidget::kRomChangedCmd, _selectedItem, static_cast<int>(_base));
 
   // Reset to normal data entry
   EditableWidget::endEditMode();
