@@ -1,20 +1,18 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: Cart4A50.cxx 3316 2016-08-24 23:57:07Z stephena $
 //============================================================================
 
 #include "System.hxx"
@@ -23,41 +21,34 @@
 #include "Cart4A50.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Cartridge4A50::Cartridge4A50(const uInt8* image, uInt32 size,
-                             const Settings& settings)
-  : Cartridge(settings),
-    mySize(size),
-    mySliceLow(0),
-    mySliceMiddle(0),
-    mySliceHigh(0),
-    myIsRomLow(true),
-    myIsRomMiddle(true),
-    myIsRomHigh(true),
-    myLastAddress(0),
-    myLastData(0)
+Cartridge4A50::Cartridge4A50(const ByteBuffer& image, size_t size,
+                             const string& md5, const Settings& settings)
+  : Cartridge(settings, md5),
+    myImage{make_unique<uInt8[]>(128_KB)},
+    mySize{size}
 {
   // Copy the ROM image into my buffer
   // Supported file sizes are 32/64/128K, which are duplicated if necessary
-  if(size < 65536)        size = 32768;
-  else if(size < 131072)  size = 65536;
-  else                    size = 131072;
-  for(uInt32 slice = 0; slice < 131072 / size; ++slice)
-    memcpy(myImage + (slice*size), image, size);
+  if(size < 64_KB)        size = 32_KB;
+  else if(size < 128_KB)  size = 64_KB;
+  else                    size = 128_KB;
+  for(uInt32 slice = 0; slice < 128_KB / size; ++slice)
+    std::copy_n(image.get(), size, myImage.get() + (slice*size));
 
-  // We use System::PageAccess.codeAccessBase, but don't allow its use
+  // We use System::PageAccess.romAccessBase, but don't allow its use
   // through a pointer, since the address space of 4A50 carts can change
   // at the instruction level, and PageAccess is normally defined at an
   // interval of 64 bytes
   //
   // Instead, access will be through the getAccessFlags and setAccessFlags
   // methods below
-  createCodeAccessBase(131072 + 32768);
+  createRomAccessArrays(128_KB + myRAM.size());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge4A50::reset()
 {
-  initializeRAM(myRAM, 32768);
+  initializeRAM(myRAM.data(), myRAM.size());
 
   mySliceLow = mySliceMiddle = mySliceHigh = 0;
   myIsRomLow = myIsRomMiddle = myIsRomHigh = true;
@@ -74,14 +65,14 @@ void Cartridge4A50::install(System& system)
   mySystem = &system;
 
   // Map all of the accesses to call peek and poke (We don't yet indicate RAM areas)
-  System::PageAccess access(this, System::PA_READ);
-  for(uInt32 i = 0x1000; i < 0x2000; i += (1 << System::PAGE_SHIFT))
-    mySystem->setPageAccess(i >> System::PAGE_SHIFT, access);
+  const System::PageAccess access(this, System::PageAccessType::READ);
+  for(uInt16 addr = 0x1000; addr < 0x2000; addr += System::PAGE_SIZE)
+    mySystem->setPageAccess(addr, access);
 
   // Mirror all access in TIA and RIOT; by doing so we're taking responsibility
   // for that address space in peek and poke below.
-  mySystem->tia().install(system, *this);
-  mySystem->m6532().install(system, *this);
+  mySystem->tia().installDelegate(system, *this);
+  mySystem->m6532().installDelegate(system, *this);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -92,7 +83,7 @@ uInt8 Cartridge4A50::peek(uInt16 address)
   if(!(address & 0x1000))                      // Hotspots below 0x1000
   {
     // Check for RAM or TIA mirroring
-    uInt16 lowAddress = address & 0x3ff;
+    const uInt16 lowAddress = address & 0x3ff;
     if(lowAddress & 0x80)
       value = mySystem->m6532().peek(address);
     else if(!(lowAddress & 0x200))
@@ -121,7 +112,7 @@ uInt8 Cartridge4A50::peek(uInt16 address)
     else if((address & 0x1f00) == 0x1f00)      // 256B region from 0x1f00 - 0x1fff
     {
       value = myImage[(address & 0xff) + 0x1ff00];
-      if(!bankLocked() && ((myLastData & 0xe0) == 0x60) &&
+      if(!hotspotsLocked() && ((myLastData & 0xe0) == 0x60) &&
          ((myLastAddress >= 0x1000) || (myLastAddress < 0x200)))
         mySliceHigh = (mySliceHigh & 0xf0ff) | ((address & 0x8) << 8) |
                       ((address & 0x70) << 4);
@@ -139,7 +130,7 @@ bool Cartridge4A50::poke(uInt16 address, uInt8 value)
   if(!(address & 0x1000))                      // Hotspots below 0x1000
   {
     // Check for RAM or TIA mirroring
-    uInt16 lowAddress = address & 0x3ff;
+    const uInt16 lowAddress = address & 0x3ff;
     if(lowAddress & 0x80)
       mySystem->m6532().poke(address, value);
     else if(!(lowAddress & 0x200))
@@ -176,7 +167,7 @@ bool Cartridge4A50::poke(uInt16 address, uInt8 value)
     }
     else if((address & 0x1f00) == 0x1f00)      // 256B region at 0x1f00 - 0x1fff
     {
-      if(!bankLocked() && ((myLastData & 0xe0) == 0x60) &&
+      if(!hotspotsLocked() && ((myLastData & 0xe0) == 0x60) &&
          ((myLastAddress >= 0x1000) || (myLastAddress < 0x200)))
       {
         mySliceHigh = (mySliceHigh & 0xf0ff) | ((address & 0x8) << 8) |
@@ -191,73 +182,75 @@ bool Cartridge4A50::poke(uInt16 address, uInt8 value)
   return myBankChanged;
 }
 
+#ifdef DEBUGGER_SUPPORT
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-uInt8 Cartridge4A50::getAccessFlags(uInt16 address) const
+Device::AccessFlags Cartridge4A50::getAccessFlags(uInt16 address) const
 {
   if((address & 0x1800) == 0x1000)           // 2K region from 0x1000 - 0x17ff
   {
     if(myIsRomLow)
-      return myCodeAccessBase[(address & 0x7ff) + mySliceLow];
+      return myRomAccessBase[(address & 0x7ff) + mySliceLow];
     else
-      return myCodeAccessBase[131072 + (address & 0x7ff) + mySliceLow];
+      return myRomAccessBase[131072 + (address & 0x7ff) + mySliceLow];
   }
   else if(((address & 0x1fff) >= 0x1800) &&  // 1.5K region from 0x1800 - 0x1dff
           ((address & 0x1fff) <= 0x1dff))
   {
     if(myIsRomMiddle)
-      return myCodeAccessBase[(address & 0x7ff) + mySliceMiddle + 0x10000];
+      return myRomAccessBase[(address & 0x7ff) + mySliceMiddle + 0x10000];
     else
-      return myCodeAccessBase[131072 + (address & 0x7ff) + mySliceMiddle];
+      return myRomAccessBase[131072 + (address & 0x7ff) + mySliceMiddle];
   }
   else if((address & 0x1f00) == 0x1e00)      // 256B region from 0x1e00 - 0x1eff
   {
     if(myIsRomHigh)
-      return myCodeAccessBase[(address & 0xff) + mySliceHigh + 0x10000];
+      return myRomAccessBase[(address & 0xff) + mySliceHigh + 0x10000];
     else
-      return myCodeAccessBase[131072 + (address & 0xff) + mySliceHigh];
+      return myRomAccessBase[131072 + (address & 0xff) + mySliceHigh];
   }
   else if((address & 0x1f00) == 0x1f00)      // 256B region from 0x1f00 - 0x1fff
   {
-    return myCodeAccessBase[(address & 0xff) + 0x1ff00];
+    return myRomAccessBase[(address & 0xff) + 0x1ff00];
   }
   return 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Cartridge4A50::setAccessFlags(uInt16 address, uInt8 flags)
+void Cartridge4A50::setAccessFlags(uInt16 address, Device::AccessFlags flags)
 {
   if((address & 0x1800) == 0x1000)           // 2K region from 0x1000 - 0x17ff
   {
     if(myIsRomLow)
-      myCodeAccessBase[(address & 0x7ff) + mySliceLow] |= flags;
+      myRomAccessBase[(address & 0x7ff) + mySliceLow] |= flags;
     else
-      myCodeAccessBase[131072 + (address & 0x7ff) + mySliceLow] |= flags;
+      myRomAccessBase[131072 + (address & 0x7ff) + mySliceLow] |= flags;
   }
   else if(((address & 0x1fff) >= 0x1800) &&  // 1.5K region from 0x1800 - 0x1dff
           ((address & 0x1fff) <= 0x1dff))
   {
     if(myIsRomMiddle)
-      myCodeAccessBase[(address & 0x7ff) + mySliceMiddle + 0x10000] |= flags;
+      myRomAccessBase[(address & 0x7ff) + mySliceMiddle + 0x10000] |= flags;
     else
-      myCodeAccessBase[131072 + (address & 0x7ff) + mySliceMiddle] |= flags;
+      myRomAccessBase[131072 + (address & 0x7ff) + mySliceMiddle] |= flags;
   }
   else if((address & 0x1f00) == 0x1e00)      // 256B region from 0x1e00 - 0x1eff
   {
     if(myIsRomHigh)
-      myCodeAccessBase[(address & 0xff) + mySliceHigh + 0x10000] |= flags;
+      myRomAccessBase[(address & 0xff) + mySliceHigh + 0x10000] |= flags;
     else
-      myCodeAccessBase[131072 + (address & 0xff) + mySliceHigh] |= flags;
+      myRomAccessBase[131072 + (address & 0xff) + mySliceHigh] |= flags;
   }
   else if((address & 0x1f00) == 0x1f00)      // 256B region from 0x1f00 - 0x1fff
   {
-    myCodeAccessBase[(address & 0xff) + 0x1ff00] |= flags;
+    myRomAccessBase[(address & 0xff) + 0x1ff00] |= flags;
   }
 }
+#endif
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Cartridge4A50::checkBankSwitch(uInt16 address, uInt8 value)
 {
-  if(bankLocked()) return;
+  if(hotspotsLocked()) return;
 
   // This scheme contains so many hotspots that it's easier to just check
   // all of them
@@ -358,7 +351,7 @@ bool Cartridge4A50::patch(uInt16 address, uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const uInt8* Cartridge4A50::getImage(int& size) const
+const ByteBuffer& Cartridge4A50::getImage(size_t& size) const
 {
   size = mySize;
   return myImage;
@@ -369,10 +362,8 @@ bool Cartridge4A50::save(Serializer& out) const
 {
   try
   {
-    out.putString(name());
-
     // The 32K bytes of RAM
-    out.putByteArray(myRAM, 32768);
+    out.putByteArray(myRAM.data(), myRAM.size());
 
     // Index pointers
     out.putShort(mySliceLow);
@@ -402,10 +393,7 @@ bool Cartridge4A50::load(Serializer& in)
 {
   try
   {
-    if(in.getString() != name())
-      return false;
-
-    in.getByteArray(myRAM, 32768);
+    in.getByteArray(myRAM.data(), myRAM.size());
 
     // Index pointers
     mySliceLow = in.getShort();
