@@ -1,61 +1,41 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: PropsSet.cxx 3302 2016-04-02 23:47:46Z stephena $
 //============================================================================
 
-#include <fstream>
-#include <sstream>
 #include <map>
 
 #include "bspf.hxx"
-
+#include "FSNode.hxx"
+#include "Logger.hxx"
 #include "DefProps.hxx"
 #include "Props.hxx"
-
 #include "PropsSet.hxx"
+#include "repository/CompositeKeyValueRepositoryNoop.hxx"
+#include "repository/KeyValueRepositoryPropertyFile.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-PropertiesSet::PropertiesSet(const string& propsfile)
+PropertiesSet::PropertiesSet()
+  : myRepository{make_shared<CompositeKeyValueRepositoryNoop>()}
 {
-  load(propsfile);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PropertiesSet::load(const string& filename)
+void PropertiesSet::setRepository(shared_ptr<CompositeKeyValueRepository> repository)
 {
-  ifstream in(filename);
-
-  Properties prop;
-  while(in >> prop)
-    insert(prop);
-}
-
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool PropertiesSet::save(const string& filename) const
-{
-  ofstream out(filename);
-  if(!out)
-    return false;
-
-  // Only save those entries in the external list
-  for(const auto& i: myExternalProps)
-    out << i.second;
-
-  return true;
+  myRepository = repository;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -76,16 +56,14 @@ bool PropertiesSet::getMD5(const string& md5, Properties& properties,
   // First check properties from external file
   if(!useDefaults)
   {
-    // Check external list
-    auto ext = myExternalProps.find(md5);
-    if(ext != myExternalProps.end())
-    {
-      properties = ext->second;
+    if (myRepository->has(md5)) {
+      properties.load(*myRepository->get(md5));
+
       found = true;
     }
     else  // Search temp list
     {
-      auto tmp = myTempProps.find(md5);
+      const auto tmp = myTempProps.find(md5);
       if(tmp != myTempProps.end())
       {
         properties = tmp->second;
@@ -100,14 +78,15 @@ bool PropertiesSet::getMD5(const string& md5, Properties& properties,
     int low = 0, high = DEF_PROPS_SIZE - 1;
     while(low <= high)
     {
-      int i = (low + high) / 2;
-      int cmp = BSPF::compareIgnoreCase(md5, DefProps[i][Cartridge_MD5]);
+      const int i = (low + high) / 2;
+      const int cmp = BSPF::compareIgnoreCase(md5,
+          DefProps[i][static_cast<uInt8>(PropType::Cart_MD5)]);
 
       if(cmp == 0)  // found it
       {
-        for(int p = 0; p < LastPropType; ++p)
+        for(uInt8 p = 0; p < static_cast<uInt8>(PropType::NumTypes); ++p)
           if(DefProps[i][p][0] != 0)
-            properties.set(PropertyType(p), DefProps[i][p]);
+            properties.set(PropType{p}, DefProps[i][p]);
 
         found = true;
         break;
@@ -122,23 +101,10 @@ bool PropertiesSet::getMD5(const string& md5, Properties& properties,
   return found;
 }
 
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PropertiesSet::getMD5WithInsert(const FilesystemNode& rom,
-                                     const string& md5, Properties& properties)
-{
-  if(!getMD5(md5, properties))
-  {
-    properties.set(Cartridge_MD5, md5);
-    // Create a name suitable for using in properties
-    properties.set(Cartridge_Name, rom.getNameWithExt(""));
-
-    insert(properties, false);
-  }
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void PropertiesSet::insert(const Properties& properties, bool save)
 {
+  // TODO
   // Note that the following code is optimized for insertion when an item
   // doesn't already exist, and when the external properties file is
   // relatively small (which is the case with current versions of Stella,
@@ -149,7 +115,7 @@ void PropertiesSet::insert(const Properties& properties, bool save)
   // most people tend not to do
 
   // Since the PropSet is keyed by md5, we can't insert without a valid one
-  const string& md5 = properties.get(Cartridge_MD5);
+  const string& md5 = properties.get(PropType::Cart_MD5);
   if(md5 == "")
     return;
 
@@ -159,27 +125,58 @@ void PropertiesSet::insert(const Properties& properties, bool save)
     return;
   else if(getMD5(md5, defaultProps, true) && defaultProps == properties)
   {
-    myExternalProps.erase(md5);
+    cerr << "DELETE" << endl << std::flush;
+    myRepository->remove(md5);
     return;
   }
 
-  // The status of 'save' determines which list to save to
-  PropsList& list = save ? myExternalProps : myTempProps;
-
-  auto ret = list.emplace(md5, properties);
-  if(ret.second == false)
-  {
-    // Remove old item and insert again
-    list.erase(ret.first);
-    list.insert(make_pair(md5, properties));
+  if (save) {
+    properties.save(*myRepository->get(md5));
+  } else {
+    const auto ret = myTempProps.emplace(md5, properties);
+    if(ret.second == false)
+    {
+      // Remove old item and insert again
+      myTempProps.erase(ret.first);
+      myTempProps.emplace(md5, properties);
+    }
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void PropertiesSet::removeMD5(const string& md5)
+void PropertiesSet::loadPerROM(const FilesystemNode& rom, const string& md5)
 {
-  // We only remove from the external list
-  myExternalProps.erase(md5);
+  Properties props;
+
+  // Handle ROM properties, do some error checking
+  // Only add to the database when necessary
+  bool toInsert = false;
+
+  // First, does this ROM have a per-ROM properties entry?
+  // If so, load it into the database
+  FilesystemNode propsNode(rom.getPathWithExt(".pro"));
+  if (propsNode.exists()) {
+    KeyValueRepositoryPropertyFile repo(propsNode);
+    props.load(repo);
+
+    insert(props, false);
+  }
+
+  // Next, make sure we have a valid md5 and name
+  if(!getMD5(md5, props))
+  {
+    props.set(PropType::Cart_MD5, md5);
+    toInsert = true;
+  }
+  if(toInsert || props.get(PropType::Cart_Name) == EmptyString)
+  {
+    props.set(PropType::Cart_Name, rom.getNameWithExt(""));
+    toInsert = true;
+  }
+
+  // Finally, insert properties if any info was missing
+  if(toInsert)
+    insert(props, false);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -200,14 +197,14 @@ void PropertiesSet::print() const
   // This is fine, since a duplicate in the built-in list means it should
   // be overrided anyway (and insertion shouldn't be done)
   Properties properties;
-  for(int i = 0; i < DEF_PROPS_SIZE; ++i)
+  for(uInt32 i = 0; i < DEF_PROPS_SIZE; ++i)
   {
     properties.setDefaults();
-    for(int p = 0; p < LastPropType; ++p)
+    for(uInt8 p = 0; p < static_cast<uInt8>(PropType::NumTypes); ++p)
       if(DefProps[i][p][0] != 0)
-        properties.set(PropertyType(p), DefProps[i][p]);
+        properties.set(PropType{p}, DefProps[i][p]);
 
-    list.insert(make_pair(DefProps[i][Cartridge_MD5], properties));
+    list.emplace(DefProps[i][static_cast<uInt8>(PropType::Cart_MD5)], properties);
   }
 
   // Now, print the resulting list
