@@ -1,25 +1,20 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: Serializer.cxx 3308 2016-05-24 16:55:45Z stephena $
 //============================================================================
 
-#include <fstream>
-#include <sstream>
-
 #include "FSNode.hxx"
 #include "Serializer.hxx"
 
@@ -27,20 +22,20 @@ using std::ios;
 using std::ios_base;
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Serializer::Serializer(const string& filename, bool readonly)
-  : myStream(nullptr)
+Serializer::Serializer(const string& filename, Mode m)
 {
-  if(readonly)
+  if(m == Mode::ReadOnly)
   {
     FilesystemNode node(filename);
     if(node.isFile() && node.isReadable())
     {
-      unique_ptr<fstream> str = make_ptr<fstream>(filename, ios::in | ios::binary);
+      unique_ptr<fstream> str = make_unique<fstream>(filename, ios::in | ios::binary);
       if(str && str->is_open())
       {
         myStream = std::move(str);
-        myStream->exceptions( ios_base::failbit | ios_base::badbit | ios_base::eofbit );
-        reset();
+        rewind();
+        myStream->exceptions( ios_base::failbit | ios_base::badbit |
+                              ios_base::eofbit );
       }
     }
   }
@@ -56,40 +51,62 @@ Serializer::Serializer(const string& filename, bool readonly)
     fstream temp(filename, ios::out | ios::app);
     temp.close();
 
-    unique_ptr<fstream> str = make_ptr<fstream>(filename, ios::in | ios::out | ios::binary);
+    ios_base::openmode stream_mode = ios::in | ios::out | ios::binary;
+    if(m == Mode::ReadWriteTrunc)
+      stream_mode |= ios::trunc;
+    unique_ptr<fstream> str = make_unique<fstream>(filename, stream_mode);
     if(str && str->is_open())
     {
       myStream = std::move(str);
-      myStream->exceptions( ios_base::failbit | ios_base::badbit | ios_base::eofbit );
-      reset();
+      rewind();
+      myStream->exceptions( ios_base::failbit | ios_base::badbit |
+                            ios_base::eofbit );
     }
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Serializer::Serializer()
-  : myStream(nullptr)
+  : myStream{make_unique<stringstream>(ios::in | ios::out | ios::binary)}
 {
-  myStream = make_ptr<stringstream>(ios::in | ios::out | ios::binary);
-  
-  // For some reason, Windows and possibly OSX needs to store something in
+  // For some reason, Windows and possibly macOS needs to store something in
   // the stream before it is used for the first time
   if(myStream)
   {
-    myStream->exceptions( ios_base::failbit | ios_base::badbit | ios_base::eofbit );
     putBool(true);
-    reset();
+    rewind();
+    myStream->exceptions( ios_base::failbit | ios_base::badbit | ios_base::eofbit );
   }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Serializer::reset()
+void Serializer::setPosition(size_t pos)
+{
+  myStream->clear();
+  myStream->seekg(pos);
+  myStream->seekp(pos);
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Serializer::rewind()
 {
   myStream->clear();
   myStream->seekg(ios_base::beg);
   myStream->seekp(ios_base::beg);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+size_t Serializer::size()
+{
+  const std::streampos oldPos = myStream->tellp();
+
+  myStream->seekp(0, std::ios::end);
+  const size_t s = myStream->tellp();
+  myStream->seekp(oldPos);
+
+  return s;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 uInt8 Serializer::getByte() const
 {
@@ -100,7 +117,7 @@ uInt8 Serializer::getByte() const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Serializer::getByteArray(uInt8* array, uInt32 size) const
+void Serializer::getByteArray(uInt8* array, size_t size) const
 {
   myStream->read(reinterpret_cast<char*>(array), size);
 }
@@ -115,7 +132,7 @@ uInt16 Serializer::getShort() const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Serializer::getShortArray(uInt16* array, uInt32 size) const
+void Serializer::getShortArray(uInt16* array, size_t size) const
 {
   myStream->read(reinterpret_cast<char*>(array), sizeof(uInt16)*size);
 }
@@ -130,15 +147,33 @@ uInt32 Serializer::getInt() const
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Serializer::getIntArray(uInt32* array, uInt32 size) const
+void Serializer::getIntArray(uInt32* array, size_t size) const
 {
   myStream->read(reinterpret_cast<char*>(array), sizeof(uInt32)*size);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+uInt64 Serializer::getLong() const
+{
+  uInt64 val = 0;
+  myStream->read(reinterpret_cast<char*>(&val), sizeof(uInt64));
+
+  return val;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+double Serializer::getDouble() const
+{
+  double val = 0.0;
+  myStream->read(reinterpret_cast<char*>(&val), sizeof(double));
+
+  return val;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 string Serializer::getString() const
 {
-  int len = getInt();
+  const int len = getInt();
   string str;
   str.resize(len);
   myStream->read(&str[0], len);
@@ -159,7 +194,7 @@ void Serializer::putByte(uInt8 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Serializer::putByteArray(const uInt8* array, uInt32 size)
+void Serializer::putByteArray(const uInt8* array, size_t size)
 {
   myStream->write(reinterpret_cast<const char*>(array), size);
 }
@@ -171,7 +206,7 @@ void Serializer::putShort(uInt16 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Serializer::putShortArray(const uInt16* array, uInt32 size)
+void Serializer::putShortArray(const uInt16* array, size_t size)
 {
   myStream->write(reinterpret_cast<const char*>(array), sizeof(uInt16)*size);
 }
@@ -183,15 +218,27 @@ void Serializer::putInt(uInt32 value)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void Serializer::putIntArray(const uInt32* array, uInt32 size)
+void Serializer::putIntArray(const uInt32* array, size_t size)
 {
   myStream->write(reinterpret_cast<const char*>(array), sizeof(uInt32)*size);
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Serializer::putLong(uInt64 value)
+{
+  myStream->write(reinterpret_cast<char*>(&value), sizeof(uInt64));
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void Serializer::putDouble(double value)
+{
+  myStream->write(reinterpret_cast<char*>(&value), sizeof(double));
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void Serializer::putString(const string& str)
 {
-  int len = int(str.length());
+  const uInt32 len = static_cast<uInt32>(str.length());
   putInt(len);
   myStream->write(str.data(), len);
 }
