@@ -1,48 +1,41 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: ContextMenu.cxx 3304 2016-04-03 00:35:00Z stephena $
 //============================================================================
 
 #include "OSystem.hxx"
+#include "EventHandler.hxx"
 #include "FrameBuffer.hxx"
+#include "FBSurface.hxx"
+#include "Font.hxx"
 #include "Dialog.hxx"
 #include "DialogContainer.hxx"
 #include "ScrollBarWidget.hxx"
+#include "PopUpWidget.hxx"
 #include "ContextMenu.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ContextMenu::ContextMenu(GuiObject* boss, const GUI::Font& font,
-                         const VariantList& items, int cmd)
-  : Dialog(boss->instance(), boss->parent()),
+                         const VariantList& items, int cmd, int width)
+  : Dialog(boss->instance(), boss->parent(), font),
     CommandSender(boss),
-    _rowHeight(font.getLineHeight()),
-    _firstEntry(0),
-    _numEntries(0),
-    _selectedOffset(0),
-    _selectedItem(-1),
-    _showScroll(false),
-    _isScrolling(false),
-    _scrollUpColor(kColor),
-    _scrollDnColor(kColor),
-    _font(font),
-    _cmd(cmd),
-    _xorig(0),
-    _yorig(0)
+    _rowHeight{font.getLineHeight()},
+    _cmd{cmd},
+    _maxWidth{width}
 {
+  setArrows();
   addItems(items);
 }
 
@@ -53,24 +46,29 @@ void ContextMenu::addItems(const VariantList& items)
   _entries = items;
 
   // Resize to largest string
-  int maxwidth = 0;
+  int maxwidth = _maxWidth;
   for(const auto& e: _entries)
-    maxwidth = std::max(maxwidth, _font.getStringWidth(e.first));
+    maxwidth = std::max(maxwidth, _font.getStringWidth(e.first) + _textOfs * 2 + 2);
 
   _x = _y = 0;
-  _w = maxwidth + 10;
+  _w = maxwidth;
   _h = 1;  // recalculate this in ::recalc()
 
   _scrollUpColor = _firstEntry > 0 ? kScrollColor : kColor;
-  _scrollDnColor = (_firstEntry + _numEntries < int(_entries.size())) ?
+  _scrollDnColor = (_firstEntry + _numEntries < static_cast<int>(_entries.size())) ?
       kScrollColor : kColor;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::show(uInt32 x, uInt32 y, int item)
+void ContextMenu::show(uInt32 x, uInt32 y, const Common::Rect& bossRect, int item)
 {
-  _xorig = x;
-  _yorig = y;
+  const uInt32 scale = instance().frameBuffer().hidpiScaleFactor();
+  _xorig = bossRect.x() + x * scale;
+  _yorig = bossRect.y() + y * scale;
+
+  // Only show menu if we're inside the visible area
+  if(!bossRect.contains(_xorig, _yorig))
+    return;
 
   recalc(instance().frameBuffer().imageRect());
   open();
@@ -79,39 +77,35 @@ void ContextMenu::show(uInt32 x, uInt32 y, int item)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::center()
+void ContextMenu::setPosition()
 {
-  // Make sure the menu is exactly where it should be, in case the image
-  // offset has changed
-  const GUI::Rect& image = instance().frameBuffer().imageRect();
-  recalc(image);
-  uInt32 x = image.x() + _xorig;
-  uInt32 y = image.y() + _yorig;
-  uInt32 tx = image.x() + image.width();
-  uInt32 ty = image.y() + image.height();
-  if(x + _w > tx) x -= (x + _w - tx);
-  if(y + _h > ty) y -= (y + _h - ty);
+  // First set position according to original coordinates
+  surface().setDstPos(_xorig, _yorig);
 
-  surface().setDstPos(x, y);
+  // Now make sure that the entire menu can fit inside the screen bounds
+  // If not, we reset its position
+  if(!instance().frameBuffer().screenRect().contains(
+      _xorig, _yorig, surface().dstRect()))
+    surface().setDstPos(_xorig, _yorig);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::recalc(const GUI::Rect& image)
+void ContextMenu::recalc(const Common::Rect& image)
 {
   // Now is the time to adjust the height
   // If it's higher than the screen, we need to scroll through
-  uInt32 maxentries = std::min(18u, (image.height() - 4) / _rowHeight);
+  const uInt32 maxentries = std::min<uInt32>(18, (image.h() - 2) / _rowHeight);
   if(_entries.size() > maxentries)
   {
     // We show two less than the max, so we have room for two scroll buttons
     _numEntries = maxentries - 2;
-    _h = maxentries * _rowHeight + 4;
+    _h = maxentries * _rowHeight + 2;
     _showScroll = true;
   }
   else
   {
-    _numEntries = int(_entries.size());
-    _h = int(_entries.size()) * _rowHeight + 4;
+    _numEntries = static_cast<int>(_entries.size());
+    _h = static_cast<int>(_entries.size()) * _rowHeight + 2;
     _showScroll = false;
   }
   _isScrolling = false;
@@ -120,7 +114,7 @@ void ContextMenu::recalc(const GUI::Rect& image)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ContextMenu::setSelectedIndex(int idx)
 {
-  if(idx >= 0 && idx < int(_entries.size()))
+  if(idx >= 0 && idx < static_cast<int>(_entries.size()))
     _selectedItem = idx;
   else
     _selectedItem = -1;
@@ -129,33 +123,31 @@ void ContextMenu::setSelectedIndex(int idx)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ContextMenu::setSelected(const Variant& tag, const Variant& defaultTag)
 {
-  if(tag != "")  // indicates that the defaultTag should be used instead
+  const auto SEARCH_AND_SELECT = [&](const Variant& t)
   {
     for(uInt32 item = 0; item < _entries.size(); ++item)
-    { 
-      if(BSPF::equalsIgnoreCase(_entries[item].second.toString(), tag.toString()))
+    {
+      if(BSPF::equalsIgnoreCase(_entries[item].second.toString(), t.toString()))
       {
         setSelectedIndex(item);
-        return;
+        return true;
       }
     }
-  }
+    return false;
+  };
 
-  // If we get this far, the value wasn't found; use the default value
-  for(uInt32 item = 0; item < _entries.size(); ++item)
-  {
-    if(BSPF::equalsIgnoreCase(_entries[item].second.toString(), defaultTag.toString()))
-    {
-      setSelectedIndex(item);
-      return;
-    }
-  }
+  // First try searching for a valid tag
+  bool tagSelected = tag != "" && SEARCH_AND_SELECT(tag);
+
+  // Otherwise use the default tag
+  if(!tagSelected)
+    SEARCH_AND_SELECT(defaultTag);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ContextMenu::setSelectedMax()
 {
-  setSelectedIndex(int(_entries.size()) - 1);
+  setSelectedIndex(static_cast<int>(_entries.size()) - 1);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -176,6 +168,13 @@ const string& ContextMenu::getSelectedName() const
   return (_selectedItem >= 0) ? _entries[_selectedItem].first : EmptyString;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void ContextMenu::setSelectedName(const string& name)
+{
+  if(_selectedItem >= 0)
+    _entries[_selectedItem].first = name;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 const Variant& ContextMenu::getSelectedTag() const
 {
@@ -189,18 +188,18 @@ bool ContextMenu::sendSelectionUp()
     return false;
 
   _selectedItem--;
-  sendCommand(_cmd ? _cmd : ContextMenu::kItemSelectedCmd, _selectedItem, -1);
+  sendCommand(_cmd ? _cmd : ContextMenu::kItemSelectedCmd, _selectedItem, _id);
   return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool ContextMenu::sendSelectionDown()
 {
-  if(isVisible() || _selectedItem >= int(_entries.size()) - 1)
+  if(isVisible() || _selectedItem >= static_cast<int>(_entries.size()) - 1)
     return false;
 
   _selectedItem++;
-  sendCommand(_cmd ? _cmd : ContextMenu::kItemSelectedCmd, _selectedItem, -1);
+  sendCommand(_cmd ? _cmd : ContextMenu::kItemSelectedCmd, _selectedItem, _id);
   return true;
 }
 
@@ -211,7 +210,7 @@ bool ContextMenu::sendSelectionFirst()
     return false;
 
   _selectedItem = 0;
-  sendCommand(_cmd ? _cmd : ContextMenu::kItemSelectedCmd, _selectedItem, -1);
+  sendCommand(_cmd ? _cmd : ContextMenu::kItemSelectedCmd, _selectedItem, _id);
   return true;
 }
 
@@ -221,19 +220,19 @@ bool ContextMenu::sendSelectionLast()
   if(isVisible())
     return false;
 
-  _selectedItem = int(_entries.size()) - 1;
-  sendCommand(_cmd ? _cmd : ContextMenu::kItemSelectedCmd, _selectedItem, -1);
+  _selectedItem = static_cast<int>(_entries.size()) - 1;
+  sendCommand(_cmd ? _cmd : ContextMenu::kItemSelectedCmd, _selectedItem, _id);
   return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::handleMouseDown(int x, int y, int button, int clickCount)
+void ContextMenu::handleMouseDown(int x, int y, MouseButton b, int clickCount)
 {
   // Compute over which item the mouse is...
-  int item = findItem(x, y);
+  const int item = findItem(x, y);
 
   // Only do a selection when the left button is in the dialog
-  if(button == 1)
+  if(b == MouseButton::LEFT)
   {
     if(item != -1)
     {
@@ -246,10 +245,10 @@ void ContextMenu::handleMouseDown(int x, int y, int button, int clickCount)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::handleMouseMoved(int x, int y, int button)
+void ContextMenu::handleMouseMoved(int x, int y)
 {
   // Compute over which item the mouse is...
-  int item = findItem(x, y);
+  const int item = findItem(x, y);
   if(item == -1)
     return;
 
@@ -258,7 +257,7 @@ void ContextMenu::handleMouseMoved(int x, int y, int button)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ContextMenu::handleMouseClicks(int x, int y, int button)
+bool ContextMenu::handleMouseClicks(int x, int y, MouseButton b)
 {
   // Let continuous mouse clicks come through, as the scroll buttons need them
   return true;
@@ -278,28 +277,28 @@ void ContextMenu::handleMouseWheel(int x, int y, int direction)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::handleKeyDown(StellaKey key, StellaMod mod)
+void ContextMenu::handleKeyDown(StellaKey key, StellaMod mod, bool repeated)
 {
-  handleEvent(instance().eventHandler().eventForKey(key, kMenuMode));
+  handleEvent(instance().eventHandler().eventForKey(EventMode::kMenuMode, key, mod));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::handleJoyDown(int stick, int button)
+void ContextMenu::handleJoyUp(int stick, int button)
 {
-  handleEvent(instance().eventHandler().eventForJoyButton(stick, button, kMenuMode));
+  handleEvent(instance().eventHandler().eventForJoyButton(EventMode::kMenuMode, stick, button));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::handleJoyAxis(int stick, int axis, int value)
+void ContextMenu::handleJoyAxis(int stick, JoyAxis axis, JoyDir adir, int button)
 {
-  if(value != 0)  // we don't care about 'axis up' events
-    handleEvent(instance().eventHandler().eventForJoyAxis(stick, axis, value, kMenuMode));
+  if(adir != JoyDir::NONE)  // we don't care about 'axis off' events
+    handleEvent(instance().eventHandler().eventForJoyAxis(EventMode::kMenuMode, stick, axis, adir, button));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-bool ContextMenu::handleJoyHat(int stick, int hat, int value)
+bool ContextMenu::handleJoyHat(int stick, int hat, JoyHatDir hdir, int button)
 {
-  handleEvent(instance().eventHandler().eventForJoyHat(stick, hat, value, kMenuMode));
+  handleEvent(instance().eventHandler().eventForJoyHat(EventMode::kMenuMode, stick, hat, hdir, button));
   return true;
 }
 
@@ -332,6 +331,7 @@ void ContextMenu::handleEvent(Event::Type e)
       moveToLast();
       break;
     case Event::UICancel:
+    case Event::UIOK:
       close();
       break;
     default:
@@ -352,8 +352,11 @@ int ContextMenu::findItem(int x, int y) const
 void ContextMenu::drawCurrentSelection(int item)
 {
   // Change selection
-  _selectedOffset = item;
-  setDirty();
+  if(_selectedOffset != item)
+  {
+    _selectedOffset = item;
+    setDirty();
+  }
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -383,7 +386,7 @@ void ContextMenu::sendSelection()
 
   // Send any command associated with the selection
   _selectedItem = item;
-  sendCommand(_cmd ? _cmd : ContextMenu::kItemSelectedCmd, _selectedItem, -1);
+  sendCommand(_cmd ? _cmd : ContextMenu::kItemSelectedCmd, _selectedItem, _id);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -416,12 +419,12 @@ void ContextMenu::moveDown()
     // Otherwise, the offset should increase by 1
     if(_selectedOffset == _numEntries)
       scrollDown();
-    else if(_selectedOffset < int(_entries.size()))
-      drawCurrentSelection(_selectedOffset+1);    
+    else if(_selectedOffset < static_cast<int>(_entries.size()))
+      drawCurrentSelection(_selectedOffset+1);
   }
   else
   {
-    if(_selectedOffset < int(_entries.size()) - 1)
+    if(_selectedOffset < static_cast<int>(_entries.size()) - 1)
       drawCurrentSelection(_selectedOffset+1);
   }
 }
@@ -438,7 +441,7 @@ void ContextMenu::movePgUp()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ContextMenu::movePgDown()
 {
-  if(_firstEntry == int(_entries.size() - _numEntries))
+  if(_firstEntry == static_cast<int>(_entries.size() - _numEntries))
     moveToLast();
   else
     scrollDown(_numEntries);
@@ -457,7 +460,7 @@ void ContextMenu::moveToFirst()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ContextMenu::moveToLast()
 {
-  _firstEntry = int(_entries.size()) - _numEntries;
+  _firstEntry = static_cast<int>(_entries.size()) - _numEntries;
   _scrollUpColor = kScrollColor;
   _scrollDnColor = kColor;
 
@@ -467,7 +470,7 @@ void ContextMenu::moveToLast()
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ContextMenu::moveToSelected()
 {
-  if(_selectedItem < 0 || _selectedItem >= int(_entries.size()))
+  if(_selectedItem < 0 || _selectedItem >= static_cast<int>(_entries.size()))
     return;
 
   // First jump immediately to the item
@@ -476,7 +479,7 @@ void ContextMenu::moveToSelected()
 
   // Now check if we've gone past the current 'window' size, and scale
   // back accordingly
-  int max_offset = int(_entries.size()) - _numEntries;
+  const int max_offset = static_cast<int>(_entries.size()) - _numEntries;
   if(_firstEntry > max_offset)
   {
     offset = _firstEntry - max_offset;
@@ -505,7 +508,7 @@ void ContextMenu::scrollUp(int distance)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void ContextMenu::scrollDown(int distance)
 {
-  int max_offset = int(_entries.size()) - _numEntries;
+  const int max_offset = static_cast<int>(_entries.size()) - _numEntries;
   if(_firstEntry == max_offset)
     return;
 
@@ -517,73 +520,115 @@ void ContextMenu::scrollDown(int distance)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-void ContextMenu::drawDialog()
+void ContextMenu::setArrows()
 {
-  static uInt32 up_arrow[8] = {
-    0x00011000,
-    0x00011000,
-    0x00111100,
-    0x00111100,
-    0x01111110,
-    0x01111110,
-    0x11111111,
-    0x11111111
+  static constexpr std::array<uInt32, 8> up_arrow = {
+    0b00011000,
+    0b00011000,
+    0b00111100,
+    0b00111100,
+    0b01111110,
+    0b01111110,
+    0b11111111,
+    0b11111111
   };
-  static uInt32 down_arrow[8] = {
-    0x11111111,
-    0x11111111,
-    0x01111110,
-    0x01111110,
-    0x00111100,
-    0x00111100,
-    0x00011000,
-    0x00011000
+  static constexpr std::array<uInt32, 8> down_arrow = {
+    0b11111111,
+    0b11111111,
+    0b01111110,
+    0b01111110,
+    0b00111100,
+    0b00111100,
+    0b00011000,
+    0b00011000
   };
 
+  static constexpr std::array<uInt32, 12> up_arrow_large = {
+    0b000001100000,
+    0b000001100000,
+    0b000011110000,
+    0b000011110000,
+    0b000111111000,
+    0b000111111000,
+    0b001111111100,
+    0b001111111100,
+    0b011111111110,
+    0b011111111110,
+    0b111111111111,
+    0b111111111111
+  };
+  static constexpr std::array<uInt32, 12> down_arrow_large = {
+    0b111111111111,
+    0b111111111111,
+    0b011111111110,
+    0b011111111110,
+    0b001111111100,
+    0b001111111100,
+    0b000111111000,
+    0b000111111000,
+    0b000011110000,
+    0b000011110000,
+    0b000001100000,
+    0b000001100000
+  };
+
+  if(_font.getFontHeight() < 24)
+  {
+    _textOfs = 2;
+    _arrowSize = 8;
+    _upImg = up_arrow.data();
+    _downImg = down_arrow.data();
+  }
+  else
+  {
+    _textOfs = 4;
+    _arrowSize = 12;
+    _upImg = up_arrow_large.data();
+    _downImg = down_arrow_large.data();
+  }
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void ContextMenu::drawDialog()
+{
   // Normally we add widgets and let Dialog::draw() take care of this
   // logic.  But for some reason, this Dialog was written differently
   // by the ScummVM guys, so I'm not going to mess with it.
   FBSurface& s = surface();
 
-  if(_dirty)
+  // Draw menu border and background
+  s.fillRect(_x+1, _y+1, _w-2, _h-2, kWidColor);
+  s.frameRect(_x, _y, _w, _h, kTextColor);
+
+  // Draw the entries, taking scroll buttons into account
+  const int x = _x + 1, w = _w - 2;
+  int y = _y + 1;
+
+  // Show top scroll area
+  int offset = _selectedOffset;
+  if(_showScroll)
   {
-    // Draw menu border and background
-    s.fillRect(_x+1, _y+1, _w-2, _h-2, kWidColor);
-    s.box(_x, _y, _w, _h, kColor, kShadowColor);
-
-    // Draw the entries, taking scroll buttons into account
-    int x = _x + 2, y = _y + 2, w = _w - 4;
-
-    // Show top scroll area
-    int offset = _selectedOffset;
-    if(_showScroll)
-    {
-      s.hLine(x, y+_rowHeight-1, w+2, kShadowColor);
-      s.drawBitmap(up_arrow, ((_w-_x)>>1)-4, (_rowHeight>>1)+y-4, _scrollUpColor, 8);
-      y += _rowHeight;
-      offset--;
-    }
-
-    for(int i = _firstEntry, current = 0; i < _firstEntry + _numEntries; ++i, ++current)
-    {
-      bool hilite = offset == current;
-      if(hilite) s.fillRect(x, y, w, _rowHeight, kTextColorHi);
-      s.drawString(_font, _entries[i].first, x + 1, y + 2, w,
-                   !hilite ? kTextColor : kWidColor);
-      y += _rowHeight;
-    }
-
-    // Show bottom scroll area
-    if(_showScroll)
-    {
-      s.hLine(x, y, w+2, kShadowColor);
-      s.drawBitmap(down_arrow, ((_w-_x)>>1)-4, (_rowHeight>>1)+y-4, _scrollDnColor, 8);
-    }
-
-    s.setDirty();
-    _dirty = false;
+    s.hLine(x, y+_rowHeight-1, w+2, kColor);
+    s.drawBitmap(_upImg, ((_w-_x)>>1)-4, (_rowHeight>>1)+y-4, _scrollUpColor, _arrowSize);
+    y += _rowHeight;
+    offset--;
   }
 
-  // Commit surface changes to screen
-  s.render();
+  for(int i = _firstEntry, current = 0; i < _firstEntry + _numEntries; ++i, ++current)
+  {
+    const bool hilite = offset == current;
+    if(hilite) s.fillRect(x, y, w, _rowHeight, kTextColorHi);
+    s.drawString(_font, _entries[i].first, x + _textOfs, y + 2, w,
+                 !hilite ? kTextColor : kTextColorInv);
+    y += _rowHeight;
+  }
+
+  // Show bottom scroll area
+  if(_showScroll)
+  {
+    s.hLine(x, y, w+2, kColor);
+    s.drawBitmap(_downImg, ((_w-_x)>>1)-4, (_rowHeight>>1)+y-4, _scrollDnColor, _arrowSize);
+  }
+
+  clearDirty();
 }
