@@ -1,206 +1,195 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: GlobalPropsDialog.cxx 3302 2016-04-02 23:47:46Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
-
-#include "Cart.hxx"
+#include "Bankswitch.hxx"
 #include "Control.hxx"
 #include "Dialog.hxx"
 #include "OSystem.hxx"
+#include "FrameBuffer.hxx"
 #include "PopUpWidget.hxx"
 #include "Settings.hxx"
 #include "Widget.hxx"
+#include "Font.hxx"
 #include "LauncherDialog.hxx"
-
 #include "GlobalPropsDialog.hxx"
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 GlobalPropsDialog::GlobalPropsDialog(GuiObject* boss, const GUI::Font& font)
-  : Dialog(boss->instance(), boss->parent()),
+  : Dialog(boss->instance(), boss->parent(), font, "Power-on options"),
     CommandSender(boss)
 {
-  const int lineHeight   = font.getLineHeight(),
-            fontWidth    = font.getMaxCharWidth(),
-            fontHeight   = font.getFontHeight(),
-            buttonWidth  = font.getStringWidth("Defaults") + 20,
-            buttonHeight = font.getLineHeight() + 4;
-  int xpos, ypos;
-  int lwidth = font.getStringWidth("Right Difficulty: "),
+  const int lineHeight   = Dialog::lineHeight(),
+            fontWidth    = Dialog::fontWidth(),
+            buttonHeight = Dialog::buttonHeight(),
+            VBORDER      = Dialog::vBorder(),
+            HBORDER      = Dialog::hBorder(),
+            VGAP         = Dialog::vGap();
+  int lwidth = font.getStringWidth("Right difficulty "),
       pwidth = font.getStringWidth("CM (SpectraVideo CompuMate)");
   WidgetArray wid;
   VariantList items;
   const GUI::Font& infofont = instance().frameBuffer().infoFont();
 
   // Set real dimensions
-  _w = lwidth + pwidth + fontWidth*3 + 15;
-  _h = 17 * (lineHeight + 4) + buttonHeight + 20;
+  _w = HBORDER * 2 + std::max(lwidth + pwidth + PopUpWidget::dropDownWidth(font),
+                              49 * infofont.getMaxCharWidth());
+  _h = _th + 11 * (lineHeight + VGAP) + 3 * infofont.getLineHeight() + VGAP * 12 + buttonHeight + VBORDER * 2;
 
-  xpos = 10;  ypos = 10;
+  int xpos = HBORDER, ypos = VBORDER + _th;
 
   // Bankswitch type
-  new StaticTextWidget(this, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Bankswitch type:", kTextAlignLeft);
-  for(int i = 0; i < Cartridge::ourNumBSTypes; ++i)
-    VarList::push_back(items, Cartridge::ourBSList[i].desc, Cartridge::ourBSList[i].type);
+  new StaticTextWidget(this, font, xpos, ypos+1, "Bankswitch type");
+  for(uInt32 i = 0; i < static_cast<uInt32>(Bankswitch::Type::NumSchemes); ++i)
+    VarList::push_back(items, Bankswitch::BSList[i].desc, Bankswitch::BSList[i].name);
   myBSType = new PopUpWidget(this, font, xpos+lwidth, ypos,
-                             pwidth, lineHeight, items, "", 0, 0);
+                             pwidth, lineHeight, items, "");
   wid.push_back(myBSType);
-  ypos += lineHeight + 10;
+  ypos += lineHeight + VGAP * 3;
 
-  // Left difficulty
-  pwidth = font.getStringWidth("Debugger");
-  new StaticTextWidget(this, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Left Difficulty:", kTextAlignLeft);
-  items.clear();
-  VarList::push_back(items, "Default", "DEFAULT");
-  VarList::push_back(items, "B", "B");
-  VarList::push_back(items, "A", "A");
-  myLeftDiff = new PopUpWidget(this, font, xpos+lwidth, ypos,
-                               pwidth, lineHeight, items, "", 0, 0);
-  wid.push_back(myLeftDiff);
-  ypos += lineHeight + 5;
-
-  // Right difficulty
-  new StaticTextWidget(this, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Right Difficulty:", kTextAlignLeft);
-  // ... use same items as above
-  myRightDiff = new PopUpWidget(this, font, xpos+lwidth, ypos,
-                                pwidth, lineHeight, items, "", 0, 0);
-  wid.push_back(myRightDiff);
-  ypos += lineHeight + 5;
+  pwidth = font.getStringWidth("A (Expert)");
 
   // TV type
-  new StaticTextWidget(this, font, xpos, ypos+1, lwidth, fontHeight,
-                       "TV Type:", kTextAlignLeft);
+  new StaticTextWidget(this, font, xpos, ypos + 1, "TV type");
   items.clear();
   VarList::push_back(items, "Default", "DEFAULT");
   VarList::push_back(items, "Color", "COLOR");
-  VarList::push_back(items, "B & W", "BW");
-  myTVType = new PopUpWidget(this, font, xpos+lwidth, ypos,
-                             pwidth, lineHeight, items, "", 0, 0);
+  VarList::push_back(items, "B/W", "BW");
+  myTVType = new PopUpWidget(this, font, xpos + lwidth, ypos,
+                             pwidth, lineHeight, items, "");
   wid.push_back(myTVType);
-  ypos += lineHeight + 10;
+  ypos += lineHeight + VGAP;
+
+  // Left difficulty
+  new StaticTextWidget(this, font, xpos, ypos+1, GUI::LEFT_DIFFICULTY);
+  items.clear();
+  VarList::push_back(items, "Default", "DEFAULT");
+  VarList::push_back(items, "A (Expert)", "A");
+  VarList::push_back(items, "B (Novice)", "B");
+  myLeftDiff = new PopUpWidget(this, font, xpos+lwidth, ypos,
+                               pwidth, lineHeight, items, "");
+  wid.push_back(myLeftDiff);
+  ypos += lineHeight + VGAP;
+
+  // Right difficulty
+  new StaticTextWidget(this, font, xpos, ypos+1, GUI::RIGHT_DIFFICULTY);
+  // ... use same items as above
+  myRightDiff = new PopUpWidget(this, font, xpos+lwidth, ypos,
+                                pwidth, lineHeight, items, "");
+  wid.push_back(myRightDiff);
+  ypos += lineHeight + VGAP * 3;
 
   // Start in debugger mode
-  new StaticTextWidget(this, font, xpos, ypos+1, lwidth, fontHeight,
-                       "Startup Mode:", kTextAlignLeft);
+  new StaticTextWidget(this, font, xpos, ypos+1, "Startup mode");
   items.clear();
   VarList::push_back(items, "Console", "false");
+#ifdef DEBUGGER_SUPPORT
   VarList::push_back(items, "Debugger", "true");
+#endif
   myDebug = new PopUpWidget(this, font, xpos+lwidth, ypos,
-                            pwidth, lineHeight, items, "", 0, 0);
+                            pwidth, lineHeight, items, "");
   wid.push_back(myDebug);
-  ypos += lineHeight + 10;
+  ypos += lineHeight + VGAP * 3;
 
   // Start console with buttons held down
   new StaticTextWidget(this, font, xpos, ypos+1,
-      font.getStringWidth("Start console with the following held down:"),
-      fontHeight, "Start console with the following held down:",
-      kTextAlignLeft);
-  xpos += 10;  ypos += lineHeight;
-  new StaticTextWidget(this, infofont, xpos, ypos+1, _w - 40, infofont.getFontHeight(),
-      "(*) Buttons are automatically released shortly",
-      kTextAlignLeft);
-  ypos += infofont.getLineHeight();
-  new StaticTextWidget(this, infofont, xpos, ypos+1, _w - 40, infofont.getFontHeight(),
-      "    after emulation has started",
-      kTextAlignLeft);
+      "Start with the following held down:");
+  ypos += lineHeight;
+  new StaticTextWidget(this, infofont, xpos, ypos+1,
+      "(automatically released shortly after start)");
 
   // Start with console joystick direction/buttons held down
-  xpos = 30;  ypos += lineHeight + 10;
+  xpos = fontWidth * 4;  ypos += infofont.getLineHeight() + VGAP * 2;
   ypos = addHoldWidgets(font, xpos, ypos, wid);
 
   // Add message concerning usage
-  xpos = 10;  ypos += 2 * fontHeight;
-  new StaticTextWidget(this, infofont, xpos, ypos,  _w - 20, infofont.getFontHeight(),
-    "(*) These options are not saved, but apply to all", kTextAlignLeft);
+  xpos = HBORDER;
+  ypos = _h - VBORDER - buttonHeight - VGAP * 3 - infofont.getLineHeight() * 2;
+  new StaticTextWidget(this, infofont, xpos, ypos,
+    "(*) These options are not saved, but apply to all");
   ypos += infofont.getLineHeight();
-  new StaticTextWidget(this, infofont, xpos, ypos,  _w - 20, infofont.getFontHeight(),
-    "    further ROMs until clicking 'Defaults'", kTextAlignLeft);
+  new StaticTextWidget(this, infofont, xpos, ypos,
+    "    further ROMs until selecting 'Defaults'.");
 
   // Add Defaults, OK and Cancel buttons
-  ButtonWidget* b;
-  b = new ButtonWidget(this, font, 10, _h - buttonHeight - 10,
-                       buttonWidth, buttonHeight, "Defaults", kDefaultsCmd);
-  wid.push_back(b);
-  addOKCancelBGroup(wid, font, "Load ROM", "Close");
+  addDefaultsOKCancelBGroup(wid, font, "Load ROM", "Cancel");
 
   addToFocusList(wid);
+
+  setHelpAnchor("PowerOn");
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int GlobalPropsDialog::addHoldWidgets(const GUI::Font& font, int x, int y,
                                       WidgetArray& wid)
 {
-  const int fontHeight = font.getFontHeight();
+  const int fontWidth  = Dialog::fontWidth(),
+            VGAP       = Dialog::vGap();
   int xpos = x, ypos = y;
+  const int xdiff = CheckboxWidget::boxSize(font) - 9;
 
   // Left joystick
-  StaticTextWidget* t = new StaticTextWidget(this, font, xpos, ypos+2,
-    font.getStringWidth("Left Joy:"), fontHeight, "Left Joy:",
-    kTextAlignLeft);
-  xpos += t->getWidth()/2 - 5;  ypos += t->getHeight() + 10;
+  StaticTextWidget* t = new StaticTextWidget(this, font, xpos, ypos + 2, "Left joy");
+  xpos += t->getWidth()/2 - xdiff - 2;  ypos += t->getHeight() + VGAP;
   myJoy[kJ0Up] = new CheckboxWidget(this, font, xpos, ypos, "", kJ0Up);
-  ypos += myJoy[kJ0Up]->getHeight() * 2 + 10;
+  ypos += myJoy[kJ0Up]->getHeight() * 2 + VGAP * 2;
   myJoy[kJ0Down] = new CheckboxWidget(this, font, xpos, ypos, "", kJ0Down);
-  xpos -= myJoy[kJ0Up]->getWidth() + 5;
-  ypos -= myJoy[kJ0Up]->getHeight() + 5;
+  xpos -= myJoy[kJ0Up]->getWidth() + xdiff;
+  ypos -= myJoy[kJ0Up]->getHeight() + VGAP;
   myJoy[kJ0Left] = new CheckboxWidget(this, font, xpos, ypos, "", kJ0Left);
-  xpos += (myJoy[kJ0Up]->getWidth() + 5) * 2;
+  xpos += (myJoy[kJ0Up]->getWidth() + xdiff) * 2;
   myJoy[kJ0Right] = new CheckboxWidget(this, font, xpos, ypos, "", kJ0Right);
-  xpos -= (myJoy[kJ0Up]->getWidth() + 5) * 2;
-  ypos += myJoy[kJ0Down]->getHeight() * 2 + 10;
+  xpos -= (myJoy[kJ0Up]->getWidth() + xdiff) * 2;
+  ypos += myJoy[kJ0Down]->getHeight() * 2 + VGAP * 2;
   myJoy[kJ0Fire] = new CheckboxWidget(this, font, xpos, ypos, "Fire", kJ0Fire);
 
-  int final_y = ypos;
+  const int final_y = ypos;
   xpos = _w / 3;  ypos = y;
 
   // Right joystick
-  t = new StaticTextWidget(this, font, xpos, ypos+2,
-    font.getStringWidth("Right Joy:"), fontHeight, "Right Joy:",
-    kTextAlignLeft);
-  xpos += t->getWidth()/2 - 5;  ypos += t->getHeight() + 10;
+  t = new StaticTextWidget(this, font, xpos, ypos + 2, "Right joy");
+  xpos += t->getWidth()/2 - xdiff - 2;  ypos += t->getHeight() + VGAP;
   myJoy[kJ1Up] = new CheckboxWidget(this, font, xpos, ypos, "", kJ1Up);
-  ypos += myJoy[kJ1Up]->getHeight() * 2 + 10;
+  ypos += myJoy[kJ1Up]->getHeight() * 2 + VGAP * 2;
   myJoy[kJ1Down] = new CheckboxWidget(this, font, xpos, ypos, "", kJ1Down);
-  xpos -= myJoy[kJ1Up]->getWidth() + 5;
-  ypos -= myJoy[kJ1Up]->getHeight() + 5;
+  xpos -= myJoy[kJ1Up]->getWidth() + xdiff;
+  ypos -= myJoy[kJ1Up]->getHeight() + VGAP;
   myJoy[kJ1Left] = new CheckboxWidget(this, font, xpos, ypos, "", kJ1Left);
-  xpos += (myJoy[kJ1Up]->getWidth() + 5) * 2;
+  xpos += (myJoy[kJ1Up]->getWidth() + xdiff) * 2;
   myJoy[kJ1Right] = new CheckboxWidget(this, font, xpos, ypos, "", kJ1Right);
-  xpos -= (myJoy[kJ1Up]->getWidth() + 5) * 2;
-  ypos += myJoy[kJ1Down]->getHeight() * 2 + 10;
+  xpos -= (myJoy[kJ1Up]->getWidth() + xdiff) * 2;
+  ypos += myJoy[kJ1Down]->getHeight() * 2 + VGAP * 2;
   myJoy[kJ1Fire] = new CheckboxWidget(this, font, xpos, ypos, "Fire", kJ1Fire);
 
-  xpos = 2 * _w / 3;  ypos = y;
+  xpos = 2 * _w / 3 + fontWidth;  ypos = y;
 
   // Console Select/Reset
-  t = new StaticTextWidget(this, font, xpos, ypos+2,
-    font.getStringWidth("Console:"), fontHeight, "Console:",
-    kTextAlignLeft);
-  xpos -= 10;  ypos += t->getHeight() + 10;
-  myHoldSelect = new CheckboxWidget(this, font, xpos, ypos, "Select");
-  ypos += myHoldSelect->getHeight() + 5;
+  t = new StaticTextWidget(this, font, xpos, ypos+2, "Console");
+  ypos += t->getHeight() + VGAP;
+  myHoldSelect = new CheckboxWidget(this, font, xpos, ypos, GUI::SELECT);
+  ypos += myHoldSelect->getHeight() + VGAP;
   myHoldReset = new CheckboxWidget(this, font, xpos, ypos, "Reset");
 
+  static constexpr std::array<int, 10> TAB_ORDER = {
+    kJ0Up, kJ0Left, kJ0Right, kJ0Down, kJ0Fire,
+    kJ1Up, kJ1Left, kJ1Right, kJ1Down, kJ1Fire
+  };
   for(int i = kJ0Up; i <= kJ1Fire; ++i)
-    wid.push_back(myJoy[i]);
+    wid.push_back(myJoy[TAB_ORDER[i]]);
 
   wid.push_back(myHoldSelect);
   wid.push_back(myHoldReset);
@@ -211,7 +200,7 @@ int GlobalPropsDialog::addHoldWidgets(const GUI::Font& font, int x, int y,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void GlobalPropsDialog::loadConfig()
 {
-  Settings& settings = instance().settings();
+  const Settings& settings = instance().settings();
 
   myBSType->setSelected(settings.getString("bs"), "AUTO");
   myLeftDiff->setSelected(settings.getString("ld"), "DEFAULT");
@@ -281,8 +270,6 @@ void GlobalPropsDialog::setDefaults()
 
   myHoldSelect->setState(false);
   myHoldReset->setState(false);
-
-  _dirty = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -291,14 +278,14 @@ void GlobalPropsDialog::handleCommand(CommandSender* sender, int cmd,
 {
   switch(cmd)
   {
-    case kOKCmd:
+    case GuiObject::kOKCmd:
       saveConfig();
       close();
       // Inform parent to load the ROM
       sendCommand(LauncherDialog::kLoadROMCmd, 0, 0);
       break;
 
-    case kDefaultsCmd:
+    case GuiObject::kDefaultsCmd:
       setDefaults();
       saveConfig();
       break;
@@ -310,6 +297,6 @@ void GlobalPropsDialog::handleCommand(CommandSender* sender, int cmd,
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* GlobalPropsDialog::ourJoyState[10] = {
+const std::array<string, 10> GlobalPropsDialog::ourJoyState = {
   "U", "D", "L", "R", "F", "U", "D", "L", "R", "F"
 };
