@@ -27,8 +27,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: convbdf.c 3239 2015-12-29 19:22:46Z stephena $
- *
  */
 #include <stdarg.h>
 #include <stdio.h>
@@ -769,14 +767,12 @@ int gen_c_source(struct font* pf, char *path)
     "//  SS  SS   tt   ee      ll   ll  aa  aa\n"
     "//   SSSS     ttt  eeeee llll llll  aaaaa\n"
     "//\n"
-    "// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony\n"
+    "// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony\n"
     "// and the Stella Team\n"
     "//\n"
     "// See the file \"License.txt\" for information on usage and redistribution of\n"
     "// this file, and for a DISCLAIMER OF ALL WARRANTIES.\n"
     "//\n"
-    "// $Id: convbdf.c 3239 2015-12-29 19:22:46Z stephena $\n"
-    "//\n"
     "// Generated by src/tools/convbdf on %s.\n"
     "//============================================================================\n"
     "\n"
@@ -803,7 +799,7 @@ int gen_c_source(struct font* pf, char *path)
     "namespace GUI {\n"
     "\n"
     "// Font character bitmap data.\n"
-    "static const uInt16 %s_font_bits[] = {\n"
+    "static const uInt16 %s_font_bits[] = {  // NOLINT : too complicated to convert\n"
   };
 
   ofp = fopen(path, "w");
@@ -940,17 +936,17 @@ int gen_c_source(struct font* pf, char *path)
   if (pf->offset)
     sprintf(obuf, "%s_sysfont_offset,", fontname);
   else
-    sprintf(obuf, "0,  /* no encode table*/");
+    sprintf(obuf, "nullptr,  /* no encode table*/");
 
   if (pf->width)
     sprintf(buf, "%s_sysfont_width,", fontname);
   else
-    sprintf(buf, "0,  /* fixed width*/");
+    sprintf(buf, "nullptr,  /* fixed width*/");
 
   if (pf->bbx)
     sprintf(bbuf, "%s_sysfont_bbx,", fontname);
   else
-    sprintf(bbuf, "0,  /* fixed bbox*/");
+    sprintf(bbuf, "nullptr,  /* fixed bbox*/");
 
   fprintf(ofp,
       "/* Exported structure definition. */\n"
@@ -984,7 +980,7 @@ int gen_c_source(struct font* pf, char *path)
       fontname);
 
   fprintf(ofp, "\n} // End of namespace GUI\n\n#endif\n");
-  fcloise(ofp);
+  fclose(ofp);
 
   return 0;
 }
