@@ -1,35 +1,25 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: FSNodeWINDOWS.cxx 3302 2016-04-02 23:47:46Z stephena $
 //============================================================================
 
 #include <cassert>
 #pragma warning( disable : 4091 )
 #include <shlobj.h>
-
-#ifdef ARRAYSIZE
-  #undef ARRAYSIZE
-#endif
-
 #include <io.h>
 #include <stdio.h>
-#include <windows.h>
-// winnt.h defines ARRAYSIZE, but we want our own one...
-#undef ARRAYSIZE
 
 // F_OK, R_OK and W_OK are not defined under MSVC, so we define them here
 // For more information on the modes used by MSVC, check:
@@ -46,33 +36,9 @@
   #define W_OK 2
 #endif
 
+#include "Windows.hxx"
 #include "FSNodeWINDOWS.hxx"
 
-/**
- * Returns the last component of a given path.
- *
- * Examples:
- *			c:\foo\bar.txt would return "\bar.txt"
- *			c:\foo\bar\    would return "\bar\"
- *
- * @param str Path to obtain the last component from.
- * @return Pointer to the first char of the last component inside str.
- */
-// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-const char* lastPathComponent(const string& str)
-{
-  if(str.empty())
-    return "";
-
-  const char* start = str.c_str();
-  const char* cur = start + str.size() - 2;
-
-  while(cur >= start && *cur != '\\')
-    --cur;
-
-  return cur + 1;
-}
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool FilesystemNodeWINDOWS::exists() const
 {
@@ -102,7 +68,7 @@ void FilesystemNodeWINDOWS::setFlags()
   _displayName = lastPathComponent(_path);
 
   // Check whether it is a directory, and whether the file actually exists
-  DWORD fileAttribs = GetFileAttributes(toUnicode(_path.c_str()));
+  const DWORD fileAttribs = GetFileAttributes(toUnicode(_path.c_str()));
 
   if(fileAttribs == INVALID_FILE_ATTRIBUTES)
   {
@@ -126,18 +92,18 @@ void FilesystemNodeWINDOWS::addFile(AbstractFSList& list, ListMode mode,
                                     const char* base, WIN32_FIND_DATA* find_data)
 {
   FilesystemNodeWINDOWS entry;
-  char* asciiName = toAscii(find_data->cFileName);
-  bool isDirectory, isFile;
+  const char* const asciiName = toAscii(find_data->cFileName);
+  bool isDirectory = false, isFile = false;
 
   // Skip local directory (.) and parent (..)
   if(!strncmp(asciiName, ".", 1) || !strncmp(asciiName, "..", 2))
     return;
 
-  isDirectory = (find_data->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ? true : false);
+  isDirectory = ((find_data->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) ? true : false);
   isFile = !isDirectory;//(find_data->dwFileAttributes & FILE_ATTRIBUTE_NORMAL ? true : false);
 
-  if((isFile && mode == FilesystemNode::kListDirectoriesOnly) ||
-     (isDirectory && mode == FilesystemNode::kListFilesOnly))
+  if((isFile && mode == FilesystemNode::ListMode::DirectoriesOnly) ||
+     (isDirectory && mode == FilesystemNode::ListMode::FilesOnly))
     return;
 
   entry._isDirectory = isDirectory;
@@ -150,14 +116,14 @@ void FilesystemNodeWINDOWS::addFile(AbstractFSList& list, ListMode mode,
   entry._isValid = true;
   entry._isPseudoRoot = false;
 
-  list.emplace_back(new FilesystemNodeWINDOWS(entry));
+  list.emplace_back(make_shared<FilesystemNodeWINDOWS>(entry));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 char* FilesystemNodeWINDOWS::toAscii(TCHAR* str)
 {
 #ifndef UNICODE
-  return (char*)str;
+  return str;
 #else
   static char asciiString[MAX_PATH];
   WideCharToMultiByte(CP_ACP, 0, str, _tcslen(str) + 1, asciiString, sizeof(asciiString), NULL, NULL);
@@ -169,7 +135,7 @@ char* FilesystemNodeWINDOWS::toAscii(TCHAR* str)
 const TCHAR* FilesystemNodeWINDOWS::toUnicode(const char* str)
 {
 #ifndef UNICODE
-  return (const TCHAR *)str;
+  return str;
 #else
   static TCHAR unicodeString[MAX_PATH];
   MultiByteToWideChar(CP_ACP, 0, str, strlen(str) + 1, unicodeString, sizeof(unicodeString) / sizeof(TCHAR));
@@ -180,20 +146,12 @@ const TCHAR* FilesystemNodeWINDOWS::toUnicode(const char* str)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 FilesystemNodeWINDOWS::FilesystemNodeWINDOWS()
 {
-  // Create a virtual root directory for standard Windows system
-  _isDirectory = true;
-  _isFile = false;
-  _isValid = false;
-  _path = "";
-  _isPseudoRoot = true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 FilesystemNodeWINDOWS::FilesystemNodeWINDOWS(const string& p)
+  : _path{p.length() > 0 ? p : "~"}  // Default to home directory
 {
-  // Default to home directory
-  _path = p.length() > 0 ? p : "~";
-
   // Expand '~' to the users 'home' directory
   if(_path[0] == '~')
     _path.replace(0, 1, myHomeFinder.getHomePath());
@@ -219,12 +177,10 @@ string FilesystemNodeWINDOWS::getShortPath() const
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool FilesystemNodeWINDOWS::
-    getChildren(AbstractFSList& myList, ListMode mode, bool hidden) const
+    getChildren(AbstractFSList& myList, ListMode mode) const
 {
   assert(_isDirectory);
 
-  //TODO: honor the hidden flag
-
   if(_isPseudoRoot)
   {
     // Drives enumeration
@@ -235,7 +191,7 @@ bool FilesystemNodeWINDOWS::
         current_drive += _tcslen(current_drive) + 1)
     {
       FilesystemNodeWINDOWS entry;
-      char drive_name[2];
+      char drive_name[2] = { 0, 0 };
 
       drive_name[0] = toAscii(current_drive)[0];
       drive_name[1] = '\0';
@@ -245,7 +201,7 @@ bool FilesystemNodeWINDOWS::
       entry._isValid = true;
       entry._isPseudoRoot = false;
       entry._path = toAscii(current_drive);
-      myList.emplace_back(new FilesystemNodeWINDOWS(entry));
+      myList.emplace_back(make_shared<FilesystemNodeWINDOWS>(entry));
     }
   }
   else
@@ -253,12 +209,11 @@ bool FilesystemNodeWINDOWS::
     // Files enumeration
     WIN32_FIND_DATA desc;
     HANDLE handle;
-    char searchPath[MAX_PATH + 10];
 
-    sprintf(searchPath, "%s*", _path.c_str());
-
-    handle = FindFirstFile(toUnicode(searchPath), &desc);
+    ostringstream searchPath;
+    searchPath << _path << "*";
 
+    handle = FindFirstFile(searchPath.str().c_str(), &desc);
     if(handle == INVALID_HANDLE_VALUE)
       return false;
 
@@ -273,6 +228,13 @@ bool FilesystemNodeWINDOWS::
   return true;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+size_t FilesystemNodeWINDOWS::getSize() const
+{
+  struct _stat st;
+  return _stat(_path.c_str(), &st) == 0 ? st.st_size : 0;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 bool FilesystemNodeWINDOWS::makeDir()
 {
@@ -298,24 +260,18 @@ bool FilesystemNodeWINDOWS::rename(const string& newfile)
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-AbstractFSNode* FilesystemNodeWINDOWS::getParent() const
+AbstractFSNodePtr FilesystemNodeWINDOWS::getParent() const
 {
-  if(!_isValid || _isPseudoRoot)
+  if(_isPseudoRoot)
     return nullptr;
 
-  FilesystemNodeWINDOWS* p = new FilesystemNodeWINDOWS();
   if(_path.size() > 3)
   {
     const char* start = _path.c_str();
     const char* end = lastPathComponent(_path);
 
-    p->_path = string(start, end - start);
-    p->_isValid = true;
-    p->_isDirectory = true;
-    p->_isFile = false;
-    p->_displayName = lastPathComponent(p->_path);
-    p->_isPseudoRoot = false;
+    return make_shared<FilesystemNodeWINDOWS>(string(start, static_cast<size_t>(end - start)));
   }
-
-  return p;
+  else
+    return make_shared<FilesystemNodeWINDOWS>();
 }
