@@ -1,98 +1,101 @@
 //============================================================================
 //
-//   SSSS    tt          lll  lll       
-//  SS  SS   tt           ll   ll        
-//  SS     tttttt  eeee   ll   ll   aaaa 
+//   SSSS    tt          lll  lll
+//  SS  SS   tt           ll   ll
+//  SS     tttttt  eeee   ll   ll   aaaa
 //   SSSS    tt   ee  ee  ll   ll      aa
 //      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2016 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2022 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
-//
-// $Id: YaccParser.cxx 3302 2016-04-02 23:47:46Z stephena $
 //============================================================================
 
-//#include "YaccParser.hxx"
-
-//#ifdef __cplusplus
-//extern "C" {
-//#endif
-
 #include "Base.hxx"
-#include "Expression.hxx"
-#include "CartDebug.hxx"
-#include "CpuDebug.hxx"
-#include "TIADebug.hxx"
-
 #include "DebuggerExpressions.hxx"
 
+#include "YaccParser.hxx"
+
 namespace YaccParser {
-#include <stdio.h>
-#include <ctype.h>
+#include <cstdio>
+#include <cctype>
 
 #include "y.tab.h"
-YYSTYPE result;
-string errMsg;
-#include "y.tab.c"
+static YYSTYPE result;
+static string errMsg;
 
-const string& errorMessage() 
+void yyerror(const char* e);
+
+#ifdef __clang__
+  #pragma clang diagnostic push
+  #pragma clang diagnostic ignored "-Wold-style-cast"
+  #pragma clang diagnostic ignored "-Wimplicit-fallthrough"
+  #pragma clang diagnostic ignored "-Wmissing-variable-declarations"
+  #include "y.tab.c"
+  #pragma clang diagnostic pop
+#else
+  #include "y.tab.c"
+#endif
+
+enum class State {
+  DEFAULT,
+  IDENTIFIER,
+  OPERATOR,
+  SPACE
+};
+
+static State state = State::DEFAULT;
+static const char *input, *c;
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+const string& errorMessage()
 {
   return errMsg;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Expression* getResult()
 {
-  lastExp = 0;
+  lastExp = nullptr;
   return result.exp;
 }
 
-const char *input, *c;
-
-enum {
-  ST_DEFAULT,
-  ST_IDENTIFIER,
-  ST_OPERATOR,
-  ST_SPACE
-};
-
-int state = ST_DEFAULT;
-
-//extern int yylval; // bison provides this
-
-void setInput(const char *in)
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+void setInput(const string& in)
 {
-  input = c = in;
-  state = ST_DEFAULT;
+  input = c = in.c_str();
+  state = State::DEFAULT;
 }
 
-int parse(const char *in)
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+int parse(const string& in)
 {
-  lastExp = 0;
+  lastExp = nullptr;
   errMsg = "(no error)";
   setInput(in);
   return yyparse();
 }
 
-/* hand-rolled lexer. Hopefully faster than flex... */
-inline bool is_base_prefix(char x)
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// hand-rolled lexer. Hopefully faster than flex...
+inline constexpr bool is_base_prefix(char x)
 {
   return ( (x=='\\' || x=='$' || x=='#') );
 }
 
-inline bool is_identifier(char x)
+inline constexpr bool is_identifier(char x)
 {
-  return ( (x>='0' && x<='9') || 
+  return ( (x>='0' && x<='9') ||
            (x>='a' && x<='z') ||
            (x>='A' && x<='Z') ||
             x=='.' || x=='_'  );
 }
 
-inline bool is_operator(char x)
+inline constexpr bool is_operator(char x)
 {
   return ( (x=='+' || x=='-' || x=='*' ||
             x=='/' || x=='<' || x=='>' ||
@@ -102,27 +105,29 @@ inline bool is_operator(char x)
             x=='[' || x==']' ) );
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // const_to_int converts a string into a number, in either the
 // current base, or (if there's a base override) the selected base.
 // Returns -1 on error, since negative numbers are the parser's
 // responsibility, not the lexer's
-int const_to_int(char* ch) {
+int const_to_int(char* ch)
+{
   // what base is the input in?
-  Common::Base::Format format = Common::Base::format();
+  Common::Base::Fmt format = Common::Base::format();
 
   switch(*ch) {
     case '\\':
-      format = Common::Base::F_2;
+      format = Common::Base::Fmt::_2;
       ch++;
       break;
 
     case '#':
-      format = Common::Base::F_10;
+      format = Common::Base::Fmt::_10;
       ch++;
       break;
 
     case '$':
-      format = Common::Base::F_16;
+      format = Common::Base::Fmt::_16;
       ch++;
       break;
 
@@ -132,7 +137,7 @@ int const_to_int(char* ch) {
 
   int ret = 0;
   switch(format) {
-    case Common::Base::F_2:
+    case Common::Base::Fmt::_2:
       while(*ch) {
         if(*ch != '0' && *ch != '1')
           return -1;
@@ -142,7 +147,7 @@ int const_to_int(char* ch) {
       }
       return ret;
 
-    case Common::Base::F_10:
+    case Common::Base::Fmt::_10:
       while(*ch) {
         if(!isdigit(*ch))
           return -1;
@@ -152,7 +157,7 @@ int const_to_int(char* ch) {
       }
       return ret;
 
-    case Common::Base::F_16:
+    case Common::Base::Fmt::_16:
       while(*ch) { // FIXME: error check!
         if(!isxdigit(*ch))
           return -1;
@@ -165,11 +170,31 @@ int const_to_int(char* ch) {
       return ret;
 
     default:
-      fprintf(stderr, "INVALID BASE in lexer!");
+      cerr << "INVALID BASE in lexer!" << endl;
       return 0;
   }
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// special methods that get Cart RAM/ROM internal state
+CartMethod getCartSpecial(char* ch)
+{
+  if(BSPF::equalsIgnoreCase(ch, "_bank"))
+    return &CartDebug::getPCBank;
+
+  else if(BSPF::equalsIgnoreCase(ch, "__lastBaseRead"))
+    return &CartDebug::lastReadBaseAddress;
+  else if(BSPF::equalsIgnoreCase(ch, "__lastBaseWrite"))
+    return &CartDebug::lastWriteBaseAddress;
+  else if(BSPF::equalsIgnoreCase(ch, "__lastRead"))
+    return &CartDebug::lastReadAddress;
+  else if(BSPF::equalsIgnoreCase(ch, "__lastWrite"))
+    return &CartDebug::lastWriteAddress;
+  else
+    return nullptr;
+}
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // special methods that get e.g. CPU registers
 CpuMethod getCpuSpecial(char* ch)
 {
@@ -197,73 +222,95 @@ CpuMethod getCpuSpecial(char* ch)
     return &CpuDebug::i;
   else if(BSPF::equalsIgnoreCase(ch, "b"))
     return &CpuDebug::b;
+  else if(BSPF::equalsIgnoreCase(ch, "_iCycles"))
+    return &CpuDebug::icycles;
   else
-    return 0;
+    return nullptr;
 }
 
-// special methods that get Cart RAM/ROM internal state
-CartMethod getCartSpecial(char* ch)
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+// special methods that get RIOT internal state
+RiotMethod getRiotSpecial(char* ch)
 {
-  if(BSPF::equalsIgnoreCase(ch, "_bank"))
-    return &CartDebug::getBank;
-  else if(BSPF::equalsIgnoreCase(ch, "_rwport"))
-    return &CartDebug::readFromWritePort;
+  if(BSPF::equalsIgnoreCase(ch, "_timWrapRead"))
+    return &RiotDebug::timWrappedOnRead;
+  else if(BSPF::equalsIgnoreCase(ch, "_timWrapWrite"))
+    return &RiotDebug::timWrappedOnWrite;
+  else if(BSPF::equalsIgnoreCase(ch, "_fTimReadCycles"))
+    return &RiotDebug::timReadCycles;
+  else if(BSPF::equalsIgnoreCase(ch, "_inTim"))
+    return &RiotDebug::intimAsInt;
+  else if(BSPF::equalsIgnoreCase(ch, "_timInt"))
+    return &RiotDebug::timintAsInt;
   else
-    return 0;
+    return nullptr;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 // special methods that get TIA internal state
 TiaMethod getTiaSpecial(char* ch)
 {
   if(BSPF::equalsIgnoreCase(ch, "_scan"))
     return &TIADebug::scanlines;
-  else if(BSPF::equalsIgnoreCase(ch, "_fcount"))
+  else if(BSPF::equalsIgnoreCase(ch, "_scanEnd"))
+    return &TIADebug::scanlinesLastFrame;
+  else if(BSPF::equalsIgnoreCase(ch, "_sCycles"))
+    return &TIADebug::cyclesThisLine;
+  else if(BSPF::equalsIgnoreCase(ch, "_fCount"))
     return &TIADebug::frameCount;
-  else if(BSPF::equalsIgnoreCase(ch, "_cclocks"))
+  else if(BSPF::equalsIgnoreCase(ch, "_fCycles"))
+    return &TIADebug::frameCycles;
+  else if(BSPF::equalsIgnoreCase(ch, "_fWsyncCycles"))
+    return &TIADebug::frameWsyncCycles;
+  else if(BSPF::equalsIgnoreCase(ch, "_cyclesLo"))
+    return &TIADebug::cyclesLo;
+  else if(BSPF::equalsIgnoreCase(ch, "_cyclesHi"))
+    return &TIADebug::cyclesHi;
+  else if(BSPF::equalsIgnoreCase(ch, "_cClocks"))
     return &TIADebug::clocksThisLine;
-  else if(BSPF::equalsIgnoreCase(ch, "_vsync"))
+  else if(BSPF::equalsIgnoreCase(ch, "_vSync"))
     return &TIADebug::vsyncAsInt;
-  else if(BSPF::equalsIgnoreCase(ch, "_vblank"))
+  else if(BSPF::equalsIgnoreCase(ch, "_vBlank"))
     return &TIADebug::vblankAsInt;
   else
-    return 0;
+    return nullptr;
 }
 
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 int yylex() {
-  static char idbuf[255];
+  static char idbuf[255];  // NOLINT  (will be rewritten soon)
   char o, p;
   yylval.val = 0;
   while(*c != '\0') {
-    //fprintf(stderr, "looking at %c, state %d\n", *c, state);
     switch(state) {
-      case ST_SPACE:
+      case State::SPACE:
         yylval.val = 0;
         if(isspace(*c)) {
           c++;
         } else if(is_identifier(*c) || is_base_prefix(*c)) {
-          state = ST_IDENTIFIER;
+          state = State::IDENTIFIER;
         } else if(is_operator(*c)) {
-          state = ST_OPERATOR;
+          state = State::OPERATOR;
         } else {
-          state = ST_DEFAULT;
+          state = State::DEFAULT;
         }
 
         break;
 
-      case ST_IDENTIFIER:
+      case State::IDENTIFIER:
         {
           CartMethod cartMeth;
           CpuMethod  cpuMeth;
+          RiotMethod riotMeth;
           TiaMethod  tiaMeth;
 
           char *bufp = idbuf;
           *bufp++ = *c++; // might be a base prefix
           while(is_identifier(*c)) { // may NOT be base prefixes
             *bufp++ = *c++;
-            //fprintf(stderr, "yylval==%d, *c==%c\n", yylval, *c);
           }
           *bufp = '\0';
-          state = ST_DEFAULT;
+          state = State::DEFAULT;
 
           // Note: specials (like "a" for accumulator) have priority over
           // numbers. So "a" always means accumulator, not hex 0xa. User
@@ -283,6 +330,9 @@ int yylex() {
           } else if( (cartMeth = getCartSpecial(idbuf)) ) {
             yylval.cartMethod = cartMeth;
             return CART_METHOD;
+          } else if( (riotMeth = getRiotSpecial(idbuf)) ) {
+            yylval.riotMethod = riotMeth;
+            return RIOT_METHOD;
           } else if( (tiaMeth = getTiaSpecial(idbuf)) ) {
             yylval.tiaMethod = tiaMeth;
             return TIA_METHOD;
@@ -298,19 +348,18 @@ int yylex() {
           }
         }
 
-      case ST_OPERATOR:
+      case State::OPERATOR:
         o = *c++;
         if(!*c) return o;
         if(isspace(*c)) {
-          state = ST_SPACE;
+          state = State::SPACE;
           return o;
         } else if(is_identifier(*c) || is_base_prefix(*c)) {
-          state = ST_IDENTIFIER;
+          state = State::IDENTIFIER;
           return o;
         } else {
-          state = ST_DEFAULT;
+          state = State::DEFAULT;
           p = *c++;
-          //fprintf(stderr, "o==%c, p==%c\n", o, p);
           if(o == '>' && p == '=')
             return GTE;
           else if(o == '<' && p == '=')
@@ -334,15 +383,14 @@ int yylex() {
         }
         // break;  Never executed
 
-      case ST_DEFAULT:
-      default:
+      case State::DEFAULT:
         yylval.val = 0;
         if(isspace(*c)) {
-          state = ST_SPACE;
+          state = State::SPACE;
         } else if(is_identifier(*c) || is_base_prefix(*c)) {
-          state = ST_IDENTIFIER;
+          state = State::IDENTIFIER;
         } else if(is_operator(*c)) {
-          state = ST_OPERATOR;
+          state = State::OPERATOR;
         } else {
           yylval.val = *c++;
           return yylval.val;
@@ -351,24 +399,7 @@ int yylex() {
     }
   }
 
-  //fprintf(stderr, "end of input\n");
   return 0; // hit NUL, end of input.
 }
 
-
-#if 0
-int main(int argc, char **argv) {
-  int l;
-
-  set_input(argv[1]);
-  while( (l = yylex()) != 0 )
-    printf("ret %d, %d\n", l, yylval);
-
-  printf("%d\n", yylval);
-}
-#endif
-}
-
-//#ifdef __cplusplus
-//}
-//#endif
+} // namespace YaccParser
